const express = require('express');
const router = express.Router();
const TeslaService = require('../tesla/tesla.service').TeslaService;
const TeslaMqttClient = require('../tesla/tesla.mqtt-client');

router.get('/vehicle', async (req, res) => {
    try {
        // Vérifier si des données MQTT fraîches sont disponibles (moins de 60s)
        const lastMqttUpdate = TeslaMqttClient.getVehicleData('1'); // Assumer ID '1' comme véhicule principal
        const freshMqttData = TeslaMqttClient.isDataFresh(60000);  // 60 secondes
        
        if (lastMqttUpdate && freshMqttData) {
            console.log("Utilisation des données MQTT pour /vehicle");
            
            // Convertir au format attendu par le front-end
            const formattedData = {
                response: {
                    ...lastMqttUpdate,
                    _source: 'mqtt',
                    _timestamp: Date.now()
                }
            };
            return res.json(formattedData);
        }
        
        // Sinon, demander un rafraîchissement et utiliser les données existantes
        await TeslaService.getImmediateVehicleData();
        const data = TeslaService.getLastReceiveData();
        res.json({ response: data });
    } catch (error) {
        res.status(500).json({ error: error.message });
    }
});

// Autres routes Tesla...
router.post('/wake', async (req, res) => {
    try {
        await TeslaService.getImmediateVehicleData();
        res.json({ success: true });
    } catch (error) {
        res.status(500).json({ error: error.message });
    }
});

// Modifier la route /status pour utiliser le bon nom de variable

router.get('/status', (req, res) => {
    // Cache pour les données valides
    if (!router.lastValidTeslaData) router.lastValidTeslaData = null;
    
    // Récupérer les données MQTT
    const mqttData = TeslaMqttClient.getVehicleData('1');
    
    // Récupérer les données API
    const apiData = TeslaService.getLastReceiveData(); // Correction ici: TeslaService au lieu de tesla_service_1.TeslaService
    
    // Variables pour la décision source de données
    let bestData;
    let dataSource;
    
    // Décider quelle source utiliser (priorité à MQTT si valide)
    if (mqttData?.charge_state?.battery_level > 0) {
        bestData = mqttData;
        dataSource = 'mqtt';
        console.log("API STATUS: Utilisation données MQTT");
    } 
    else if (apiData?.charge_state?.battery_level > 0) {
        bestData = apiData;
        dataSource = 'api';
        console.log("API STATUS: Utilisation données API");
    }
    else if (router.lastValidTeslaData && router.lastValidTeslaData.battery_level > 0) {
        console.log("API STATUS: Utilisation données cache");
        return res.json(router.lastValidTeslaData);
    }
    else {
        bestData = { charge_state: {}, climate_state: {} };
        dataSource = 'unknown';
        console.log("API STATUS: Aucune donnée valide");
    }
    
    // Formater la réponse
    const response = {
        battery_level: bestData.charge_state?.battery_level || 0,
        charging_state: bestData.charge_state?.charging_state || "Unknown",
        range_km: bestData.charge_state?.est_battery_range_km || 0,
        inside_temp: bestData.climate_state?.inside_temp || 0,
        outside_temp: bestData.climate_state?.outside_temp || 0,
        is_climate_on: bestData.climate_state?.is_climate_on || false,
        is_preconditioning: bestData.climate_state?.is_preconditioning || false,
        data_source: dataSource,
        timestamp: Date.now(),
        display_name: bestData.display_name || "Tesla",
        chargingMode: TeslaService.chargingMode // Correction ici aussi
    };
    
    // Ne mettre en cache que si les données sont valides
    if (response.battery_level > 0) {
        router.lastValidTeslaData = response;
    }
    
    res.json(response);
});

// Utiliser router.get au lieu de app.get
router.get('/compare', async (req, res) => {
    try {
      // Exécuter la comparaison
      TeslaService.compareDataSources();
      
      // Envoyer une réponse simple
      res.json({ 
        status: 'success', 
        message: 'Comparaison effectuée, vérifiez les logs du serveur',
        api_data_timestamp: TeslaService.last_receive_data?._timestamp || "N/A",
        mqtt_data_timestamp: TeslaMqttClient.lastUpdateTime || "N/A"
      });
    } catch (error) {
      console.error('Erreur lors de la comparaison:', error);
      res.status(500).json({ status: 'error', message: error.message });
    }
});


module.exports = router;