const fs = require('fs');
const path = require('path');

class TeslaChargingLogger {
    static logsDir = path.join(__dirname, '../logs');
    static initialized = false;
    static retentionDays = 14; // Nombre de jours à conserver

    /**
     * Initialise le système de journalisation avec rotation
     */
    static init() {
        if (this.initialized) return;

        // Créer le dossier logs s'il n'existe pas
        if (!fs.existsSync(this.logsDir)) {
            fs.mkdirSync(this.logsDir, { recursive: true });
        }
        
        // Nettoyer les anciens logs au démarrage
        this.cleanupOldLogs();

        // Ajouter une ligne d'en-tête dans le log du jour
        this.writeToLog('===== DÉMARRAGE DE LA JOURNALISATION DE CHARGE TESLA =====');
        this.initialized = true;
    }

    /**
     * Obtient le chemin du fichier de log pour la date actuelle
     */
    static getCurrentLogFilePath() {
        const now = new Date();
        const dateStr = `${now.getFullYear()}-${String(now.getMonth() + 1).padStart(2, '0')}-${String(now.getDate()).padStart(2, '0')}`;
        return path.join(this.logsDir, `tesla-charging-${dateStr}.log`);
    }

    /**
     * Nettoie les fichiers de log plus anciens que la période de rétention
     */
    static cleanupOldLogs() {
        try {
            // Calcul de la date limite de conservation
            const cutoffDate = new Date();
            cutoffDate.setDate(cutoffDate.getDate() - this.retentionDays);
            
            // Lister tous les fichiers de log
            const files = fs.readdirSync(this.logsDir);
            
            let deletedCount = 0;
            files.forEach(file => {
                if (!file.startsWith('tesla-charging-') || !file.endsWith('.log')) return;
                
                // Extraire la date du nom de fichier (format: tesla-charging-YYYY-MM-DD.log)
                const dateMatch = file.match(/tesla-charging-(\d{4}-\d{2}-\d{2})\.log/);
                if (!dateMatch) return;
                
                const fileDate = new Date(dateMatch[1]);
                
                // Supprimer si plus ancien que la période de rétention
                if (fileDate < cutoffDate) {
                    fs.unlinkSync(path.join(this.logsDir, file));
                    deletedCount++;
                }
            });
            
            if (deletedCount > 0) {
                console.log(`TeslaChargingLogger: ${deletedCount} fichiers de log obsolètes supprimés`);
            }
        } catch (error) {
            console.error('Erreur lors du nettoyage des anciens logs:', error);
        }
    }

    /**
     * Écrit un message dans le fichier journal du jour
     */
    static writeToLog(message) {
        try {
            // Création d'un timestamp au format français
            const now = new Date();
            const timestamp = now.toLocaleString('fr-FR', { 
                timeZone: 'Europe/Paris',
                year: 'numeric',
                month: '2-digit',
                day: '2-digit',
                hour: '2-digit',
                minute: '2-digit',
                second: '2-digit'
            }).replace(',', '');
            
            // Format ISO pour compatibilité mais avec l'heure française
            const isoTimestamp = `[${now.getFullYear()}-${String(now.getMonth() + 1).padStart(2, '0')}-${String(now.getDate()).padStart(2, '0')}T${timestamp.split(' ')[1]}Z]`;
            
            // Obtention du chemin du fichier log pour aujourd'hui
            const todaysLogFile = this.getCurrentLogFilePath();
            
            const logMessage = `${isoTimestamp} ${message}`;
            fs.appendFileSync(todaysLogFile, logMessage + '\n');
        } catch (error) {
            console.error('Erreur lors de l\'écriture dans le fichier journal:', error);
        }
    }

    /**
     * Journalise un événement de vérification d'adaptation de charge
     */
    static logChargeCheck(chargingMode, isAtHome, vehicleState, adaptChargeFrequency) {
        this.init();
        this.writeToLog(`CHECK CHARGE: Mode=${chargingMode}, AtHome=${isAtHome}, VehicleState=${vehicleState}, Frequency=${adaptChargeFrequency}s`);
    }

    /**
     * Journalise une décision de maintien d'activité
     */
    static logKeepActive(nextPollTime) {
        this.init();
        this.writeToLog(`KEEP ACTIVE: Prochaine action dans ${nextPollTime/1000}s`);
    }

    /**
     * Journalise un forçage de réveil pendant la charge
     */
    static logForceWakeup() {
        this.init();
        this.writeToLog(`FORCE WAKEUP: Véhicule inactif pendant la charge, forçage du réveil`);
    }

    /**
     * Journalise une commande de maintien d'activité
     */
    static logKeepActiveCommand(currentAmps) {
        this.init();
        this.writeToLog(`KEEP ACTIVE COMMAND: Envoi commande avec même ampérage (${currentAmps}A)`);
    }

    /**
     * Journalise un calcul d'adaptation de puissance
     */
    static logPowerCalculation(overProduction, currentAmps, chargeOffset, volts, phases, ampsDelta, newAmps, minAmps, maxAmps) {
        this.init();
        this.writeToLog(`POWER CALC: Production=${overProduction}W, Offset=${chargeOffset}W, Tension=${volts}V, Phases=${phases}`);
        this.writeToLog(`POWER CALC: AmpsCourant=${currentAmps}A, DeltaAmps=${ampsDelta}A, NouveauxAmps=${newAmps}A (min=${minAmps}A, max=${maxAmps}A)`);
    }

    /**
     * Journalise une commande d'ampérage
     */
    static logSetAmps(newAmps, oldAmps) {
        this.init();
        this.writeToLog(`SET AMPS: Modification ampérage de ${oldAmps}A à ${newAmps}A`);
    }

    /**
     * Journalise un appel API et sa réponse
     */
    static logApiCall(url, method, response) {
        this.init();
        this.writeToLog(`API CALL: ${method} ${url}`);
        if (response) {
            this.writeToLog(`API RESPONSE: Status ${response.status}, Data: ${JSON.stringify(response.data).substring(0, 200)}...`);
        }
    }

    /**
     * Journal d'erreur sécurisé pour les appels API
     * @param {string} endpoint - Point d'accès API
     * @param {string} method - Méthode HTTP
     * @param {Object} details - Détails d'erreur
     */
    static logApiError(endpoint, method, details) {
        try {
            this.init();
            this.writeToLog(`API ERROR: ${method} ${endpoint}`);
          
            // CORRECTION: Gestion sécurisée de tous les cas possibles
            let errorMessage = "Détails non disponibles";
          
            if (details) {
                // Cas 1: details.message existe et est une chaîne
                if (details.message && typeof details.message === 'string') {
                    errorMessage = details.message;  // Utiliser le message complet sans substring
                } 
                // Cas 2: details.message existe mais n'est pas une chaîne
                else if (details.message) {
                    errorMessage = String(details.message);
                } 
                // Cas 3: details est un objet sans propriété message
                else if (typeof details === 'object') {
                    try {
                        errorMessage = JSON.stringify(details).slice(0, 100) + "...";
                    } catch (e) {
                        errorMessage = "Objet non sérialisable";
                    }
                }
            }
          
            this.writeToLog(`API ERROR: ${errorMessage}`);
          
            // Stocker l'erreur pour référence
            if (typeof TeslaService !== 'undefined') {
                TeslaService.lastLoggedError = errorMessage;
            }
        } catch (e) {
            console.error("Erreur pendant le logging:", e);
            this.writeToLog(`API ERROR LOGGING FAILED: ${e.message}`);
        }
    }

    /**
     * Journalise les événements du cycle principal
     */
    static logMainLoop(step, mode, nextActionTime) {
        this.init();
        const now = new Date().getTime();
        const waitTime = nextActionTime - now;
        const stepNames = {
            0: "CONNECT",
            10: "GET_VEHICLES",
            20: "GET_ONLINE",
            30: "WAKE_UP", 
            40: "GET_DATA"
        };
        const modeNames = {
            0: "INIT",
            10: "GET_DATA",
            11: "GET_DATA_NOW",
            20: "WAIT_ONLINE",
            30: "GET_CONTINUS_DATA",
            40: "REFRESH_TOKEN"
        };
        
        this.writeToLog(`MAIN LOOP: Étape=${stepNames[step] || step}, Mode=${modeNames[mode] || mode}, Attente=${Math.round(waitTime/1000)}s`);
    }

    /**
     * Programme un nettoyage périodique des logs (à appeler une fois au démarrage)
     */
    static schedulePeriodicCleanup() {
        // Nettoyer une fois par jour à minuit
        const scheduleNextCleanup = () => {
            const now = new Date();
            const tonight = new Date(
                now.getFullYear(),
                now.getMonth(),
                now.getDate() + 1, // demain
                0, 0, 0 // minuit
            );
            
            const msUntilMidnight = tonight.getTime() - now.getTime();
            
            setTimeout(() => {
                this.cleanupOldLogs();
                scheduleNextCleanup(); // Planifier le prochain nettoyage
            }, msUntilMidnight);
        };
        
        scheduleNextCleanup();
    }
}

module.exports = TeslaChargingLogger;