/**
 * Module de configuration du proxy local Tesla
 * Centralise les URLs et la gestion des tokens
 */

const axios = require('axios');
const fs = require('fs');
const path = require('path');
const https = require('https');
const { ConfigurationService } = require('../services/configuration.service');
const TeslaChargingLogger = require('./tesla.charging-logger');

class TeslaProxy {
  // Configuration par défaut
  static lastRequestTime = 0;
  static tokenRefreshInterval = null;
  static requestsInProgress = new Map();
  static MIN_REQUEST_INTERVAL = 3000; // 3 secondes entre requêtes
  static CONFIG = {
    // État du service
    enabled: false,
    ready: false,
    
    // URLs et points d'accès
    proxy_url: 'localhost',
    proxy_port: 4443,
    base_url: '',
    refresh_token_url: '',
    api_path: '',
    
    // Authentification
    client_id: '',
    client_secret: '',
    cacert: '',
    
    // Tokens
    auth: {
      accessToken: null,
      refreshToken: null, 
      expiresAt: 0,
      tokenType: 'Bearer'
    },
    
    // Cache de token
    use_cached_token: false,
    token_cache_file: 'tesla_tokens.json',
    
    // Options diverses
    timeout: 30000,
    retry_delay: 5000,
    max_retries: 3
  };

/**
 * Initialise le proxy avec la configuration
 */
static init() {
  try {
    // Récupérer la configuration du proxy
    const config = ConfigurationService.getConfiguration();
    
    // Si la section proxy n'existe pas ou n'est pas activée
    if (!config.teslaproxy || !config.teslaproxy.enabled) {
      TeslaChargingLogger.writeToLog("PROXY: Non activé dans la configuration");
      return false;
    }
    
    // Charger les paramètres de configuration
    this.CONFIG.enabled = config.teslaproxy.enabled;
    this.CONFIG.proxy_url = config.teslaproxy.proxy_url || 'localhost';
    this.CONFIG.proxy_port = config.teslaproxy.proxy_port || 4443;
    this.CONFIG.client_id = config.teslaproxy.client_id;
    this.CONFIG.client_secret = config.teslaproxy.client_secret;
    this.CONFIG.cacert = config.teslaproxy.cacert;
    this.CONFIG.base_url = config.teslaproxy.base_url;
    this.CONFIG.refresh_token_url = config.teslaproxy.refresh_token_url;
    this.CONFIG.api_path = config.teslaproxy.api_path || '/api/1/vehicles';
    this.CONFIG.use_cached_token = config.teslaproxy.use_cached_token !== false;
    this.CONFIG.token_cache_file = config.teslaproxy.token_cache_file || 'tesla_tokens.json';
    
    // NOUVEAU: Charger tous les tokens depuis config.json
    if (config.tesla) {
      // Charger le refresh token (existant)
      this.CONFIG.auth.refreshToken = config.tesla.token;
      
      // Charger l'access token et ses propriétés s'ils existent
      if (config.tesla.accessToken) {
        this.CONFIG.auth.accessToken = config.tesla.accessToken;
        this.CONFIG.auth.expiresAt = config.tesla.expiresAt || 0;
        this.CONFIG.auth.tokenType = config.tesla.tokenType || 'Bearer';
        this.CONFIG.auth.lastRefreshTimestamp = config.tesla.lastRefreshTimestamp || Date.now();
        
        const now = Date.now();
        const isTokenValid = this.CONFIG.auth.expiresAt > now;
        
        if (isTokenValid) {
          const expiresIn = Math.round((this.CONFIG.auth.expiresAt - now) / 1000);
          TeslaChargingLogger.writeToLog(`🔐 TOKEN: Access token chargé depuis config.json (valide pour ${expiresIn}s)`);
        } else {
          TeslaChargingLogger.writeToLog(`⚠️ TOKEN: Access token expiré dans config.json, rafraîchissement requis`);
        }
      } else {
        TeslaChargingLogger.writeToLog(`ℹ️ TOKEN: Refresh token présent mais pas d'access token dans config.json`);
      }
    } else {
      TeslaChargingLogger.writeToLog(`⚠️ TOKEN: Section tesla manquante dans la configuration`);
    }
    
    // Charger le token depuis le cache uniquement si pas d'access token valide dans config.json
    if (!this.CONFIG.auth.accessToken && this.CONFIG.use_cached_token) {
      TeslaChargingLogger.writeToLog(`ℹ️ TOKEN: Tentative de chargement depuis le cache secondaire`);
      this.loadTokenFromCache();
    }
    
    // Vérifier la cohérence entre config.json et le cache
    if (this.CONFIG.use_cached_token) {
      this.checkTokenConsistency();
    }
    
    TeslaChargingLogger.writeToLog(`PROXY: Initialisation avec proxy sur ${this.CONFIG.proxy_url}:${this.CONFIG.proxy_port}`);
    
    // NOUVEAU: Configurer un timer dédié pour le rafraîchissement périodique des tokens
    if (this.tokenRefreshInterval) {
      clearInterval(this.tokenRefreshInterval);
      TeslaChargingLogger.writeToLog("🔄 TOKEN: Timer existant nettoyé");
    }
    
    // Définir l'intervalle un peu plus court que la période de rafraîchissement standard
    // pour garantir un rafraîchissement préventif
    const refreshIntervalMs = 240000; // 4 minutes (240 secondes)
    
    this.tokenRefreshInterval = setInterval(async () => {
      try {
        TeslaChargingLogger.writeToLog("⏰ TOKEN: Timer de rafraîchissement déclenché");
        await this.ensureAccessToken();
      } catch (error) {
        TeslaChargingLogger.writeToLog(`❌ TOKEN: Erreur dans le timer de rafraîchissement - ${error.message}`);
      }
    }, refreshIntervalMs);
    
    TeslaChargingLogger.writeToLog(`⏱️ TOKEN: Timer de rafraîchissement configuré (intervalle: ${refreshIntervalMs/1000}s)`);
    
    this.CONFIG.ready = true;
    return true;
  } catch (error) {
    console.error("Erreur lors de l'initialisation du proxy Tesla:", error);
    return false;
  }
}

/**
 * Vérifie la cohérence entre les tokens dans config.json et le cache
 * Version corrigée pour garantir la synchronisation complète
 */
static checkTokenConsistency() {
  try {
    const cacheFilePath = path.join(__dirname, '..', 'assets', 'conf', this.CONFIG.token_cache_file);
    const configPath = path.join(__dirname, '..', 'assets', 'conf', 'config.json');
    
    if (!fs.existsSync(cacheFilePath) || !fs.existsSync(configPath)) {
      TeslaChargingLogger.writeToLog(`⚠️ TOKEN: Impossible de vérifier la cohérence - fichiers manquants`);
      return false;
    }
    
    const cacheData = JSON.parse(fs.readFileSync(cacheFilePath, 'utf8'));
    const configData = JSON.parse(fs.readFileSync(configPath, 'utf8'));
    
    if (!configData.tesla || !cacheData.refresh_token) {
      TeslaChargingLogger.writeToLog(`⚠️ TOKEN: Données insuffisantes pour vérifier la cohérence`);
      return false;
    }
    
    // Vérifier si les tokens sont différents
    if (this.CONFIG.auth.refreshToken && cacheData.refresh_token && 
        this.CONFIG.auth.refreshToken !== cacheData.refresh_token) {
      
      TeslaChargingLogger.writeToLog(`❌ TOKEN: Incohérence détectée - refresh token différent`);
      TeslaChargingLogger.writeToLog(`  • Config: ${this.CONFIG.auth.refreshToken.substring(0, 8)}...`);
      TeslaChargingLogger.writeToLog(`  • Cache: ${cacheData.refresh_token.substring(0, 8)}...`);
      
      // Trouver le token le plus récent
      // La stratégie: regarder l'historique des tokens dans le cache
      const history = cacheData.refresh_token_history || [];
      
      // Si le token de config.json n'est pas dans l'historique, le cache est plus récent
      // Si le token du cache n'est pas dans l'historique, config.json est plus récent
      const configTokenInHistory = history.some(item => item.token === this.CONFIG.auth.refreshToken);
      const cacheTokenInHistory = history.some(item => item.token === cacheData.refresh_token);
      
      // Trouver le dernier token de l'historique (le plus récent)
      const lastHistoryToken = history.length > 0 ? history[history.length - 1].token : null;
      
      let syncDirection = '';
      
      if (lastHistoryToken === cacheData.refresh_token && lastHistoryToken !== this.CONFIG.auth.refreshToken) {
        // Le cache contient le token le plus récent
        syncDirection = 'cache-to-config';
        TeslaChargingLogger.writeToLog(`⬆️ TOKEN: Le cache contient un token plus récent, mise à jour de config.json`);
        
        // Mettre à jour config.json et la mémoire
        configData.tesla.token = cacheData.refresh_token;
        configData.tesla.lastRefreshTimestamp = Date.now();
        this.CONFIG.auth.refreshToken = cacheData.refresh_token;
        fs.writeFileSync(configPath, JSON.stringify(configData, null, 2));
      } else {
        // config.json contient le token le plus récent ou n'est pas dans l'historique
        syncDirection = 'config-to-cache';
        TeslaChargingLogger.writeToLog(`⬇️ TOKEN: config.json semble contenir le token le plus récent, mise à jour du cache`);
        
        // Mettre à jour le cache
        cacheData.refresh_token = this.CONFIG.auth.refreshToken;
        fs.writeFileSync(cacheFilePath, JSON.stringify(cacheData, null, 2));
      }
      
      const syncedToken = (syncDirection === 'cache-to-config') ? cacheData.refresh_token : this.CONFIG.auth.refreshToken;
      TeslaChargingLogger.writeToLog(`✅ TOKEN: Synchronisation (${syncDirection}) terminée avec succès - Token: ${syncedToken.substring(0, 8)}...`);
    } else {
      TeslaChargingLogger.writeToLog(`✅ TOKEN: Tokens déjà synchronisés entre config.json et cache`);
    }
    
    // CORRECTION CRITIQUE: S'assurer que le token en mémoire est synchronisé avec config.json
    // Cette étape garantit que this.CONFIG.auth.refreshToken est toujours à jour
    const latestConfig = JSON.parse(fs.readFileSync(configPath, 'utf8'));
    if (latestConfig.tesla && latestConfig.tesla.token) {
      if (this.CONFIG.auth.refreshToken !== latestConfig.tesla.token) {
        TeslaChargingLogger.writeToLog(`🔄 TOKEN: Synchronisation finale - Mise à jour du token en mémoire depuis config.json`);
        this.CONFIG.auth.refreshToken = latestConfig.tesla.token;
      }
    }
    
    // Vérification supplémentaire pour s'assurer que tout est cohérent
    if (cacheData.refresh_token !== this.CONFIG.auth.refreshToken) {
      TeslaChargingLogger.writeToLog(`⚠️ TOKEN: Le cache n'est toujours pas synchronisé avec la mémoire, mise à jour finale`);
      cacheData.refresh_token = this.CONFIG.auth.refreshToken;
      fs.writeFileSync(cacheFilePath, JSON.stringify(cacheData, null, 2));
    }
    
    TeslaChargingLogger.writeToLog(`🔒 TOKEN: Vérification finale - Tous les tokens sont synchronisés`);
    return true;
  } catch (error) {
    TeslaChargingLogger.writeToLog(`❌ TOKEN: Erreur lors de la vérification de cohérence - ${error.message}`);
    console.error("Erreur de vérification de cohérence des tokens:", error);
    return false;
  }
}

  /**
   * Charge les tokens depuis le fichier de cache
   */
  static loadTokenFromCache() {
    try {
      const cacheFilePath = path.join(__dirname, '..', 'assets', 'conf', this.CONFIG.token_cache_file);
      
      if (fs.existsSync(cacheFilePath)) {
        const cacheData = JSON.parse(fs.readFileSync(cacheFilePath, 'utf8'));
        
        if (cacheData.access_token && cacheData.expires_at > Date.now()) {
          this.CONFIG.auth.accessToken = cacheData.access_token;
          this.CONFIG.auth.expiresAt = cacheData.expires_at;
          this.CONFIG.auth.tokenType = cacheData.token_type || 'Bearer';
          
          TeslaChargingLogger.writeToLog("TOKEN: Chargé depuis le cache (valide)");
          return true;
        } else {
          TeslaChargingLogger.writeToLog("TOKEN: Cache expiré ou invalide, nouveau token requis");
        }
      }
    } catch (error) {
      TeslaChargingLogger.writeToLog(`TOKEN: Erreur lors du chargement du cache - ${error.message}`);
    }
    
    return false;
  }
  
/**
 * Sauvegarde les tokens dans le fichier de cache avec historique des refresh tokens
 */
static saveTokenToCache() {
  try {
    const cacheDir = path.join(__dirname, '..', 'assets', 'conf');
    const cacheFilePath = path.join(cacheDir, this.CONFIG.token_cache_file);
    
    // Créer le répertoire s'il n'existe pas
    if (!fs.existsSync(cacheDir)) {
      fs.mkdirSync(cacheDir, { recursive: true });
    }
    
    // Charger le fichier existant s'il existe pour préserver l'historique
    let cacheData = {
      access_token: this.CONFIG.auth.accessToken,
      refresh_token: this.CONFIG.auth.refreshToken,
      expires_at: this.CONFIG.auth.expiresAt,
      token_type: this.CONFIG.auth.tokenType,
      refresh_token_history: []  // Initialiser l'historique s'il n'existe pas
    };
    
    if (fs.existsSync(cacheFilePath)) {
      try {
        const existingData = JSON.parse(fs.readFileSync(cacheFilePath, 'utf8'));
        // Préserver l'historique existant
        if (existingData.refresh_token_history && Array.isArray(existingData.refresh_token_history)) {
          cacheData.refresh_token_history = existingData.refresh_token_history;
        }
      } catch (parseError) {
        TeslaChargingLogger.writeToLog(`⚠️ TOKEN: Erreur de lecture du cache existant - ${parseError.message}`);
      }
    }
    
    // Ajouter le token actuel à l'historique s'il existe et s'il est différent du dernier
    if (this.CONFIG.auth.refreshToken) {
      const now = new Date().toISOString();
      const historyItem = {
        token: this.CONFIG.auth.refreshToken,
        timestamp: now,
        config_status: "N/A"
      };
      
      // Vérifier si le token dans config.json correspond à celui en mémoire
      try {
        const configPath = path.join(__dirname, '..', 'assets', 'conf', 'config.json');
        if (fs.existsSync(configPath)) {
          const configData = JSON.parse(fs.readFileSync(configPath, 'utf8'));
          if (configData.tesla && configData.tesla.token) {
            historyItem.config_status = (configData.tesla.token === this.CONFIG.auth.refreshToken) 
              ? "SYNC" // Token synchronisé avec config.json
              : "DESYNC"; // Token désynchronisé avec config.json
          }
        }
      } catch (configError) {
        historyItem.config_status = `ERROR: ${configError.message}`;
      }
      
      // N'ajouter que si c'est un nouveau token ou si l'état de synchronisation a changé
      const lastItem = cacheData.refresh_token_history[cacheData.refresh_token_history.length - 1];
      if (!lastItem || lastItem.token !== historyItem.token || lastItem.config_status !== historyItem.config_status) {
        cacheData.refresh_token_history.push(historyItem);
        TeslaChargingLogger.writeToLog(`📝 TOKEN: Nouvel historique d'entrée pour refresh token (${historyItem.token.substring(0, 5)}... - ${historyItem.config_status})`);
      }
    }
    
    // Limiter la taille de l'historique à 50 entrées
    if (cacheData.refresh_token_history.length > 50) {
      cacheData.refresh_token_history = cacheData.refresh_token_history.slice(-50);
    }
    
    fs.writeFileSync(cacheFilePath, JSON.stringify(cacheData, null, 2));
    TeslaChargingLogger.writeToLog("✅ TOKEN: Sauvegardé dans le cache avec historique");
    
    return true;
  } catch (error) {
    TeslaChargingLogger.writeToLog(`❌ TOKEN: Erreur lors de la sauvegarde du cache - ${error.message}`);
    return false;
  }
}
  
/**
 * Crée une instance Axios configurée pour le proxy local
 */
static createAxiosInstance(options = {}) {
    // CORRECTION: Désactiver la vérification SSL pour le proxy local
    const httpsAgent = new https.Agent({
      rejectUnauthorized: false  // Cette option est obligatoire pour les certificats auto-signés
    });
    
    const config = {
      timeout: options.shortTimeout ? 10000 : (options.timeout || this.CONFIG.timeout),
      headers: {
        'User-Agent': 'Energy Board Tesla Client',
        'Accept': 'application/json'
      },
      httpsAgent: httpsAgent
    };
    
    // Ajouter le token d'authentification s'il existe
    if (this.CONFIG.auth.accessToken) {
      config.headers['Authorization'] = `${this.CONFIG.auth.tokenType} ${this.CONFIG.auth.accessToken}`;
    }
    
    // Ajouter Content-Type pour les requêtes POST
    if (options.jsonContent) {
      config.headers['Content-Type'] = 'application/json';
    }
    
    return axios.create(config);
  }
  
// Variable de classe pour empêcher les rafraîchissements simultanés
static refreshInProgress = false;
static refreshPromise = null;

/**
 * Rafraîchit le token d'accès si nécessaire
 * Utilise d'abord le refresh token s'il existe, sinon essaie client_credentials
 * Protégé contre les appels simultanés
 */
static async ensureAccessToken() {
  // Si un rafraîchissement est déjà en cours, attendre qu'il se termine
  if (this.refreshInProgress) {
    TeslaChargingLogger.writeToLog(`🔒 TOKEN: Rafraîchissement déjà en cours, attente...`);
    
    // Si nous avons une promesse en cours, attendons son résultat plutôt que de dupliquer le travail
    if (this.refreshPromise) {
      try {
        return await this.refreshPromise;
      } catch (error) {
        TeslaChargingLogger.writeToLog(`⚠️ TOKEN: La tentative précédente a échoué, nouvelle tentative`);
        // Continuer avec une nouvelle tentative
      }
    }
  }
  
  // Initialiser le timestamp du dernier rafraîchissement s'il n'existe pas encore
  if (!this.CONFIG.auth.lastRefreshTimestamp) {
    this.CONFIG.auth.lastRefreshTimestamp = 0;
  }
  
  // Après avoir attendu, vérifier si le token est maintenant valide (un autre processus l'a peut-être rafraîchi)
  const now = Date.now();
  const isTokenValid = this.CONFIG.auth.accessToken && this.CONFIG.auth.expiresAt > now;
  const timeSinceLastRefresh = now - this.CONFIG.auth.lastRefreshTimestamp;
  const REFRESH_INTERVAL = 4 * 60 * 60 * 1000; // 4 heures
  
  // Si le token est valide et qu'il n'a pas besoin d'être rafraîchi préventivement
  if (isTokenValid && timeSinceLastRefresh <= REFRESH_INTERVAL) {
    const nextRefreshIn = REFRESH_INTERVAL - timeSinceLastRefresh;
    TeslaChargingLogger.writeToLog(`✅ TOKEN: Token d'accès valide. Prochain rafraîchissement préventif dans ${Math.round(nextRefreshIn/1000)}s`);
    return true;
  }
  
  // Marquer le début du processus de rafraîchissement
  this.refreshInProgress = true;
  
  // Créer une promesse pour le processus de rafraîchissement que d'autres appels peuvent attendre
  this.refreshPromise = (async () => {
    try {
      // Si le token est valide mais que l'intervalle de rafraîchissement préventif est dépassé
      if (isTokenValid) {
        TeslaChargingLogger.writeToLog(`🔄 TOKEN: PRÉVENTIF: Rafraîchissement préventif du token (dernier refresh il y a ${Math.round(timeSinceLastRefresh/1000)}s > intervalle ${REFRESH_INTERVAL/1000}s)`);
      } else {
        // Log clair sur l'expiration/absence
        if (this.CONFIG.auth.accessToken) {
          const expiredAgo = now - this.CONFIG.auth.expiresAt;
          TeslaChargingLogger.writeToLog(`⌛ TOKEN: Token d'accès expiré depuis ${Math.round(expiredAgo/1000)}s, rafraîchissement nécessaire`);
        } else {
          TeslaChargingLogger.writeToLog(`❌ TOKEN: Token d'accès manquant, rafraîchissement nécessaire`);
        }
      }
      
      // Créer une instance Axios pour la requête
      const instance = axios.create({
        timeout: 10000,
        headers: {
          'Content-Type': 'application/x-www-form-urlencoded',
          'User-Agent': 'TeslaApp/4.20.0'
        }
      });
      
      let response = null;
      
      // Essayer d'abord avec refresh token
      if (this.CONFIG.auth.refreshToken) {
        TeslaChargingLogger.writeToLog("🔑 TOKEN: Tentative de rafraîchissement avec refresh_token");
        
        const refreshTokenData = {
          grant_type: 'refresh_token',
          client_id: this.CONFIG.client_id,
          refresh_token: this.CONFIG.auth.refreshToken
        };
        
        response = await instance.post(
          this.CONFIG.refresh_token_url,
          new URLSearchParams(refreshTokenData).toString()
        );
        
        TeslaChargingLogger.writeToLog(`✅ TOKEN: Rafraîchissement via refresh_token réussi (statut ${response?.status})`);
      }
      
      // Si pas de réponse via refresh token, essayer client credentials
      if (!response && this.CONFIG.client_id && this.CONFIG.client_secret) {
        TeslaChargingLogger.writeToLog("🔑 TOKEN: Tentative d'authentification par client_credentials");
        
        const clientCredentialsData = {
          grant_type: 'client_credentials',
          client_id: this.CONFIG.client_id,
          client_secret: this.CONFIG.client_secret
        };
        
        response = await instance.post(
          this.CONFIG.refresh_token_url,
          new URLSearchParams(clientCredentialsData).toString()
        );
      }
      
      // Traitement de la réponse
      if (response && response.data) {
        TeslaChargingLogger.writeToLog(`🔐 TOKEN: Nouveau token reçu - access_token présent: ${!!response.data.access_token}, refresh_token présent: ${!!response.data.refresh_token}`);
        
        // Mise à jour du token en mémoire
        this.CONFIG.auth.accessToken = response.data.access_token;
        
        // Calculer et stocker l'expiration
        const expiresIn = response.data.expires_in || 7200; // Par défaut 2h
        this.CONFIG.auth.expiresAt = Date.now() + (expiresIn * 1000);
        this.CONFIG.auth.lastRefreshTimestamp = Date.now(); // Enregistrer le timestamp du rafraîchissement
        
        const expiryDate = new Date(this.CONFIG.auth.expiresAt);
        TeslaChargingLogger.writeToLog(`⏱️ TOKEN: Expiration calculée à ${expiryDate.toISOString()} (dans ${expiresIn}s)`);
        
        // MISE À JOUR CENTRALISÉE dans config.json
        try {
            const configPath = path.join(__dirname, '..', 'assets', 'conf', 'config.json');
            TeslaChargingLogger.writeToLog(`💾 TOKEN: Sauvegarde de tous les tokens dans ${configPath}`);
            
            const configData = JSON.parse(fs.readFileSync(configPath, 'utf8'));
            
            if (configData.tesla) {
                // Conserver l'ancien refresh token pour les logs
                let oldRefreshToken = 'non défini';
                let newRefreshToken = 'inchangé';
                
                if (response.data.refresh_token) {
                  // Utiliser la valeur en mémoire comme référence pour l'ancien token
                  oldRefreshToken = this.CONFIG.auth.refreshToken ? this.CONFIG.auth.refreshToken.substring(0, 5) + '...' : 'non défini';
                  newRefreshToken = response.data.refresh_token.substring(0, 5) + '...';
                  
                  // Mettre à jour le refresh token en mémoire
                  this.CONFIG.auth.refreshToken = response.data.refresh_token;
                  // Mettre à jour le refresh token dans config.json
                  configData.tesla.token = response.data.refresh_token;
              }
                
                // NOUVEAU: Stocker l'access token et sa date d'expiration dans config.json
                configData.tesla.accessToken = response.data.access_token;
                configData.tesla.expiresAt = this.CONFIG.auth.expiresAt;
                configData.tesla.tokenType = response.data.token_type || "Bearer";
                
                // Écriture atomique du fichier config
                fs.writeFileSync(configPath, JSON.stringify(configData, null, 2));
                
                if (response.data.refresh_token) {
                    TeslaChargingLogger.writeToLog(`✅ TOKEN: Refresh token sauvegardé (${oldRefreshToken} → ${newRefreshToken})`);
                }
                TeslaChargingLogger.writeToLog(`✅ TOKEN: Access token et date d'expiration sauvegardés dans config.json`);
            } else {
                TeslaChargingLogger.writeToLog(`❌ TOKEN: Section tesla manquante dans config.json`);
            }
        } catch (configError) {
            // Erreur critique - afficher clairement le problème
            TeslaChargingLogger.writeToLog(`❌ TOKEN CRITIQUE: Échec sauvegarde tokens - ${configError.message}`);
            console.error("ERREUR CRITIQUE: Échec de sauvegarde des tokens Tesla:", configError);
        }
        
        // Conserver le cache pour compatibilité (optionnel, peut être supprimé plus tard)
        await this.saveTokenToCache();
        
        return true;
      } else {
        TeslaChargingLogger.writeToLog("❌ TOKEN: Échec de rafraîchissement, aucune réponse valide");
        return false;
      }
    } catch (error) {
      // Gestion améliorée des erreurs
      const errorMessage = error.message || 'Erreur inconnue';
      const statusCode = error.response?.status || 'N/A';
      
      TeslaChargingLogger.writeToLog(`❌ TOKEN: ERREUR: Échec de rafraîchissement - Statut=${statusCode}, Message=${errorMessage}`);
      
      if (error.response?.data) {
        try {
          const errorDataStr = typeof error.response.data === 'string' 
            ? error.response.data 
            : JSON.stringify(error.response.data);
          TeslaChargingLogger.writeToLog(`❌ TOKEN: ERREUR DÉTAILS: ${errorDataStr.substring(0, 300)}`);
        } catch (logError) {
          TeslaChargingLogger.writeToLog(`❌ TOKEN: ERREUR: Impossible de logger les détails - ${logError.message}`);
        }
      }
      
      console.error("Erreur de rafraîchissement du token Tesla:", error);
      return false;
    } finally {
      // Toujours libérer le verrou à la fin du processus
      this.refreshInProgress = false;
    }
  })();
  
  // Attendre la résolution de la promesse et retourner son résultat
  try {
    return await this.refreshPromise;
  } finally {
    // Une fois terminé, nettoyer la promesse
    if (!this.refreshInProgress) {
      this.refreshPromise = null;
    }
  }
}
  
  /**
   * Exécute une requête API avec gestion automatique du token
   */
/**
 * Exécute une requête API avec gestion automatique du token
 * et protection contre les appels multiples simultanés
 */
static async request(method, endpoint, data = null, options = {}) {
  // S'assurer d'avoir un token valide
  if (!this.CONFIG.ready) {
    throw new Error("Le proxy Tesla n'est pas initialisé");
  }
  
  // Créer une clé unique pour cette requête
  const requestKey = `${method.toUpperCase()}:${endpoint}`;
  
  // PROTECTION: Vérifier si une requête identique est déjà en cours
  if (this.requestsInProgress.has(requestKey)) {
    TeslaChargingLogger.writeToLog(`⚠️ API PROTECTION: Requête ${requestKey} déjà en cours, abandon du doublon`);
    throw new Error(`Requête ${method} ${endpoint} déjà en cours de traitement`);
  }
  
  // PROTECTION: Ajouter un délai entre les requêtes pour éviter les erreurs 502
  const now = Date.now();
  const timeSinceLastRequest = now - this.lastRequestTime;
  
  if (timeSinceLastRequest < this.MIN_REQUEST_INTERVAL) {
    const waitTime = this.MIN_REQUEST_INTERVAL - timeSinceLastRequest;
    TeslaChargingLogger.writeToLog(`⏱️ API THROTTLE: Attente de ${waitTime}ms avant nouvel appel API`);
    
    // Attendre le délai requis
    await new Promise(resolve => setTimeout(resolve, waitTime));
  }
  
  // Marquer cette requête comme en cours
  this.requestsInProgress.set(requestKey, now);
  
  try {
    // Construire l'URL complète
    const isFullUrl = endpoint.startsWith('http');
    const url = isFullUrl ? endpoint : `https://${this.CONFIG.proxy_url}:${this.CONFIG.proxy_port}${endpoint}`;
    
    // S'assurer d'avoir un token valide
    await this.ensureAccessToken();
    
    // Log pour tracer les appels coûteux
    TeslaChargingLogger.writeToLog(`💸 API: Appel coûteux ${method.toUpperCase()} vers ${endpoint}`);
    
    // Mettre à jour le timestamp de dernière requête
    this.lastRequestTime = Date.now();
    
    // Créer l'instance avec le token à jour
    const instance = this.createAxiosInstance(options);
    
    let response;
    
    // Exécuter la requête selon la méthode
    if (method.toUpperCase() === 'GET') {
      response = await instance.get(url, { params: data });
    } else if (method.toUpperCase() === 'POST') {
      response = await instance.post(url, data);
    } else if (method.toUpperCase() === 'PUT') {
      response = await instance.put(url, data);
    } else if (method.toUpperCase() === 'DELETE') {
      response = await instance.delete(url, { data });
    } else {
      throw new Error(`Méthode non supportée: ${method}`);
    }
    
    return response;
  } catch (error) {
    // Si erreur d'authentification, essayer de rafraîchir le token et réessayer
    if (error.response && error.response.status === 401) {
      TeslaChargingLogger.writeToLog("API: Erreur d'authentification, tentative de refresh token");
      
      // Forcer le rafraîchissement du token
      this.CONFIG.auth.accessToken = null;
      await this.ensureAccessToken();
      
      // Recréer l'instance et réessayer
      const refreshedInstance = this.createAxiosInstance(options);
      
      if (method.toUpperCase() === 'GET') {
        return await refreshedInstance.get(url, { params: data });
      } else if (method.toUpperCase() === 'POST') {
        return await refreshedInstance.post(url, data);
      } else if (method.toUpperCase() === 'PUT') {
        return await refreshedInstance.put(url, data);
      } else if (method.toUpperCase() === 'DELETE') {
        return await refreshedInstance.delete(url, { data });
      }
    }
    
    // Log des erreurs 502 pour aider au diagnostic
    if (error.response && error.response.status === 502) {
      TeslaChargingLogger.writeToLog(`⚠️ API ERREUR 502: Bad Gateway pour ${requestKey} - Possible limitation d'API Tesla`);
    }
    
    // Si ce n'est pas une erreur 401 ou si le retry a échoué, propager l'erreur
    throw error;
  } finally {
    // IMPORTANT: Toujours libérer le verrou à la fin, même en cas d'erreur
    this.requestsInProgress.delete(requestKey);
    TeslaChargingLogger.writeToLog(`🔓 API: Fin de requête ${requestKey}`);
  }
}
}

module.exports = TeslaProxy;