const TeslaChargingLogger = require('./tesla.charging-logger');
const TeslaMqttClient = require('./tesla.mqtt-client');
const { TeslaControlBLE } = require('./tesla.ble.service');

// Variables pour stocker les dernières mises à jour MQTT
let lastMqttUpdate = null;
let lastMqttUpdateTime = 0;

// Écouter les mises à jour MQTT pour mettre à jour lastMqttUpdate
TeslaMqttClient.addUpdateListener((carId, attribute, value) => {
    // Stocker l'heure de dernière mise à jour
    lastMqttUpdateTime = Date.now();
    
    // Pour les attributs importants, déclencher une mise à jour
    if (['battery_level', 'charging_state', 'state', 'plugged_in'].includes(attribute)) {
        // Récupérer toutes les données actuelles
        lastMqttUpdate = TeslaMqttClient.getVehicleData(carId);
    }
});

"use strict";
/* exported TeslaUtility */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.TeslaService = void 0;
const axios_1 = __importDefault(require("axios"));
const enphase_data_service_1 = require("../enphase/enphase.data.service");
const configuration_service_1 = require("../services/configuration.service");

const pollInterval = 10000; // 10 secondes
let pollingActive = true;

// Fonction de polling qui émet les données via Socket.IO
function startPollingWithSocketEmit() {
  setInterval(async () => {
    if (!pollingActive) return;
    
    try {
      // Utiliser le proxy local avec certificat TLS
      const instance = TeslaService.createAxiosInstance({
        httpsAgent: new https.Agent({
          ca: fs.readFileSync(path.join(__dirname, '../tesla_http_proxy/config/tls-cert.pem'))
        })
      });
      
      const response = await instance.get(`${TeslaProxy.CONFIG.api_path}/${vehicleIdOrVin}/vehicle_data`);
      
      if (response.status === 200 && response.data?.response) {
        // Émettre via Socket.IO
        global.io.emit('tesla:vehicle_update', response.data.response);
        
        // Mettre à jour aussi les données locales
        TeslaService.last_receive_data = response.data.response;
      }
    } catch (error) {
      TeslaChargingLogger.writeToLog(`API ERROR: ${error.message}`);
    }
  }, pollInterval);
}

class TeslaService {
    static TIMEOUTS = {
        WAKE_UP_RETRY: 5000,
        LOCK_RELEASE: 5000,
        STATE_CACHE_TTL: 30000
    };
    
        static refreshInProgress = false;
        static lastRefreshRequest = 0;
        static lastHomeCheckTime = 0;
        static lastKnownAtHome = true;
        static wakeupInProgress = false;
        static wake408Attempts = 0;
        static lastGetVehicleDataCall = 0;
        static chargingOperationInProgress = false;
        static chargingOperationTimeout = null;
        static lastChargingOperation = 0;

        /**
 * Crée une instance Axios configurée pour les API Tesla avec options personnalisables
 * @param {Object} options - Options de personnalisation
 * @param {number} options.timeout - Timeout en ms (défaut: 30000)
 * @param {boolean} options.jsonContent - Ajouter Content-Type JSON (défaut: false) 
 * @param {boolean} options.shortTimeout - Utiliser un timeout court pour les requêtes légères (défaut: false)
 * @returns {AxiosInstance} Instance Axios configurée
 */


/**
 * Récupère les données du véhicule depuis MQTT en utilisant l'ID configuré
 * @returns {Object|null} Les données du véhicule ou null si indisponible
 */
static getMqttVehicleData() {
    try {
        const TeslaMqttClient = require('./tesla.mqtt-client');
        const config = configuration_service_1.ConfigurationService.getConfiguration();
        const carId = config.teslamate?.mqtt?.carId || 1;
        return TeslaMqttClient?.getVehicleData(carId);
    } catch (error) {
        TeslaChargingLogger.writeToLog(`❌ MQTT DATA ERROR: ${error.message}`);
        return null;
    }
}


/**
 * Initialisation des variables d'état
 */
static initializeState() {
    TeslaChargingLogger.writeToLog("🔧 INIT STATE: Démarrage initialisation des variables d'état");
    
    // Variables d'état du cycle
    const previousStep = this.step || 'non défini';
    const previousMode = this.mode || 'non défini';
    
    this.step = this.STEP_CONNECT;
    this.mode = this.MODE_GET_DATA;
    this.previousMode = this.MODE_GET_DATA;
    this.previousStep = this.STEP_CONNECT;
    
    TeslaChargingLogger.writeToLog(`🔧 INIT STATE: État du cycle - Step=${previousStep} -> ${this.step} (CONNECT)`);
    TeslaChargingLogger.writeToLog(`🔧 INIT STATE: Mode - ${previousMode} -> ${this.mode} (GET_DATA)`);
    TeslaChargingLogger.writeToLog(`🔧 INIT STATE: PreviousStep=${this.previousStep}, PreviousMode=${this.previousMode}`);
    
    // Variables pour la charge
    const oldChargingMode = this.chargingMode || 'non défini';
    this.chargingMode = 'normal'; // Valeur par défaut
    
    const oldLastChangeTime = this.lastChangeChargingAmps;
    this.lastChangeChargingAmps = 0;
    
    const oldLastCheckTime = this.lastCheckTime;
    this.lastCheckTime = 0;
    
    TeslaChargingLogger.writeToLog(`🔧 INIT STATE: Mode de charge - ${oldChargingMode} -> ${this.chargingMode}`);
    TeslaChargingLogger.writeToLog(`🔧 INIT STATE: Timestamps charge - lastChange=${oldLastChangeTime || 0} -> 0, lastCheck=${oldLastCheckTime || 0} -> 0`);
    
    // Variables pour le réveil
    const oldWakeupState = this.wakeupInProgress ? 'en cours' : 'inactif';
    this.wakeupInProgress = false;
    
    const oldWakeupTime = this.lastWakeupCall;
    this.lastWakeupCall = 0;
    
    TeslaChargingLogger.writeToLog(`🔧 INIT STATE: État réveil - ${oldWakeupState} -> inactif`);
    TeslaChargingLogger.writeToLog(`🔧 INIT STATE: Dernier réveil - ${oldWakeupTime || 0} -> 0`);
    
    // Données véhicule
    const hadPreviousData = this.last_receive_data && Object.keys(this.last_receive_data).length > 0;
    
    if (!this.last_receive_data) {
        TeslaChargingLogger.writeToLog("🔧 INIT STATE: Création de l'objet de données véhicule (précédemment inexistant)");
        this.last_receive_data = {};
    } else if (hadPreviousData) {
        TeslaChargingLogger.writeToLog("🔧 INIT STATE: Conservation de l'objet de données existant");
        TeslaChargingLogger.writeToLog(`🔧 INIT STATE: Données véhicule - ${Object.keys(this.last_receive_data).length} propriétés présentes`);
        
        // Log des données de charge si présentes
        if (this.last_receive_data.charge_state) {
            TeslaChargingLogger.writeToLog(`🔧 INIT STATE: État charge - Niveau batterie=${this.last_receive_data.charge_state.battery_level || 'N/A'}, État=${this.last_receive_data.charge_state.charging_state || 'N/A'}`);
        }
        
        // Log de l'état du véhicule si présent
        if (this.last_receive_data.vehicle_state) {
            TeslaChargingLogger.writeToLog(`🔧 INIT STATE: État véhicule - État=${this.last_receive_data.vehicle_state.state || 'N/A'}, Verrouillé=${this.last_receive_data.vehicle_state.locked || 'N/A'}`);
        }
    } else {
        TeslaChargingLogger.writeToLog("🔧 INIT STATE: Réinitialisation de l'objet de données véhicule vide");
        this.last_receive_data = {};
    }
    
    // Log de confirmation finale
    TeslaChargingLogger.writeToLog("✅ INIT STATE: Initialisation des variables d'état terminée avec succès");
    
    // Diagnostic
    console.log("TeslaService: Variables d'état initialisées");
}

// Méthode helper pour initialiser le proxy quand nécessaire
static ensureProxyInitialized() {
    const TeslaProxy = require('./tesla.proxy');
    if (!TeslaProxy.CONFIG.ready) {
        TeslaProxy.init();
        return false; // Indique que le proxy vient d'être initialisé
    }
    return true; // Indique que le proxy était déjà initialisé
}

// Méthode helper pour les logs
static logWithContext(level, context, message) {
    const emoji = {
        'INFO': 'ℹ️',
        'ERROR': '❌', 
        'WARN': '⚠️',
        'SUCCESS': '✅'
    }[level] || '🔹';
    
    TeslaChargingLogger.writeToLog(`${emoji} ${context}: ${message}`);
}

/**
 * Démarre le polling des données Tesla avec émission Socket.IO
 */
static startPollingWithSocketEmit() {
    // Configuration du polling
    const fs = require('fs');
    const path = require('path');
    const https = require('https');
    const pollInterval = 60000; // Augmenté à 60 secondes au lieu de 10 secondes
    let pollingActive = true;
    
    TeslaChargingLogger.writeToLog("🚀 POLLING INIT: Démarrage du service de polling Socket.IO");
    console.log(`TeslaService: Démarrage du polling Socket.IO (intervalle=${pollInterval}ms)`);
    
    // Variables pour statistiques
    let pollCount = 0;
    let successCount = 0;
    let errorCount = 0;
    let lastPollTime = 0;
    let lastApiCallTime = 0;
    
    // Créer une intervalle pour le polling
    const intervalId = setInterval(async () => {
        // Log début d'exécution
        const startTime = Date.now();
        pollCount++;
        
        // Vérification Socket.IO
        if (!global.io) {
            TeslaChargingLogger.writeToLog("⚠️ POLLING: Socket.IO non disponible, exécution ignorée");
            return;
        }
        
        // Vérification de l'état actif
        if (!pollingActive) {
            TeslaChargingLogger.writeToLog("⚠️ POLLING: Service inactif, exécution ignorée");
            return;
        }
        
        // Log anti-rebond (ne pas spammer les logs)
        if (startTime - lastPollTime < 9000) { // Log uniquement si exécution prématurée
            TeslaChargingLogger.writeToLog(`⚠️ POLLING: Exécution rapprochée (${Math.round((startTime - lastPollTime)/1000)}s)`);
        }
        lastPollTime = startTime;
        
        TeslaChargingLogger.writeToLog(`🔄 POLLING #${pollCount}: Début cycle de polling Socket.IO`);
        
        try {
            // Récupérer l'ID du véhicule pour l'API
            const vehicleIdOrVin = this.last_receive_data?.vin || this.vehicle_id;
            
            // Log de diagnostique avec contexte
            TeslaChargingLogger.writeToLog(`🔍 POLLING: Utilisation identifiant véhicule: ${vehicleIdOrVin || 'NON DÉFINI'}`);
            
            if (!vehicleIdOrVin) {
                TeslaChargingLogger.writeToLog("❌ POLLING: ID véhicule manquant, polling impossible");
                errorCount++;
                return;
            }
            
            // Initialiser le proxy si nécessaire
            const TeslaProxy = require('./tesla.proxy');
            if (!TeslaProxy.CONFIG.ready) {
                TeslaChargingLogger.writeToLog("🔧 POLLING: Proxy Tesla non initialisé, initialisation en cours");
                TeslaProxy.init();
                TeslaChargingLogger.writeToLog("✅ POLLING: Proxy Tesla initialisé avec succès");
            } else {
                TeslaChargingLogger.writeToLog("✅ POLLING: Proxy Tesla déjà initialisé");
            }
            
            // NOUVEAU: Vérifier d'abord si des données MQTT fraîches sont disponibles
            const mqttData = TeslaService.getMqttVehicleData();
            const mqttFresh = mqttData && mqttData._timestamp && 
                              (Date.now() - mqttData._timestamp < 10 * 60 * 1000); // 5 minutes
            
            if (mqttFresh && TeslaMqttClient?.isConnected) {
                TeslaChargingLogger.writeToLog("📡 POLLING: Données MQTT fraîches disponibles, priorité MQTT");
                
                // Ajouter métadonnées
                mqttData._timestamp = Date.now();
                mqttData._source = "mqtt";
                
                // Émettre via Socket.IO
                TeslaChargingLogger.writeToLog("📡 SOCKET.IO: Émission événement 'tesla:vehicle_data'");
                global.io.emit('tesla:vehicle_data', mqttData);
                
                // Créer objet compatible et émettre
                const compatData = TeslaService.createCompatData();
                global.io.emit('tesla:full_update', compatData);
                
                TeslaChargingLogger.writeToLog("✅ POLLING: Utilisation réussie des données MQTT fraîches");
                successCount++;
                return;
            }
            
            // NOUVEAU: Éviter les appels API trop rapprochés
            const MIN_API_INTERVAL = 180000; // 3 minutes minimum entre appels API
            const timeSinceLastApiCall = startTime - lastApiCallTime;
            
            if (timeSinceLastApiCall < MIN_API_INTERVAL) {
                TeslaChargingLogger.writeToLog(`⏱️ POLLING: Dernier appel API il y a ${Math.round(timeSinceLastApiCall/1000)}s < ${MIN_API_INTERVAL/1000}s minimum`);
                TeslaChargingLogger.writeToLog("💰 POLLING: Protection API activée, utilisation des données existantes");
                
                // Utiliser les données existantes pour Socket.IO
                if (TeslaService.last_receive_data) {
                    const compatData = TeslaService.createCompatData();
                    global.io.emit('tesla:full_update', compatData);
                    TeslaChargingLogger.writeToLog("📤 SOCKET.IO: Émission avec données existantes");
                    successCount++;
                }
                return;
            }
            
            // Vérifier d'abord si le véhicule est en ligne
            TeslaChargingLogger.writeToLog("🔍 POLLING: Vérification état du véhicule avant requête");
            const vehicleState = await this.checkVehicleOnlineStatus();
            TeslaChargingLogger.writeToLog(`🚗 POLLING: État du véhicule détecté: "${vehicleState}"`);
            
            if (vehicleState !== "online") {
                TeslaChargingLogger.writeToLog(`⚠️ POLLING: Véhicule ${vehicleState}, données non disponibles pour Socket.IO`);
                
                if (TeslaMqttClient?.isConnected) {
                    const mqttData = TeslaService.getMqttVehicleData();
                    if (mqttData) {
                        TeslaChargingLogger.writeToLog("📡 POLLING: Utilisation alternative des données MQTT pour Socket.IO");
                        
                        // Ajouter métadonnées
                        mqttData._timestamp = Date.now();
                        mqttData._source = "mqtt";
                        
                        // Émettre via Socket.IO
                        TeslaChargingLogger.writeToLog("📤 SOCKET.IO: Émission des données MQTT");
                        global.io.emit('tesla:vehicle_data', mqttData);
                        
                        // Créer objet compatible et émettre
                        const compatData = TeslaService.createCompatData();
                        global.io.emit('tesla:full_update', compatData);
                        
                        TeslaChargingLogger.writeToLog("✅ POLLING: Émission MQTT réussie malgré véhicule non en ligne");
                        successCount++;
                    }
                }
                return;
            }
            
            // MODIFIÉ: Véhicule en ligne - appel API avec mémorisation du timestamp
            lastApiCallTime = startTime;
            
            // Construire l'endpoint pour les données
            const dataEndpoint = `/api/1/vehicles/${vehicleIdOrVin}/vehicle_data`;
            TeslaChargingLogger.writeToLog(`📝 POLLING: Endpoint API construit: ${dataEndpoint}`);
            
            // Log avant requête API
            const apiStartTime = Date.now();
            TeslaChargingLogger.writeToLog("📡 POLLING: Requête des données véhicule via proxy pour Socket.IO");
            
            // NOUVEAU: Implémenter un throttling pour éviter les appels trop rapprochés
            const throttleTime = 3000; // 3 secondes entre appels API
            if (startTime - TeslaService.lastApiCallAttempt < throttleTime) {
                const waitTime = throttleTime - (startTime - TeslaService.lastApiCallAttempt);
                TeslaChargingLogger.writeToLog(`⏱️ API THROTTLE: Attente de ${waitTime}ms avant nouvel appel API`);
                await new Promise(resolve => setTimeout(resolve, waitTime));
            }
            TeslaService.lastApiCallAttempt = Date.now();
            
            // Requête des données via le proxy
            const response = await TeslaProxy.request('GET', dataEndpoint);
            
            // Log après requête API avec temps de réponse
            const apiDuration = Date.now() - apiStartTime;
            TeslaChargingLogger.writeToLog(`📡 POLLING: Réponse API reçue en ${apiDuration}ms (Statut: ${response.status})`);
            
            if (response.status === 200 && response.data?.response) {
                // Ajouter un timestamp
                response.data.response.timestamp = Date.now();
                response.data.response.data_source = 'api';
                
                // Journal des données clés reçues
                const batteryLevel = response.data.response.charge_state?.battery_level;
                const chargingState = response.data.response.charge_state?.charging_state;
                TeslaChargingLogger.writeToLog(`📊 POLLING: Données reçues - Batterie: ${batteryLevel || 'N/A'}%, État charge: ${chargingState || 'N/A'}`);
                
                // Mettre à jour les données locales
                const previousSource = this.last_receive_data?._source || 'aucun';
                this.last_receive_data = response.data.response;
                TeslaChargingLogger.writeToLog(`💾 POLLING: Mise à jour données locales (source précédente: ${previousSource})`);
                
                // Synchroniser avec MQTT si disponible
                this.syncDataSources();
                
                // Création des données compatibles
                const compatData = this.createCompatData();
                TeslaChargingLogger.writeToLog(`🔄 POLLING: Données compatibles créées pour frontend (Batterie: ${compatData.charge_state?.battery_level || 'N/A'}%)`);
                
                // Émettre via Socket.IO
                TeslaChargingLogger.writeToLog("📤 SOCKET.IO: Émission des données véhicule complètes");
                global.io.emit('tesla:vehicle_data', response.data.response);
                
                TeslaChargingLogger.writeToLog("📤 SOCKET.IO: Émission des données véhicule compatibles");
                global.io.emit('tesla:full_update', compatData);
                
                // Statistiques de réussite
                successCount++;
                TeslaChargingLogger.writeToLog(`✅ POLLING: Cycle #${pollCount} terminé avec succès (${successCount}/${pollCount} réussis)`);
            } else {
                // Réponse reçue mais incorrecte
                errorCount++;
                TeslaChargingLogger.writeToLog(`⚠️ POLLING: Réponse invalide - Statut ${response.status}, données: ${response.data ? 'présentes' : 'absentes'}`);
            }
        } catch (error) {
            // Gestion des erreurs
            errorCount++;
            
            if (error.response && error.response.status === 408) {
                TeslaChargingLogger.writeToLog("😴 POLLING: Véhicule endormi (408), polling interrompu");
                
                // Mise à jour de l'état connu
                this.lastKnownVehicleState = "asleep";
                this.lastVehicleStateCheck = Date.now();
            } else if (error.response) {
                // Erreurs avec réponse HTTP
                TeslaChargingLogger.writeToLog(`❌ POLLING ERROR: Statut ${error.response.status} - ${error.message}`);
                
                // Log détaillé de l'erreur
                if (error.response.data) {
                    try {
                        const errorDataStr = typeof error.response.data === 'string' 
                            ? error.response.data 
                            : JSON.stringify(error.response.data);
                        TeslaChargingLogger.writeToLog(`❌ POLLING ERROR DETAILS: ${errorDataStr.substring(0, 200)}`);
                    } catch (logError) {
                        TeslaChargingLogger.writeToLog(`❌ POLLING ERROR: Impossible de logger les détails - ${logError.message}`);
                    }
                }
                
                // Si erreur d'authentification
                if (error.response.status === 401) {
                    TeslaChargingLogger.writeToLog("🔑 POLLING: Erreur d'authentification, programmation reconnexion");
                    this.step = this.STEP_CONNECT;
                    this.next_action_time = Date.now() + 5000; // 5 secondes
                }
            } else if (error.request) {
                // Requête envoyée mais pas de réponse
                TeslaChargingLogger.writeToLog(`❌ POLLING ERROR: Pas de réponse du serveur - ${error.message}`);
            } else {
                // Erreur avant l'envoi de la requête
                TeslaChargingLogger.writeToLog(`❌ POLLING ERROR: ${error.message || 'Erreur inconnue'}`);
                console.error("Erreur polling Socket.IO:", error);
            }
            
            // Log statistiques d'erreur
            TeslaChargingLogger.writeToLog(`📊 POLLING STATS: ${errorCount}/${pollCount} cycles en erreur`);
        } finally {
            // Log de fin d'exécution avec temps total
            const executionTime = Date.now() - startTime;
            TeslaChargingLogger.writeToLog(`⏱️ POLLING: Cycle #${pollCount} terminé en ${executionTime}ms`);
            
            // Log statistiques périodiques (toutes les 10 exécutions)
            if (pollCount % 10 === 0) {
                TeslaChargingLogger.writeToLog(`📊 POLLING STATS: ${successCount}/${pollCount} cycles réussis (${Math.round(successCount/pollCount*100)}%), ${errorCount} erreurs`);
            }
        }
    }, pollInterval);
    
    // Log de démarrage détaillé
    TeslaChargingLogger.writeToLog(`✅ POLLING: Service démarré (interval=${pollInterval}ms, ID=${intervalId})`);
    console.log(`TeslaService: Service de polling Socket.IO actif (ID=${intervalId})`);
    
    // Retourner l'ID d'intervalle pour permettre l'arrêt ultérieur
    return intervalId;
}

/**
 * Crée un objet de données compatible avec le frontend
 * @returns {Object} Données formatées pour le frontend
 */
static createCompatData() {
    // Synchroniser les données MQTT et API avant traitement
    this.syncDataSources();
    
    TeslaChargingLogger.writeToLog("📋 CREATE_COMPAT: Démarrage création données compatibles frontend");
    
    // Vérification des données source
    const hasSourceData = !!this.last_receive_data;
    TeslaChargingLogger.writeToLog(`📋 CREATE_COMPAT: Données source ${hasSourceData ? 'présentes' : 'ABSENTES'}`);
    
    // Vérification des états spécifiques
    const hasChargeData = hasSourceData && !!this.last_receive_data.charge_state;
    const hasClimateData = hasSourceData && !!this.last_receive_data.climate_state;
    
    // Log détaillé des états disponibles
    TeslaChargingLogger.writeToLog(`📋 CREATE_COMPAT: État charge ${hasChargeData ? 'présent' : 'ABSENT'}, état climat ${hasClimateData ? 'présent' : 'ABSENT'}`);
    
    // Si des données sont manquantes, journaliser ce fait
    if (!hasChargeData) {
        TeslaChargingLogger.writeToLog("⚠️ CREATE_COMPAT: Données de charge manquantes dans la source");
    }
    if (!hasClimateData) {
        TeslaChargingLogger.writeToLog("⚠️ CREATE_COMPAT: Données de climat manquantes dans la source");
    }
    
    // Extraction des données spécifiques avec validation
    const batteryLevel = hasChargeData ? this.last_receive_data.charge_state.battery_level : null;
    const usableBatteryLevel = hasChargeData ? this.last_receive_data.charge_state.usable_battery_level : null;
    const chargingState = hasChargeData ? this.last_receive_data.charge_state.charging_state : null;
    const insideTemp = hasClimateData ? this.last_receive_data.climate_state.inside_temp : null;
    const outsideTemp = hasClimateData ? this.last_receive_data.climate_state.outside_temp : null;
    
    // Log des valeurs extraites
    TeslaChargingLogger.writeToLog(`📋 CREATE_COMPAT: Batterie=${batteryLevel ?? 'N/A'}%, Utilisable=${usableBatteryLevel ?? 'N/A'}%, État=${chargingState ?? 'N/A'}`);
    TeslaChargingLogger.writeToLog(`📋 CREATE_COMPAT: Temp. intérieure=${insideTemp ?? 'N/A'}°C, Temp. extérieure=${outsideTemp ?? 'N/A'}°C`);
    
    // Création de l'objet de données avec métadonnées enrichies
    const timestamp = Date.now();
    const source = this.last_receive_data?._source || 'unknown';
    const sourceTimestamp = this.last_receive_data?._timestamp || this.last_receive_data?.timestampMs || null;
    const dataAge = sourceTimestamp ? Math.floor((timestamp - sourceTimestamp) / 1000) : null;
    
    // Créer l'objet compatible
    const compatData = {
        charge_state: {
            battery_level: batteryLevel,
            usable_battery_level: usableBatteryLevel,
            charging_state: chargingState
        },
        climate_state: {
            inside_temp: insideTemp,
            outside_temp: outsideTemp
        },
        _timestamp: timestamp,
        _source: source,
        _age_seconds: dataAge,
        _valid: hasSourceData && hasChargeData
    };
    
    // Log de fin avec statut
    TeslaChargingLogger.writeToLog(`✅ CREATE_COMPAT: Création terminée, données ${compatData._valid ? 'valides' : 'INCOMPLÈTES'} (source: ${source})`);
    
    // Log de debug pour trace dans console
    console.log(`TeslaService: Données compatibles créées - Batterie: ${batteryLevel ?? 'N/A'}%, Age: ${dataAge ?? 'N/A'}s, Source: ${source}`);
    
    return compatData;
}
        ;

/**
 * Traite les données reçues du véhicule et les émet via Socket.IO
 * Cette méthode est appelée après chaque réception de données (API ou MQTT)
 */
static handleVehicleDataReceived() {
    // Synchroniser les données MQTT et API avant traitement
    TeslaService.syncDataSources();
    
    // Vérifier si Socket.IO est disponible
    if (!global.io) {
        TeslaChargingLogger.writeToLog("⚠️ EMISSION: Socket.IO non disponible, impossible d'émettre les données");
        return;
    }
    
    TeslaChargingLogger.writeToLog("📤 EMISSION: Traitement des données pour envoi au frontend");
    
    // Vérifier si des données sont disponibles
    if (!this.last_receive_data) {
        TeslaChargingLogger.writeToLog("❌ EMISSION ERREUR: Aucune donnée disponible à émettre");
        return;
    }
    
    // Vérifier les données de charge essentielles
    const hasChargeState = !!this.last_receive_data.charge_state;
    const batteryLevel = this.last_receive_data?.charge_state?.battery_level;
    const usableBatteryLevel = this.last_receive_data?.charge_state?.usable_battery_level;
    const chargingState = this.last_receive_data?.charge_state?.charging_state;
    
    // LOGS DE DIAGNOSTICS pour l'affichage de la batterie
    TeslaChargingLogger.writeToLog(`🔋 EMISSION DIAGNOSTIC: Batterie=${batteryLevel ?? 'N/A'}%, Utilisable=${usableBatteryLevel ?? 'N/A'}%, État=${chargingState ?? 'N/A'}`);
    
    console.log("DONNÉES TESLA - DEBUG BATTERIE:", {
        battery_level: batteryLevel,
        usable_battery_level: usableBatteryLevel,
        charging_state: chargingState,
        charge_state_existe: hasChargeState,
        structure_complete: JSON.stringify(this.last_receive_data?.charge_state || {}).substring(0, 100) + "..."
    });
    
    // Extraction des données climatiques si disponibles
    const hasClimateState = !!this.last_receive_data.climate_state;
    const insideTemp = this.last_receive_data?.climate_state?.inside_temp;
    const outsideTemp = this.last_receive_data?.climate_state?.outside_temp;
    
    if (hasClimateState) {
        TeslaChargingLogger.writeToLog(`🌡️ EMISSION CLIMAT: Intérieur=${insideTemp ?? 'N/A'}°C, Extérieur=${outsideTemp ?? 'N/A'}°C`);
    } else {
        TeslaChargingLogger.writeToLog("⚠️ EMISSION CLIMAT: Données climatiques non disponibles");
    }
    
    // Source des données et âge
    const dataSource = this.last_receive_data._source || 'inconnu';
    const timestamp = Date.now();
    const lastUpdate = this.last_receive_data?._timestamp || this.last_receive_data?.timestampMs;
    const dataAge = lastUpdate ? Math.round((timestamp - lastUpdate)/1000) : null;
    
    TeslaChargingLogger.writeToLog(`📊 EMISSION SOURCE: ${dataSource}, Âge=${dataAge ? dataAge + 's' : 'inconnu'}`);
    
    // SOLUTION DE COMPATIBILITÉ: Créer un objet avec la structure attendue par le composant Angular
    const compatData = {
        charge_state: {
            battery_level: batteryLevel,
            usable_battery_level: usableBatteryLevel,
            charging_state: chargingState
        },
        climate_state: {
            inside_temp: insideTemp,
            outside_temp: outsideTemp
        },
        _timestamp: timestamp,
        _source: dataSource,
        _age_seconds: dataAge
    };
    
    // Analyser la complétude des données
    const isComplete = hasChargeState && batteryLevel !== undefined && batteryLevel !== null;
    if (!isComplete) {
        TeslaChargingLogger.writeToLog("⚠️ EMISSION ALERTE: Données incomplètes envoyées au frontend");
    }
    
    // Log des données envoyées au frontend
    TeslaChargingLogger.writeToLog(`📤 EMISSION FRONTEND: Batterie=${compatData.charge_state?.battery_level ?? 'N/A'}%, Utilisable=${compatData.charge_state?.usable_battery_level ?? 'N/A'}%`);
    console.log("ENVOI AU FRONTEND:", {
        battery_level: compatData.charge_state?.battery_level,
        usable_battery_level: compatData.charge_state?.usable_battery_level
    });
    
    try {
        // Émettre l'événement avec la structure compatible
        TeslaChargingLogger.writeToLog("📡 SOCKET.IO: Émission événement 'tesla:full_update'");
        global.io.emit('tesla:full_update', compatData);
        
        // Optionnel : si d'autres parties du front-end ont besoin de la structure complète
        TeslaChargingLogger.writeToLog("📡 SOCKET.IO: Émission événement 'tesla:vehicle_data'");
        global.io.emit('tesla:vehicle_data', this.last_receive_data);
        
        TeslaChargingLogger.writeToLog("✅ EMISSION: Données envoyées avec succès au frontend");
    } catch (error) {
        TeslaChargingLogger.writeToLog(`❌ EMISSION ERREUR: Échec d'émission Socket.IO - ${error.message}`);
        console.error("Erreur lors de l'émission Socket.IO:", error);
    }
}

;
          
/**
 * Vérifie et ajuste le mode de charge solaire en fonction de la production
 * @param {boolean} forceCheck - Paramètre conservé pour compatibilité mais ignoré pour la protection de délai
 * @returns {Promise<boolean>} - True si ajustement effectué
 */
static async checkChargingMode(forceCheck = false) {
    // Initialiser la variable si elle n'existe pas (première exécution)
    if (!TeslaService.lastCheckAttemptTime) {
        TeslaService.lastCheckAttemptTime = 0;
    }

    // PROTECTION ABSOLUE: Limiter les tentatives d'exécution de la fonction elle-même
    const now = new Date().getTime();
    const minExecutionInterval = 5000; // 5 secondes minimum entre les TENTATIVES d'exécution
    
    if (now - TeslaService.lastCheckAttemptTime < minExecutionInterval) {
        // Ne pas journaliser pour éviter de surcharger les logs
        return false;
    }
    
    // Mettre à jour le timestamp de la tentative
    TeslaService.lastCheckAttemptTime = now;

    // Début de l'exécution (uniquement si le délai minimum est respecté)
    TeslaChargingLogger.writeToLog("🔌 CHARGE SOLAIRE: Début vérification mode de charge");

    // Obtenir la configuration
    const config = configuration_service_1.ConfigurationService.getConfiguration();
    
    // PROTECTION BASÉE UNIQUEMENT SUR LA CONFIGURATION - ignore complètement forceCheck
    if (TeslaService.lastChangeChargingAmps && 
        (now - TeslaService.lastChangeChargingAmps < config.tesla.adaptChargeFrequency * 1000)) {
        const secondsSinceLastChange = Math.round((now - TeslaService.lastChangeChargingAmps)/1000);
        TeslaChargingLogger.writeToLog(`⛔ CHARGE SOLAIRE: Dernier ajustement il y a ${secondsSinceLastChange}s, minimum ${config.tesla.adaptChargeFrequency}s requis selon configuration, AUCUNE EXCEPTION`);
        return false;
    }

    // Vérifier si le véhicule est à la maison via GPS
    const isAtHome = await TeslaService.isTeslaAtHome();
    TeslaChargingLogger.writeToLog(`🏠 CHARGE SOLAIRE: Vérification position GPS: véhicule ${isAtHome ? 'à la maison' : 'pas à la maison'}`);
    
    if (!isAtHome) {
        TeslaChargingLogger.writeToLog("❌ CHARGE SOLAIRE: Véhicule hors domicile, abandon de l'ajustement");
        return false;
    }

    // Log des informations d'état
    TeslaChargingLogger.writeToLog(`🔌 CHARGE SOLAIRE: Mode actuel="${TeslaService.chargingMode}", Step=${TeslaService.getStepName(TeslaService.step)}`);
    const chargingState = TeslaService.last_receive_data?.charge_state?.charging_state;
    TeslaChargingLogger.writeToLog(`🔌 CHARGE SOLAIRE: État de charge="${chargingState}"`);
    TeslaChargingLogger.writeToLog(`🔌 CHARGE SOLAIRE: Configuration chargée - Fréquence=${config.tesla.adaptChargeFrequency}s, Offset=${config.tesla.chargeOffset}W`);

    // Mettre à jour le dernier temps de vérification
    TeslaChargingLogger.writeToLog(`⏱️ CHARGE SOLAIRE: Vérification autorisée`);
    TeslaService.lastCheckChargingMode = now;
    
    // Vérifier si nous sommes en mode d'adaptation solaire
    if (TeslaService.chargingMode !== 'solar' && 
        TeslaService.chargingMode !== 'adapt' && 
        TeslaService.chargingMode !== 'solarPlus' && 
        TeslaService.chargingMode !== 'solarStart') {
        TeslaChargingLogger.writeToLog(`❌ CHARGE SOLAIRE: Mode "${TeslaService.chargingMode}" non compatible avec l'adaptation solaire`);
        return false;
    }
    
    // Vérifier si le véhicule est en charge
    if (chargingState !== "Charging" && chargingState !== "Starting") {
        TeslaChargingLogger.writeToLog(`❌ CHARGE SOLAIRE: Véhicule non en charge (${chargingState}), pas d'ajustement nécessaire`);
        return false;
    }
    
    // Récupérer la surproduction actuelle brute
    TeslaChargingLogger.writeToLog("🔌 CHARGE SOLAIRE: Récupération de la surproduction actuelle");
    let overProduction = enphase_data_service_1.EnphaseDataService.getOverproduction(config.tesla.adaptChargeFrequency);
    TeslaChargingLogger.writeToLog(`⚡ CHARGE SOLAIRE: Surproduction brute calculée = ${overProduction}W`);
    
    // Récupérer la tension et l'ampérage actuels
    const voltage = TeslaService.last_receive_data?.charge_state?.charger_voltage || 220;
    TeslaChargingLogger.writeToLog(`⚡ CHARGE SOLAIRE: Tension de charge = ${voltage}V`);
    
    // CORRECTION CRITIQUE: Récupérer l'ampérage actuel et calculer la consommation Tesla
    const currentAmps = TeslaService.last_receive_data?.charge_state?.charger_actual_current || 
                       TeslaService.last_receive_data?.charge_state?.charge_current_request || 
                       TeslaService.last_receive_data?.charge_state?.charge_amps || 0;
    
    // Calculer la consommation actuelle de la Tesla
    const teslaPower = currentAmps * voltage;
    TeslaChargingLogger.writeToLog(`⚡ CHARGE SOLAIRE: Consommation Tesla actuelle = ${teslaPower}W (${currentAmps}A × ${voltage}V)`);
    
    // CORRECTION: Ajouter la consommation Tesla à la surproduction pour obtenir la vraie surproduction
    const correctedOverProduction = overProduction + teslaPower;
    TeslaChargingLogger.writeToLog(`⚡ CHARGE SOLAIRE: Surproduction corrigée (avec Tesla) = ${correctedOverProduction}W`);
    
    // Appliquer l'offset de charge configuré (toujours utiliser la même valeur)
    const offsetWatts = config.tesla.chargeOffset || -250;
    TeslaChargingLogger.writeToLog(`⚡ CHARGE SOLAIRE: Offset appliqué = ${offsetWatts}W`);
    const adjustedOverProduction = correctedOverProduction + offsetWatts;
    TeslaChargingLogger.writeToLog(`⚡ CHARGE SOLAIRE: Surproduction ajustée avec offset = ${adjustedOverProduction}W`);
    
    // Calculer l'ampérage idéal basé sur la surproduction ajustée
    const idealAmps = Math.floor(adjustedOverProduction / voltage);
    TeslaChargingLogger.writeToLog(`⚡ CHARGE SOLAIRE: Ampérage idéal calculé = ${idealAmps}A (${adjustedOverProduction}W / ${voltage}V)`);
    
    // Appliquer les limites d'ampérage
    const minAmps = config.tesla.minAmps || 6;
    const maxAmps = config.tesla.maxAmps || 16;
    
    // Déterminer l'ampérage final en respectant min/max
    let targetAmps = idealAmps;
    
    // S'assurer que l'ampérage est dans les limites
    if (targetAmps < minAmps) {
        TeslaChargingLogger.writeToLog(`⚡ CHARGE SOLAIRE: Ampérage ajusté à la limite minimale: ${idealAmps}A → ${minAmps}A`);
        targetAmps = minAmps;
    } else if (targetAmps > maxAmps) {
        TeslaChargingLogger.writeToLog(`⚡ CHARGE SOLAIRE: Ampérage ajusté à la limite maximale: ${idealAmps}A → ${maxAmps}A`);
        targetAmps = maxAmps;
    }
    
    // Vérifier si un changement est nécessaire
    if (targetAmps === currentAmps) {
        TeslaChargingLogger.writeToLog(`✅ CHARGE SOLAIRE: Aucun changement nécessaire, ampérage déjà à ${currentAmps}A`);
        return false;
    }
    
    // Appliquer le nouvel ampérage
    TeslaChargingLogger.writeToLog(`🔄 CHARGE SOLAIRE: Modification de l'ampérage: ${currentAmps}A → ${targetAmps}A`);
    console.log(`Tesla: Adaptation solaire - Ampérage modifié de ${currentAmps}A à ${targetAmps}A`);
    
    try {
        const success = await TeslaService.setChargeAmps(targetAmps);
        
        if (success) {
            // Mettre à jour le timestamp du dernier changement d'ampérage
            TeslaService.lastChangeChargingAmps = now;
            TeslaChargingLogger.writeToLog(`✅ CHARGE SOLAIRE: Ampérage modifié avec succès à ${targetAmps}A`);
            return true;
        } else {
            TeslaChargingLogger.writeToLog(`❌ CHARGE SOLAIRE: Échec de la modification d'ampérage à ${targetAmps}A`);
            return false;
        }
    } catch (error) {
        TeslaChargingLogger.writeToLog(`❌ CHARGE SOLAIRE ERREUR: ${error.message}`);
        console.error(`Tesla: Erreur lors de l'adaptation solaire - ${error.message}`);
        return false;
    }
}

        ;
        
/**
 * Met à jour les données du véhicule depuis MQTT en les adaptant à la structure de l'API Tesla
 * @param {string} attribute - Le nom de l'attribut MQTT
 * @param {any} value - La valeur de l'attribut
 */
static updateLastReceiveDataFromMqtt(attribute, value) {
    TeslaChargingLogger.writeToLog(`🔄 MQTT UPDATE: Attribut="${attribute}", Valeur=${typeof value === 'object' ? 'objet' : value}`);
    
    // Initialiser la structure de données si elle n'existe pas
    if (!this.last_receive_data) {
        TeslaChargingLogger.writeToLog("🔄 MQTT UPDATE: Initialisation du cache de données");
        this.last_receive_data = {};
    }
    
    // Gérer les attributs en fonction de leur catégorie
    switch (attribute) {
        // Attributs d'état de charge
        case 'battery_level':
            this.initializeObjectPath('charge_state');
            this.last_receive_data.charge_state.battery_level = Number(value);
            TeslaChargingLogger.writeToLog(`🔋 MQTT UPDATE: Niveau de batterie mis à jour à ${value}%`);
            break;
            
        case 'usable_battery_level':
            this.initializeObjectPath('charge_state');
            this.last_receive_data.charge_state.usable_battery_level = Number(value);
            TeslaChargingLogger.writeToLog(`🔋 MQTT UPDATE: Niveau utilisable mis à jour à ${value}%`);
            break;
            
        case 'charging_state':
            this.initializeObjectPath('charge_state');
            this.last_receive_data.charge_state.charging_state = String(value);
            TeslaChargingLogger.writeToLog(`🔌 MQTT UPDATE: État de charge mis à jour à "${value}"`);
            
            // Mettre à jour l'indicateur d'état de charge
            const wasCharging = this.wasInCharge;
            this.wasInCharge = (value === 'Charging' || value === 'Starting');
            
            if (this.wasInCharge !== wasCharging) {
                TeslaChargingLogger.writeToLog(`🔌 MQTT UPDATE: Indicateur wasInCharge changé de ${wasCharging} à ${this.wasInCharge}`);
            }
            break;
            
        case 'charge_amps':
            this.initializeObjectPath('charge_state');
            this.last_receive_data.charge_state.charge_amps = Number(value);
            TeslaChargingLogger.writeToLog(`🔌 MQTT UPDATE: Ampérage de charge mis à jour à ${value}A`);
            break;
            
        case 'charger_power':
            this.initializeObjectPath('charge_state');
            this.last_receive_data.charge_state.charger_power = Number(value);
            TeslaChargingLogger.writeToLog(`⚡ MQTT UPDATE: Puissance de charge mise à jour à ${value}kW`);
            
            // Ajouter aux données de charge pour graphiques si en charge
            if (this.last_receive_data.charge_state?.charging_state === 'Charging') {
                const now = Math.floor(Date.now() / 1000);
                const wCharge = Number(value) * 1000; // Conversion en watts
                this.addToChargeData({
                    timestamp: now,
                    w_charge: wCharge
                });
                TeslaChargingLogger.writeToLog(`📊 MQTT UPDATE: Données de charge ajoutées (${wCharge}W)`);
            }
            break;
            
        case 'charge_limit_soc':
            this.initializeObjectPath('charge_state');
            this.last_receive_data.charge_state.charge_limit_soc = Number(value);
            TeslaChargingLogger.writeToLog(`🎚️ MQTT UPDATE: Limite de charge mise à jour à ${value}%`);
            break;
            
        // Attributs d'état du climat
        case 'inside_temp':
            this.initializeObjectPath('climate_state');
            this.last_receive_data.climate_state.inside_temp = Number(value);
            TeslaChargingLogger.writeToLog(`🌡️ MQTT UPDATE: Température intérieure mise à jour à ${value}°C`);
            break;
            
        case 'outside_temp':
            this.initializeObjectPath('climate_state');
            this.last_receive_data.climate_state.outside_temp = Number(value);
            TeslaChargingLogger.writeToLog(`🌡️ MQTT UPDATE: Température extérieure mise à jour à ${value}°C`);
            break;
            
        case 'is_climate_on':
            this.initializeObjectPath('climate_state');
            this.last_receive_data.climate_state.is_climate_on = value === 'true' || value === true;
            TeslaChargingLogger.writeToLog(`❄️ MQTT UPDATE: État climatisation mis à jour à ${this.last_receive_data.climate_state.is_climate_on}`);
            break;
            
        // Attributs de conduite
        case 'latitude':
            this.initializeObjectPath('drive_state');
            this.last_receive_data.drive_state.latitude = Number(value);
            // Masquage pour les logs pour préserver la confidentialité
            const latFormatted = Number(value).toFixed(2);
            TeslaChargingLogger.writeToLog(`📍 MQTT UPDATE: Latitude mise à jour à ${latFormatted}°`);
            break;
            
        case 'longitude':
            this.initializeObjectPath('drive_state');
            this.last_receive_data.drive_state.longitude = Number(value);
            // Masquage pour les logs pour préserver la confidentialité
            const lonFormatted = Number(value).toFixed(2);
            TeslaChargingLogger.writeToLog(`📍 MQTT UPDATE: Longitude mise à jour à ${lonFormatted}°`);
            break;
            
        case 'shift_state':
            this.initializeObjectPath('drive_state');
            this.last_receive_data.drive_state.shift_state = value === null ? null : String(value);
            TeslaChargingLogger.writeToLog(`🚗 MQTT UPDATE: État de conduite mis à jour à "${value || 'stationnaire'}"`);
            break;
            
        // Attributs d'état du véhicule
        case 'locked':
            this.initializeObjectPath('vehicle_state');
            this.last_receive_data.vehicle_state.locked = value === 'true' || value === true;
            TeslaChargingLogger.writeToLog(`🔒 MQTT UPDATE: État verrouillage mis à jour à ${this.last_receive_data.vehicle_state.locked}`);
            break;
            
        case 'sentry_mode':
            this.initializeObjectPath('vehicle_state');
            this.last_receive_data.vehicle_state.sentry_mode = value === 'true' || value === true;
            TeslaChargingLogger.writeToLog(`👁️ MQTT UPDATE: Mode sentinelle mis à jour à ${this.last_receive_data.vehicle_state.sentry_mode}`);
            break;
            
        case 'windows_open':
            this.initializeObjectPath('vehicle_state');
            this.last_receive_data.vehicle_state.windows_open = value === 'true' || value === true;
            TeslaChargingLogger.writeToLog(`🪟 MQTT UPDATE: État fenêtres mis à jour à ${this.last_receive_data.vehicle_state.windows_open}`);
            break;
            
        // État global du véhicule
        case 'state':
            this.last_receive_data.state = String(value);
            // Mise à jour de l'état connu du véhicule
            this.lastKnownVehicleState = String(value);
            this.lastVehicleStateCheck = Date.now();
            TeslaChargingLogger.writeToLog(`🚗 MQTT UPDATE: État global du véhicule mis à jour à "${value}"`);
            break;
            
        case 'display_name':
            this.last_receive_data.display_name = String(value);
            TeslaChargingLogger.writeToLog(`📝 MQTT UPDATE: Nom du véhicule mis à jour à "${value}"`);
            break;
            
        // État par défaut pour les attributs non reconnus
        default:
            // Log des attributs non reconnus avec leur chemin MQTT, mais sans mise à jour
            TeslaChargingLogger.writeToLog(`❓ MQTT UPDATE: Attribut non géré "${attribute}" (valeur: ${typeof value === 'object' ? 'objet' : value})`);
            break;
    }
    
    // Mettre à jour un timestamp pour signaler la fraîcheur des données
    this.last_receive_data._timestamp = Date.now();
    this.last_receive_data._source = "mqtt";
    this.last_data_timestamp = Date.now();
    
    // Log de synthèse
    const knownAttributes = [
        'battery_level', 'usable_battery_level', 'charging_state', 
        'inside_temp', 'outside_temp', 'state'
    ];
    
    if (knownAttributes.includes(attribute)) {
        TeslaChargingLogger.writeToLog(`✅ MQTT UPDATE: Attribut "${attribute}" mis à jour avec succès`);
    }
}

/**
 * Méthode utilitaire pour initialiser un chemin d'objet dans la structure de données
 * @param {string} path - Chemin d'objet à initialiser ('charge_state', 'climate_state', etc.)
 * @private
 */
static initializeObjectPath(path) {
    if (!this.last_receive_data[path]) {
        this.last_receive_data[path] = {};
        TeslaChargingLogger.writeToLog(`🔄 MQTT INIT: Création de la structure ${path}`);
    }
}

;

/**
 * Crée une instance Axios configurée pour les API Tesla avec options personnalisables
 * @param {Object} options - Options de personnalisation
 * @param {number} options.timeout - Timeout en ms (défaut: 30000)
 * @param {boolean} options.jsonContent - Ajouter Content-Type JSON (défaut: false) 
 * @param {boolean} options.shortTimeout - Utiliser un timeout court pour les requêtes légères (défaut: false)
 * @returns {AxiosInstance} Instance Axios configurée
 */
static createAxiosInstance(options = {}) {
    TeslaChargingLogger.writeToLog("📡 AXIOS: Création d'une nouvelle instance HTTP client");
    TeslaChargingLogger.writeToLog(`📡 AXIOS: Options reçues - shortTimeout=${!!options.shortTimeout}, jsonContent=${!!options.jsonContent}, timeout=${options.timeout || 'défaut'}`);
    
    // Déterminer le timeout approprié avec log explicatif
    const timeoutValue = options.shortTimeout ? 10000 : (options.timeout || 30000);
    TeslaChargingLogger.writeToLog(`⏱️ AXIOS: Timeout configuré à ${timeoutValue}ms ${options.shortTimeout ? '(mode court)' : options.timeout ? '(personnalisé)' : '(défaut)'}`);
    
    // Configuration de base
    const config = {
        timeout: timeoutValue,
        maxContentLength: Infinity,
        maxBodyLength: Infinity,
        decompress: false,
        headers: {
            'User-Agent': 'curl/8.10.1',
            'Accept': '*/*',
            'Accept-Encoding': 'identity'
        }
    };
    
    // Logger les détails de configuration pour débogage
    TeslaChargingLogger.writeToLog(`📋 AXIOS CONFIG: maxContentLength=${config.maxContentLength}, maxBodyLength=${config.maxBodyLength}, decompress=${config.decompress}`);
    TeslaChargingLogger.writeToLog(`🔤 AXIOS HEADERS: User-Agent='${config.headers['User-Agent']}', Accept='${config.headers['Accept']}', Accept-Encoding='${config.headers['Accept-Encoding']}'`);
    
    // Ajouter Content-Type pour les requêtes POST si demandé
    if (options.jsonContent) {
        config.headers['Content-Type'] = 'application/json';
        TeslaChargingLogger.writeToLog("🔤 AXIOS HEADERS: Ajout Content-Type='application/json' pour requêtes JSON");
    }
    
    // Ajouter agent HTTPS personnalisé si fourni
    if (options.httpsAgent) {
        config.httpsAgent = options.httpsAgent;
        TeslaChargingLogger.writeToLog("🔐 AXIOS CONFIG: Agent HTTPS personnalisé configuré");
        
        // Vérifier si l'agent est configuré pour ignorer les erreurs de certificat
        const rejectUnauthorized = options.httpsAgent?.options?.rejectUnauthorized !== false;
        TeslaChargingLogger.writeToLog(`🔐 AXIOS SSL: Validation certificats ${rejectUnauthorized ? 'activée' : 'DÉSACTIVÉE'}`);
    }
    
    // Créer l'instance Axios
    try {
        const instance = axios_1.default.create(config);
        TeslaChargingLogger.writeToLog("✅ AXIOS: Instance créée avec succès");
        
        // Diagnostic de l'instance
        console.log(`TeslaService: Instance Axios créée (timeout=${timeoutValue}ms, jsonContent=${!!options.jsonContent})`);
        
        return instance;
    } catch (error) {
        TeslaChargingLogger.writeToLog(`❌ AXIOS ERREUR: Échec création instance - ${error.message}`);
        console.error("Erreur lors de la création de l'instance Axios:", error);
        
        // En cas d'erreur, créer une instance minimale de secours
        TeslaChargingLogger.writeToLog("🔄 AXIOS: Création instance de secours avec configuration minimale");
        return axios_1.default.create({timeout: 30000});
    }
}

;
        
/**
 * Essai de se connecter a l'API et si réussi alors positionne le token et appel la callback
 */
static async connect() {
    try {
        // NOUVEAU: Vérifier la cohérence des tokens avant toute tentative de connexion
        const TeslaProxy = require('./tesla.proxy');
        await TeslaProxy.checkTokenConsistency();
        TeslaChargingLogger.writeToLog("✅ CONNEXION: Cohérence des tokens vérifiée avant connexion");
    
        // Log de début d'opération
        TeslaChargingLogger.writeToLog("🔑 CONNEXION: Démarrage tentative d'authentification Tesla");
        console.log("TeslaService: Tentative de connexion à l'API Tesla");
        
        let d = new Date();
        TeslaService.next_action_time = d.getTime() + 10 * 1000; // Réessaie dans 10 secondes si pas de réponse
        
        // Log du délai configuré
        TeslaChargingLogger.writeToLog(`⏱️ CONNEXION: Délai d'expiration configuré à 10s (prochaine tentative à ${new Date(TeslaService.next_action_time).toISOString()})`);
        
        // Vérification de l'état du proxy (déjà importé au début)
        TeslaChargingLogger.writeToLog(`🔍 CONNEXION: Vérification état proxy - prêt=${TeslaProxy.CONFIG.ready}`);
        
        if (!TeslaProxy.CONFIG.ready) {
            TeslaChargingLogger.writeToLog("🔄 CONNEXION: Proxy Tesla non initialisé, lancement initialisation");
            TeslaProxy.init();
            TeslaChargingLogger.writeToLog("✅ CONNEXION: Initialisation du proxy terminée");
        } else {
            TeslaChargingLogger.writeToLog("ℹ️ CONNEXION: Proxy Tesla déjà initialisé");
        }
        
        // Log des identifiants de connexion (masqués)
        TeslaChargingLogger.writeToLog("🔒 CONNEXION: Utilisation des identifiants configurés");
        
        // Vérifier configuration
        const config = configuration_service_1.ConfigurationService.getConfiguration();
        const tokenConfigured = !!(config.tesla?.token?.length > 20);
        TeslaChargingLogger.writeToLog(`🔐 CONNEXION: Configuration token ${tokenConfigured ? 'présente' : 'MANQUANTE ou INVALIDE'}`);
        
        if (!tokenConfigured) {
            TeslaChargingLogger.writeToLog("❌ CONNEXION: Impossible de se connecter - token non configuré");
            TeslaService.next_action_time = d.getTime() + 60 * 1000; // Attendre 1 minute
            return;
        }
        
        TeslaChargingLogger.writeToLog("📡 CONNEXION: Tentative d'authentification via proxy local");
        
        // S'assurer d'avoir un token d'accès valide
        const tokenSuccess = await TeslaProxy.ensureAccessToken();
        
        if (tokenSuccess) {
            TeslaChargingLogger.writeToLog("✅ CONNEXION: Authentification réussie via proxy local");
            TeslaChargingLogger.writeToLog(`🔑 CONNEXION: Token d'accès obtenu et valide`);
            
            // Sauvegarde du token pour références futures
            TeslaService.access_token = TeslaProxy.CONFIG.auth.accessToken;
            
            // Log de l'état du token (masqué)
            if (TeslaService.access_token) {
                const maskedToken = TeslaService.access_token.substring(0, 5) + '...' + 
                                  TeslaService.access_token.substring(TeslaService.access_token.length - 5);
                TeslaChargingLogger.writeToLog(`🔐 CONNEXION: Token stocké (${maskedToken})`);
            }
            
            let d2 = new Date();
            TeslaService.next_action_time = d2.getTime() + 0 * 1000;
            TeslaChargingLogger.writeToLog(`⏱️ CONNEXION: Planification action suivante immédiate (${new Date(TeslaService.next_action_time).toISOString()})`);
            
            // Gestion du mode de fonctionnement
            if (TeslaService.mode === TeslaService.MODE_REFRESH_TOKEN) {
                TeslaChargingLogger.writeToLog(`🔄 CONNEXION: Retour du mode ${TeslaService.getModeName(TeslaService.mode)} au mode précédent ${TeslaService.getModeName(TeslaService.previousMode)}`);
                TeslaChargingLogger.writeToLog(`🔄 CONNEXION: Retour de l'étape ${TeslaService.getStepName(TeslaService.step)} à l'étape précédente ${TeslaService.getStepName(TeslaService.previousStep)}`);
                
                TeslaService.mode = TeslaService.previousMode;
                TeslaService.step = TeslaService.previousStep;
                
                if (TeslaService.mode === TeslaService.MODE_REFRESH_TOKEN) {
                    TeslaChargingLogger.writeToLog(`⚠️ CONNEXION: Mode précédent était aussi REFRESH_TOKEN, correction vers GET_DATA`);
                    TeslaService.mode = TeslaService.MODE_GET_DATA;
                    TeslaService.step = TeslaService.STEP_GET_DATA;
                }
            }
            else {
                TeslaChargingLogger.writeToLog(`🔄 CONNEXION: Passage à l'étape GET_VEHICLES (${TeslaService.STEP_GET_VEHICLES})`);
                TeslaService.step = TeslaService.STEP_GET_VEHICLES;
            }
            
            TeslaChargingLogger.writeToLog(`✅ CONNEXION: État final - Mode=${TeslaService.getModeName(TeslaService.mode)}, Étape=${TeslaService.getStepName(TeslaService.step)}`);
            
            // Vérifier si un VIN est déjà configuré
            const configuredVin = config.tesla.vin;
            if (configuredVin) {
                TeslaChargingLogger.writeToLog(`🚗 CONNEXION: VIN déjà configuré: ${configuredVin.substring(0, 5)}...${configuredVin.substring(configuredVin.length - 5)}`);
            } else {
                TeslaChargingLogger.writeToLog(`⚠️ CONNEXION: VIN non configuré, sera récupéré à l'étape suivante`);
            }
        } else {
            TeslaChargingLogger.writeToLog("❌ CONNEXION: Échec de l'authentification via proxy local");
            
            // Diagnostic du problème
            const authStatus = TeslaProxy.CONFIG.auth || {};
            TeslaChargingLogger.writeToLog(`❌ DIAGNOSTIC: accessToken=${!!authStatus.accessToken}, refreshToken=${!!authStatus.refreshToken}, expires_in=${authStatus.expires_in || 'N/A'}`);
            
            // Ajuster le temps d'attente avant nouvelle tentative
            TeslaService.next_action_time = new Date().getTime() + 60000; // 1 minute
            TeslaChargingLogger.writeToLog(`⏱️ CONNEXION: Prochaine tentative dans 60s (${new Date(TeslaService.next_action_time).toISOString()})`);
        }
    } catch (error) {
        TeslaChargingLogger.writeToLog(`❌ ERREUR CONNEXION: ${error.message}`);
        console.error('Erreur de connexion au proxy Tesla local:', error);
        
        // Log amélioré pour le diagnostic
        if (error.response) {
            TeslaChargingLogger.writeToLog(`❌ ERREUR HTTP: Statut=${error.response.status}`);
            
            // Extraire et logger des détails de l'erreur de façon sécurisée
            try {
                const errorDetails = typeof error.response.data === 'string' 
                    ? error.response.data.substring(0, 100) 
                    : JSON.stringify(error.response.data).substring(0, 100);
                TeslaChargingLogger.writeToLog(`❌ ERREUR DÉTAILS: ${errorDetails}...`);
            } catch (logError) {
                TeslaChargingLogger.writeToLog(`❌ ERREUR LOGS: Impossible de formater les détails de l'erreur`);
            }
            
            console.error(`Erreur de réponse proxy: ${error.response.status}`);
            console.error('Détails:', error.response.data ? 'Données présentes' : 'Pas de données');
        } else if (error.request) {
            TeslaChargingLogger.writeToLog(`❌ ERREUR RÉSEAU: Pas de réponse du serveur`);
            console.error(`Erreur de requête: Pas de réponse`);
        } else {
            TeslaChargingLogger.writeToLog(`❌ ERREUR GÉNÉRALE: ${error.message}`);
            console.error(`Erreur de requête: ${error.message}`);
        }
        
        // Vérifier si c'est un problème de certificat SSL
        if (error.message && error.message.includes('CERT')) {
            TeslaChargingLogger.writeToLog(`🔐 ERREUR SSL: Problème potentiel de certificat - ${error.message}`);
        }
        
        // Vérifier si c'est un problème de configuration
        if (error.message && error.message.includes('config')) {
            TeslaChargingLogger.writeToLog(`⚙️ ERREUR CONFIGURATION: Problème potentiel de configuration - ${error.message}`);
            TeslaChargingLogger.writeToLog(`💡 CONSEIL: Vérifiez le fichier de configuration et les paramètres Tesla`);
        }
        
        // Redémarrage de la séquence après un délai
        TeslaService.step = TeslaService.STEP_CONNECT;
        TeslaService.next_action_time = new Date().getTime() + 30000; // 30 secondes
        TeslaChargingLogger.writeToLog(`⏱️ RECONNEXION: Planifiée dans 30s (${new Date(TeslaService.next_action_time).toISOString()})`);
    }
}

;

/**
 * Obtenir la liste des vehicles avec le proxy local
 */
static async getVehicleList() {
    TeslaChargingLogger.writeToLog("🚗 VÉHICULES: Démarrage récupération liste des véhicules");
    console.log("TeslaService: Récupération de la liste des véhicules");
    
    try {
        // Initialiser le proxy si nécessaire
        const TeslaProxy = require('./tesla.proxy');
        if (!TeslaProxy.CONFIG.ready) {
            TeslaChargingLogger.writeToLog("🔄 VÉHICULES: Proxy Tesla non initialisé, initialisation en cours");
            TeslaProxy.init();
            TeslaChargingLogger.writeToLog("✅ VÉHICULES: Proxy Tesla initialisé avec succès");
        } else {
            TeslaChargingLogger.writeToLog("✅ VÉHICULES: Proxy Tesla déjà initialisé");
        }
        
        // Récupérer l'URL exacte pour le débogage
        const requestUrl = TeslaProxy.CONFIG.api_path;
        TeslaChargingLogger.writeToLog(`📡 VÉHICULES: Requête liste via proxy local (URL: ${requestUrl})`);
        
        // Définir le prochain essai en cas d'échec
        const now = new Date().getTime();
        TeslaService.next_action_time = now + 10 * 1000; // 10 secondes
        TeslaChargingLogger.writeToLog(`⏱️ VÉHICULES: Prochain essai programmé dans 10s si échec (${new Date(TeslaService.next_action_time).toISOString()})`);
        
        // Appel via le proxy local
        TeslaChargingLogger.writeToLog("📡 VÉHICULES: Envoi requête API vers proxy local");
        const response = await TeslaProxy.request('GET', requestUrl);
        
        // Log détaillé de la réponse pour le débogage
        TeslaChargingLogger.writeToLog(`📥 VÉHICULES: Réponse reçue avec statut ${response.status}`);
        TeslaService.log(`getVehicleList Receive status ${response.status}`);
        
        if (response.status === 200) {
            // Log de la structure de données pour débogage
            TeslaChargingLogger.writeToLog("🔍 VÉHICULES: Analyse de la structure de données reçue");
            TeslaService.log("Vehicle list data structure: " + JSON.stringify(Object.keys(response.data)));
            
            let vehicle_list = response.data.response;
            const vehicleCount = Array.isArray(vehicle_list) ? vehicle_list.length : 0;
            TeslaChargingLogger.writeToLog(`🚗 VÉHICULES: ${vehicleCount} véhicules trouvés dans le compte Tesla`);
            
            if (Array.isArray(vehicle_list) && vehicle_list.length > 0) {
                // Log de la structure du premier véhicule
                TeslaChargingLogger.writeToLog(`🔍 VÉHICULES: Analyse du premier véhicule (${vehicle_list.length} au total)`);
                TeslaService.log("First vehicle structure: " + JSON.stringify(Object.keys(vehicle_list[0])));
                
                // Vérifier si un VIN est configuré, sinon prendre le premier véhicule
                const config = configuration_service_1.ConfigurationService.getConfiguration();
                const configuredVin = config.tesla.vin;
                
                if (configuredVin) {
                    TeslaChargingLogger.writeToLog(`🔍 VÉHICULES: Recherche du VIN configuré: ${configuredVin.substring(0, 5)}...${configuredVin.substring(configuredVin.length - 5)}`);
                } else {
                    TeslaChargingLogger.writeToLog("ℹ️ VÉHICULES: Aucun VIN configuré, utilisation du premier véhicule par défaut");
                }
                
                let selectedVehicle = vehicle_list[0]; // Par défaut, premier véhicule
                
                // Si un VIN est configuré, rechercher le véhicule correspondant
                if (configuredVin) {
                    const matchingVehicle = vehicle_list.find(v => v.vin === configuredVin);
                    if (matchingVehicle) {
                        selectedVehicle = matchingVehicle;
                        TeslaChargingLogger.writeToLog(`✅ VÉHICULES: Véhicule trouvé avec VIN configuré: ${configuredVin.substring(0, 5)}...${configuredVin.substring(configuredVin.length - 5)}`);
                    } else {
                        TeslaChargingLogger.writeToLog(`⚠️ VÉHICULES: VIN configuré ${configuredVin.substring(0, 5)}... non trouvé, utilisation du premier véhicule`);
                    }
                }
                
                // Utiliser id_s comme dans le test navigateur
                TeslaService.vehicle_id = selectedVehicle.id_s;
                TeslaChargingLogger.writeToLog(`🔑 VÉHICULES: ID véhicule défini à ${selectedVehicle.id_s}`);
                
                if (selectedVehicle.vin) {
                    TeslaChargingLogger.writeToLog(`🚗 VÉHICULES: VIN trouvé - ${selectedVehicle.vin.substring(0, 5)}...${selectedVehicle.vin.substring(selectedVehicle.vin.length - 5)}`);
                    
                    // Mettre à jour le VIN configuré si nécessaire
                    if (!configuredVin || configuredVin !== selectedVehicle.vin) {
                        const oldVin = configuredVin || 'non défini';
                        config.tesla.vin = selectedVehicle.vin;
                        
                        TeslaChargingLogger.writeToLog(`🔄 VÉHICULES: Mise à jour du VIN configuré: ${oldVin} -> ${selectedVehicle.vin}`);
                        configuration_service_1.ConfigurationService.saveConfiguration(config);
                        TeslaChargingLogger.writeToLog(`💾 VÉHICULES: Configuration sauvegardée avec nouveau VIN`);
                    } else {
                        TeslaChargingLogger.writeToLog(`✅ VÉHICULES: VIN déjà correctement configuré`);
                    }
                } else {
                    TeslaChargingLogger.writeToLog("❌ VÉHICULES: VIN manquant dans les données véhicule!");
                }
                
                // Stocker le VIN pour les futurs appels API
                if (!TeslaService.last_receive_data) {
                    TeslaChargingLogger.writeToLog("🔄 VÉHICULES: Création de l'objet de données car inexistant");
                    TeslaService.last_receive_data = {};
                }
                
                const oldVin = TeslaService.last_receive_data.vin;
                TeslaService.last_receive_data.vin = selectedVehicle.vin;
                TeslaService.last_receive_data.display_name = selectedVehicle.display_name;
                
                TeslaChargingLogger.writeToLog(`💾 VÉHICULES: Identifiants stockés - ID=${TeslaService.vehicle_id}, VIN=${selectedVehicle.vin || "inconnu"}, Nom="${selectedVehicle.display_name || "inconnu"}"`);
                
                // Toujours obtenir l'état du véhicule via GET_ONLINE
                const vehicleState = selectedVehicle.state || "unknown";
                TeslaChargingLogger.writeToLog(`🚗 VÉHICULES: État initial du véhicule: "${vehicleState}"`);

                // Stocker l'état pour référence future
                TeslaService.lastKnownVehicleState = vehicleState;
                TeslaService.lastVehicleStateCheck = now;
                TeslaChargingLogger.writeToLog(`💾 VÉHICULES: État et timestamp sauvegardés pour référence future`);
                
                // Toujours passer par GET_ONLINE
                if (TeslaService.mode === TeslaService.MODE_GET_DATA_NOW) {
                    if (vehicleState === "offline" || vehicleState === "asleep") {
                        TeslaChargingLogger.writeToLog(`🔄 VÉHICULES: Mode rafraîchissement avec véhicule ${vehicleState}, passage à l'étape réveil`);
                        TeslaService.step = TeslaService.STEP_WAKE_UP;
                        TeslaService.next_action_time = now; // Immédiat
                    } else {
                        TeslaChargingLogger.writeToLog(`🔄 VÉHICULES: Mode rafraîchissement avec véhicule ${vehicleState}, passage à la vérification d'état`);
                        TeslaService.step = TeslaService.STEP_GET_ONLINE;
                        TeslaService.next_action_time = now; // Immédiat
                    }
                } else {
                    TeslaChargingLogger.writeToLog(`🔄 VÉHICULES: Passage à la vérification d'état avant autre action`);
                    TeslaService.step = TeslaService.STEP_GET_ONLINE;
                    TeslaService.next_action_time = now + 1000; // Légère pause
                }
                
                TeslaChargingLogger.writeToLog(`⏱️ VÉHICULES: Prochaine étape=${TeslaService.getStepName(TeslaService.step)} à ${new Date(TeslaService.next_action_time).toISOString()}`);
            } else {
                TeslaChargingLogger.writeToLog("❌ VÉHICULES: Aucun véhicule trouvé dans la liste");
                console.error("Unexpected vehicle list structure:", JSON.stringify(vehicle_list || []).substring(0, 200));
                
                TeslaService.step = TeslaService.STEP_CONNECT;
                TeslaService.next_action_time = now + 10000;
                TeslaChargingLogger.writeToLog(`⏱️ VÉHICULES: Échec - Retour à l'étape connexion dans 10s (${new Date(TeslaService.next_action_time).toISOString()})`);
            }
        } else if (response.status === 401 || response.status === 404) {
            TeslaChargingLogger.writeToLog(`❌ VÉHICULES: Erreur d'autorisation ou d'URL (statut ${response.status})`);
            
            // Si erreur d'authentification, retourner à l'étape de connexion
            TeslaService.step = TeslaService.STEP_CONNECT;
            TeslaService.next_action_time = now; // Exécution immédiate
            TeslaChargingLogger.writeToLog(`🔄 VÉHICULES: Retour à l'étape connexion immédiatement`);
        } else {
            TeslaChargingLogger.writeToLog(`❌ VÉHICULES: Réponse API inattendue (statut ${response.status})`);
            
            // Pour autres erreurs, attendre avant de réessayer
            TeslaService.step = TeslaService.STEP_CONNECT;
            TeslaService.next_action_time = now + 30000; // 30 secondes
            TeslaChargingLogger.writeToLog(`⏱️ VÉHICULES: Nouvelle tentative dans 30s (${new Date(TeslaService.next_action_time).toISOString()})`);
        }
    } catch (error) {
        // Gestion des erreurs uniforme
        TeslaChargingLogger.writeToLog(`❌ VÉHICULES ERREUR: ${error.message || 'Erreur inconnue'}`);
        console.error('Error getting vehicle list via proxy');
        
        // Erreur liée à ConfigurationService?
        if (error.message && error.message.includes('ConfigurationService')) {
            TeslaChargingLogger.writeToLog("⚠️ VÉHICULES DIAGNOSTIC: Problème avec ConfigurationService - vérifiez l'importation");
            console.error("Stack trace:", error.stack);
        }
        
        // Problème de certificat?
        if (error.message && error.message.includes('self-signed certificate')) {
            TeslaChargingLogger.writeToLog("⚠️ VÉHICULES DIAGNOSTIC: Problème de certificat auto-signé - vérifier la configuration httpsAgent");
            console.error("Détails complets:", error);
        }
        
        // Log détaillé des erreurs HTTP
        if (error.response) {
            TeslaChargingLogger.writeToLog(`❌ VÉHICULES ERREUR HTTP: Statut=${error.response.status}`);
            try {
                const errorDataSample = typeof error.response.data === 'string' 
                    ? error.response.data.substring(0, 100) 
                    : JSON.stringify(error.response.data).substring(0, 100);
                TeslaChargingLogger.writeToLog(`❌ VÉHICULES DÉTAILS: ${errorDataSample}...`);
            } catch (logError) {
                TeslaChargingLogger.writeToLog(`❌ VÉHICULES: Impossible de formater les détails de l'erreur`);
            }
            console.error(`Vehicle list API response error: Status ${error.response.status}`);
            console.error('Response data:', JSON.stringify(error.response.data).substring(0, 200) + '...'); // Tronquer réponse longue
        } else if (error.request) {
            TeslaChargingLogger.writeToLog(`❌ VÉHICULES ERREUR RÉSEAU: Pas de réponse du serveur Tesla`);
            console.error('No response received from Tesla API for vehicle list');
        } else {
            TeslaChargingLogger.writeToLog(`❌ VÉHICULES ERREUR: ${error.message}`);
            console.error(`Vehicle list API request error: ${error.message}`);
        }
        
        // Planifier la prochaine tentative
        const now = new Date().getTime();
        TeslaService.step = TeslaService.STEP_CONNECT;
        TeslaService.next_action_time = now + 5000; // Réessayer dans 5 secondes
        TeslaChargingLogger.writeToLog(`⏱️ VÉHICULES: Nouvelle tentative de connexion dans 5s (${new Date(TeslaService.next_action_time).toISOString()})`);
    }
}

;

/**
 * Récupère les données véhicule actuelles avec priorité aux données MQTT si disponibles
 * @returns {Object} État actuel du véhicule avec source de données indiquée
 */
static getVehicleStatus() {
    TeslaChargingLogger.writeToLog("🚗 STATUS: Début récupération état véhicule");
    
    // Synchroniser les données MQTT et API pour garantir la cohérence
    this.syncDataSources();
    TeslaChargingLogger.writeToLog("🔄 STATUS: Données MQTT/API synchronisées");
    
    // Obtenir les données MQTT actuelles
    const mqttData = TeslaService.getMqttVehicleData();
    
    // Log des données MQTT trouvées ou non
    if (mqttData) {
        TeslaChargingLogger.writeToLog(`📡 STATUS MQTT: Données trouvées - Timestamp=${mqttData._timestamp || 'inconnu'}`);
        if (mqttData.charge_state) {
            TeslaChargingLogger.writeToLog(`🔋 STATUS MQTT: Batterie=${mqttData.charge_state.battery_level || 'N/A'}%, État=${mqttData.charge_state.charging_state || 'N/A'}, Ampérage=${mqttData.charge_state.charger_actual_current || 'N/A'}A, Tension=${mqttData.charge_state.charger_voltage || 'N/A'}V`);
        } else {
            TeslaChargingLogger.writeToLog(`⚠️ STATUS MQTT: Données de charge manquantes`);
        }
    } else {
        TeslaChargingLogger.writeToLog("⚠️ STATUS MQTT: Aucune donnée MQTT disponible");
    }
    
    // Obtenir les données API les plus récentes (déjà fusionnées avec MQTT via syncDataSources)
    const apiData = this.last_receive_data;
    
    // Log des données API trouvées ou non
    if (apiData && Object.keys(apiData).length > 0) {
        TeslaChargingLogger.writeToLog(`🔌 STATUS API: Données trouvées - Timestamp=${apiData._timestamp || apiData.timestampMs || 'inconnu'}`);
        if (apiData.charge_state) {
            TeslaChargingLogger.writeToLog(`🔋 STATUS API: Batterie=${apiData.charge_state.battery_level || 'N/A'}%, État=${apiData.charge_state.charging_state || 'N/A'}`);
        } else {
            TeslaChargingLogger.writeToLog(`⚠️ STATUS API: Données de charge manquantes`);
        }
    } else {
        TeslaChargingLogger.writeToLog("⚠️ STATUS API: Aucune donnée API disponible");
    }
    
    // Structure de base pour le statut
    const status = {
      battery_level: 0,
      charging_state: "Unknown",
      range_km: 0,
      inside_temp: 0,
      outside_temp: 0,
      is_climate_on: false,
      is_preconditioning: false,
      data_source: "unknown",
      timestamp: Date.now(),
      display_name: "Tesla"
    };
    
    TeslaChargingLogger.writeToLog("🔍 STATUS: Structure de base initialisée");
  
    // Fonction pour vérifier la fraîcheur des données (moins de 5 minutes = frais)
    const isFreshData = (data) => {
      if (!data || !data._timestamp) return false;
      const fiveMinutesAgo = Date.now() - 10 * 60 * 1000;
      return data._timestamp > fiveMinutesAgo;
    };
    
    // Vérifier la fraîcheur des données MQTT et API
    const mqttFresh = mqttData && isFreshData(mqttData);
    const apiFresh = apiData && isFreshData(apiData);
    
    TeslaChargingLogger.writeToLog(`⏱️ STATUS FRAÎCHEUR: MQTT=${mqttFresh ? 'récent' : 'obsolète'}, API=${apiFresh ? 'récent' : 'obsolète'}`);
  
    // Si les données MQTT sont disponibles et fraîches, utilisez-les en priorité
    if (mqttData && mqttFresh) {
      TeslaChargingLogger.writeToLog("📡 STATUS: Utilisation des données MQTT (temps réel)");
      console.log("INFO: Utilisation des données MQTT (temps réel)");
      
      // Extraction des données MQTT
      const batteryLevel = mqttData.charge_state?.usable_battery_level || 
                          mqttData.charge_state?.battery_level;
      const chargingState = mqttData.charge_state?.charging_state;
      const rangeKm = mqttData.charge_state?.est_battery_range_km;
      const insideTemp = mqttData.climate_state?.inside_temp;
      const outsideTemp = mqttData.climate_state?.outside_temp;
      const isClimateOn = mqttData.climate_state?.is_climate_on;
      const isPreconditioning = mqttData.climate_state?.is_preconditioning;
      const displayName = mqttData.vehicle_state?.vehicle_name || mqttData.display_name;
      
      // Log des valeurs extraites
      TeslaChargingLogger.writeToLog(`🔋 STATUS MQTT VALEURS: Batterie=${batteryLevel || 'N/A'}%, Charge=${chargingState || 'N/A'}, Autonomie=${rangeKm || 'N/A'}km`);
      TeslaChargingLogger.writeToLog(`🌡️ STATUS MQTT CLIMAT: Int=${insideTemp || 'N/A'}°C, Ext=${outsideTemp || 'N/A'}°C, Actif=${isClimateOn ? 'Oui' : 'Non'}`);
      
      // Conversion du format MQTT au format status
      status.battery_level = batteryLevel || 0;
      status.charging_state = chargingState || "Unknown";
      status.range_km = rangeKm || 0;
      status.inside_temp = insideTemp || 0;
      status.outside_temp = outsideTemp || 0;
      status.is_climate_on = isClimateOn || false;
      status.is_preconditioning = isPreconditioning || false;
      status.data_source = "mqtt";
      status.timestamp = mqttData._timestamp || Date.now();
      status.display_name = displayName || "Tesla";
    } 
    // Sinon, utilisez les données API/synchronisées si elles sont fraîches
    else if (apiData && apiFresh) {
      TeslaChargingLogger.writeToLog("🔌 STATUS: Utilisation des données synchronisées");
      console.log("INFO: Utilisation des données synchronisées API/MQTT");
      
      // Extraction des données API/synchronisées
      const batteryLevel = apiData.charge_state?.usable_battery_level || 
                          apiData.charge_state?.battery_level;
      const chargingState = apiData.charge_state?.charging_state;
      const rangeKm = apiData.charge_state?.est_battery_range_km;
      const insideTemp = apiData.climate_state?.inside_temp;
      const outsideTemp = apiData.climate_state?.outside_temp;
      const isClimateOn = apiData.climate_state?.is_climate_on;
      const isPreconditioning = apiData.climate_state?.is_preconditioning;
      const displayName = apiData.vehicle_state?.vehicle_name || apiData.display_name;
      
      // Log des valeurs extraites
      TeslaChargingLogger.writeToLog(`🔋 STATUS API VALEURS: Batterie=${batteryLevel || 'N/A'}%, Charge=${chargingState || 'N/A'}, Autonomie=${rangeKm || 'N/A'}km`);
      TeslaChargingLogger.writeToLog(`🌡️ STATUS API CLIMAT: Int=${insideTemp || 'N/A'}°C, Ext=${outsideTemp || 'N/A'}°C, Actif=${isClimateOn ? 'Oui' : 'Non'}`);
      
      // Conversion du format API au format status
      status.battery_level = batteryLevel || 0;
      status.charging_state = chargingState || "Unknown";
      status.range_km = rangeKm || 0;
      status.inside_temp = insideTemp || 0;
      status.outside_temp = outsideTemp || 0;
      status.is_climate_on = isClimateOn || false;
      status.is_preconditioning = isPreconditioning || false;
      status.data_source = apiData._source || "api";
      status.timestamp = apiData._timestamp || apiData.timestampMs || Date.now();
      status.display_name = displayName || "Tesla";
    } else {
      // Aucune donnée fraîche disponible
      TeslaChargingLogger.writeToLog("⚠️ STATUS: Aucune donnée fraîche disponible, utilisation des valeurs par défaut");
      
      // Tenter d'extraire quand même quelques données de base même si non fraîches
      if (mqttData || apiData) {
        const bestData = mqttData || apiData;
        status.battery_level = bestData.charge_state?.battery_level || 0;
        status.charging_state = bestData.charge_state?.charging_state || "Unknown";
        status.data_source = "stale";
        
        TeslaChargingLogger.writeToLog(`⚠️ STATUS DONNÉES OBSOLÈTES: Batterie=${status.battery_level}%, État=${status.charging_state}`);
      }
    }
    
    // Ajouter le mode de charge actuel
    status.chargingMode = TeslaService.chargingMode;
    TeslaChargingLogger.writeToLog(`🔌 STATUS MODE: Mode de charge actuel "${status.chargingMode}"`);
    
    // Vérifier la complétude des données importantes
    const missingData = [];
    if (status.battery_level === 0) missingData.push("niveau batterie");
    if (status.charging_state === "Unknown") missingData.push("état de charge");
    
    if (missingData.length > 0) {
      TeslaChargingLogger.writeToLog(`⚠️ STATUS INCOMPLET: Données manquantes: ${missingData.join(", ")}`);
    } else {
      TeslaChargingLogger.writeToLog("✅ STATUS: Données complètes récupérées avec succès");
    }
    
    // Diagnostic
    console.log(`TeslaService: Statut véhicule - Source=${status.data_source}, Batterie=${status.battery_level}%, Charge=${status.charging_state}`);
    
    TeslaChargingLogger.writeToLog("🚗 STATUS: Fin récupération état véhicule");
    return status;
  }

;

/**
 * Vérifie si le véhicule est en ligne via une requête légère au proxy local
 * @returns {Promise<string>} État du véhicule ('online', 'asleep', 'offline', etc.)
 */
static async checkVehicleOnlineStatus() {
    const now = new Date().getTime();
    TeslaChargingLogger.writeToLog("🔍 ÉTAT VÉHICULE: Début vérification état en ligne");
    
    // Utiliser le cache si récent (moins de 30 secondes)
    if (TeslaService.lastVehicleStateCheck && now - TeslaService.lastVehicleStateCheck < this.TIMEOUTS.STATE_CACHE_TTL) {
        const cachedState = TeslaService.lastKnownVehicleState || 'unknown';
        const cacheAge = Math.round((now - TeslaService.lastVehicleStateCheck)/1000);
        TeslaChargingLogger.writeToLog(`🔍 ÉTAT VÉHICULE: Utilisation du cache (${cachedState}, âge: ${cacheAge}s)`);
        console.log(`TeslaService: État véhicule depuis cache: ${cachedState} (âge: ${cacheAge}s)`);
        
        // GESTION SPÉCIFIQUE EN MODE DE RAFRAÎCHISSEMENT MANUEL
        if (TeslaService.mode === TeslaService.MODE_GET_DATA_NOW) {
            // CORRECTION: Gérer à la fois l'état online et offline/asleep
            if (cachedState === "online") {
                TeslaChargingLogger.writeToLog("✅ ÉTAT VÉHICULE: Cache indique véhicule en ligne, passage à la récupération des données");
                TeslaService.step = TeslaService.STEP_GET_DATA;
                TeslaService.next_action_time = now;
                TeslaChargingLogger.writeToLog(`⏱️ ÉTAT VÉHICULE: Prochaine étape=${TeslaService.getStepName(TeslaService.step)} immédiate`);
            } 
            else if (cachedState === "asleep" || cachedState === "offline") {
                TeslaChargingLogger.writeToLog("⏰ ÉTAT VÉHICULE: Cache indique véhicule endormi/offline, passage direct à l'étape réveil");
                TeslaService.step = TeslaService.STEP_WAKE_UP;
                TeslaService.next_action_time = now;
                TeslaChargingLogger.writeToLog(`⏱️ ÉTAT VÉHICULE: Prochaine étape=${TeslaService.getStepName(TeslaService.step)} immédiate`);
            }
        }
        
        return cachedState;
    }
    
    TeslaChargingLogger.writeToLog("🔄 ÉTAT VÉHICULE: Cache expiré ou absent, requête API nécessaire");
    
    try {
        // Initialiser le proxy si nécessaire
        const TeslaProxy = require('./tesla.proxy');
        if (!TeslaProxy.CONFIG.ready) {
            TeslaChargingLogger.writeToLog("🔄 ÉTAT VÉHICULE: Proxy Tesla non initialisé, initialisation en cours");
            TeslaProxy.init();
            TeslaChargingLogger.writeToLog("✅ ÉTAT VÉHICULE: Proxy Tesla initialisé avec succès");
        } else {
            TeslaChargingLogger.writeToLog("✅ ÉTAT VÉHICULE: Proxy Tesla déjà initialisé");
        }
        
        // Récupérer l'identifiant du véhicule
        const vehicleIdOrVin = TeslaService.last_receive_data?.vin || TeslaService.vehicle_id;
        
        if (!vehicleIdOrVin) {
            TeslaChargingLogger.writeToLog("❌ ÉTAT VÉHICULE: Identifiant véhicule manquant, impossible de vérifier l'état");
            throw new Error("Identifiant véhicule manquant");
        }
        
        // Endpoint pour vérifier l'état du véhicule (liste des véhicules filtrée)
        const endpoint = `/api/1/vehicles/${vehicleIdOrVin}`;
        TeslaChargingLogger.writeToLog(`📡 ÉTAT VÉHICULE: Requête API via endpoint ${endpoint}`);
        
        // Requête légère pour vérifier l'état avec timeout court
        const startRequestTime = Date.now();
        TeslaChargingLogger.writeToLog("📡 ÉTAT VÉHICULE: Envoi requête API vers proxy local");
        const response = await TeslaProxy.request('GET', endpoint, null, { shortTimeout: true });
        
        // Log de performance pour cette requête légère
        const requestDuration = Date.now() - startRequestTime;
        TeslaChargingLogger.writeToLog(`⏱️ ÉTAT VÉHICULE: Réponse API reçue en ${requestDuration}ms`);
        
        // Analyse de la réponse
        if (response.status === 200 && response.data?.response) {
            const state = response.data.response.state;
            
            // Mise à jour du cache d'état
            TeslaService.lastKnownVehicleState = state;
            TeslaService.lastVehicleStateCheck = now;
            
            TeslaChargingLogger.writeToLog(`🚗 ÉTAT VÉHICULE: État API "${state}", cache mis à jour`);
            console.log(`TeslaService: État véhicule depuis API: ${state}`);
            
            // GESTION SPÉCIFIQUE EN MODE DE RAFRAÎCHISSEMENT MANUEL
            if (TeslaService.mode === TeslaService.MODE_GET_DATA_NOW) {
                // CORRECTION: Gérer à la fois l'état online et offline/asleep
                if (state === "online") {
                    TeslaChargingLogger.writeToLog("✅ ÉTAT VÉHICULE: Véhicule en ligne, passage à la récupération des données");
                    TeslaService.step = TeslaService.STEP_GET_DATA;
                    TeslaService.next_action_time = now;
                    TeslaChargingLogger.writeToLog(`⏱️ ÉTAT VÉHICULE: Prochaine étape=${TeslaService.getStepName(TeslaService.step)} immédiate`);
                } 
                else if (state === "asleep" || state === "offline") {
                    TeslaChargingLogger.writeToLog("⏰ ÉTAT VÉHICULE: Véhicule endormi/offline, passage direct à l'étape réveil");
                    TeslaService.step = TeslaService.STEP_WAKE_UP;
                    TeslaService.next_action_time = now;
                    TeslaChargingLogger.writeToLog(`⏱️ ÉTAT VÉHICULE: Prochaine étape=${TeslaService.getStepName(TeslaService.step)} immédiate`);
                }
            }
            
            return state;
        } else {
            TeslaChargingLogger.writeToLog(`❌ ÉTAT VÉHICULE: Réponse API inattendue (statut ${response.status})`);
                        
            // Examiner la structure de la réponse pour diagnostic
            try {
                const responseKeys = Object.keys(response.data || {}).join(', ');
                TeslaChargingLogger.writeToLog(`❌ ÉTAT VÉHICULE: Structure réponse - clés: ${responseKeys}`);
            } catch (logError) {
                TeslaChargingLogger.writeToLog(`❌ ÉTAT VÉHICULE: Impossible d'analyser la structure de réponse`);
            }
            
            return 'unknown';
        }
    } catch (error) {
        // En cas d'erreur 408, c'est que le véhicule est endormi
        if (error.response && error.response.status === 408) {
            TeslaService.lastKnownVehicleState = 'asleep';
            TeslaService.lastVehicleStateCheck = now;
            
            TeslaChargingLogger.writeToLog("😴 ÉTAT VÉHICULE: Véhicule endormi détecté (code 408)");
            console.log("TeslaService: Véhicule détecté comme endormi (408)");
            
            // NOUVELLE CORRECTION: En mode rafraîchissement manuel, passer directement au réveil
            if (TeslaService.mode === TeslaService.MODE_GET_DATA_NOW) {
                TeslaChargingLogger.writeToLog("⏰ ÉTAT VÉHICULE: Mode rafraîchissement actif, passage direct à l'étape réveil");
                TeslaService.step = TeslaService.STEP_WAKE_UP;
                TeslaService.next_action_time = now;
                TeslaChargingLogger.writeToLog(`⏱️ ÉTAT VÉHICULE: Prochaine étape=${TeslaService.getStepName(TeslaService.step)} immédiate`);
            }
            
            return 'asleep';
        }
        
        // NOUVELLE GESTION: Pour les erreurs 502 ou autres erreurs serveur, tenter une nouvelle fois
        if (error.response && (error.response.status === 502 || error.response.status === 503 || error.response.status === 504)) {
            const errorMessage = error.message || "Erreur inconnue";
            const errorStatus = error.response.status || "N/A";
            
            TeslaChargingLogger.writeToLog(`⚠️ ÉTAT VÉHICULE: Erreur ${errorStatus} détectée, tentative de récupération`);
            TeslaChargingLogger.writeToLog(`⏱️ ÉTAT VÉHICULE: Attente de 3 secondes avant nouvelle tentative`);
            
            // Attendre 3 secondes avant de réessayer
            await new Promise(resolve => setTimeout(resolve, 3000));
            
            try {
                // Vérifier si des données MQTT fraîches sont disponibles
                
                if (TeslaMqttClient?.isConnected && TeslaMqttClient.isDataFresh(30000)) {  // 30 secondes
                    const mqttData = TeslaService.getMqttVehicleData();
                    if (mqttData?.vehicle_state?.state || mqttData?.state) {
                        const mqttState = mqttData?.vehicle_state?.state || mqttData?.state;
                        TeslaChargingLogger.writeToLog(`📡 ÉTAT VÉHICULE: Récupération via MQTT après erreur API - état: ${mqttState}`);
                        
                        // Mise à jour du cache d'état
                        TeslaService.lastKnownVehicleState = mqttState;
                        TeslaService.lastVehicleStateCheck = now;
                        
                        // Si la voiture est en charge selon MQTT, marquer comme "online"
                        if (mqttData?.charge_state?.charging_state === "Charging") {
                            TeslaChargingLogger.writeToLog(`🔌 ÉTAT VÉHICULE: Véhicule en charge selon MQTT, marqué comme "online"`);
                            TeslaService.lastKnownVehicleState = "online";
                        }
                        
                        return TeslaService.lastKnownVehicleState;
                    }
                }
                
                // Seconde tentative API avec le proxy
                TeslaChargingLogger.writeToLog("🔄 ÉTAT VÉHICULE: Seconde tentative d'appel API après échec");
                
                // Même code que précédemment pour la requête
                const response = await TeslaProxy.request('GET', endpoint, null, { shortTimeout: true });
                
                if (response.status === 200 && response.data?.response) {
                    const state = response.data.response.state;
                    TeslaChargingLogger.writeToLog(`✅ ÉTAT VÉHICULE: Seconde tentative réussie, état: "${state}"`);
                    
                    // Mise à jour du cache d'état
                    TeslaService.lastKnownVehicleState = state;
                    TeslaService.lastVehicleStateCheck = now;
                    
                    return state;
                }
            } catch (retryError) {
                // Si la seconde tentative échoue aussi, logger l'erreur
                TeslaChargingLogger.writeToLog(`❌ ÉTAT VÉHICULE: Échec de la seconde tentative - ${retryError.message}`);
            }
        }
        
        // En cas d'erreur d'authentification
        if (error.response && error.response.status === 401) {
            TeslaChargingLogger.writeToLog("🔑 ÉTAT VÉHICULE: Token expiré, reconnexion nécessaire");
            console.error("TeslaService: Erreur d'authentification lors de la vérification d'état");
            
            // Forcer le rafraîchissement du token
            const TeslaProxy = require('./tesla.proxy');
            TeslaProxy.CONFIG.auth.accessToken = null;
            
            // Planifier une reconnexion
            TeslaService.step = TeslaService.STEP_CONNECT;
            TeslaService.next_action_time = now + 1000; // 1 seconde
            TeslaChargingLogger.writeToLog(`⏱️ ÉTAT VÉHICULE: Reconnexion planifiée dans 1s (${new Date(TeslaService.next_action_time).toISOString()})`);
        }
        
        // Log complet de l'erreur
        const errorMessage = error.message || "Erreur inconnue";
        const errorStatus = error.response?.status || "N/A";
        
        TeslaChargingLogger.writeToLog(`❌ ÉTAT VÉHICULE: Erreur lors de la vérification - ${errorMessage} (Statut=${errorStatus})`);
                
        // Log détaillé de l'erreur pour le diagnostic
        if (error.code === 'ECONNREFUSED' || error.code === 'ECONNRESET') {
            TeslaChargingLogger.writeToLog(`🌐 ÉTAT VÉHICULE: Problème de connexion réseau - ${error.code}`);
        } else if (error.code === 'ETIMEDOUT') {
            TeslaChargingLogger.writeToLog(`⏱️ ÉTAT VÉHICULE: Délai d'attente dépassé - ${error.code}`);
        }
        
        return 'error';
    }
}

;
  
/**
 * Réveille le véhicule via BLE si disponible, sinon via le proxy local
 * @returns {Promise<Object|null>} Données du véhicule si réveillé avec succès, sinon null
 */
static async wakeUp() {
    const now = new Date().getTime();
    TeslaChargingLogger.writeToLog("🚗 RÉVEIL: Début tentative de réveil du véhicule");
    
    // PROTECTION: Verrou d'exécution
    if (TeslaService.wakeupInProgress) {
        TeslaChargingLogger.writeToLog("⚠️ RÉVEIL: Une tentative est déjà en cours, ignorée");
        return null;
    }
    
    // PROTECTION: Anti-rebond
    if (TeslaService.lastWakeupCall && now - TeslaService.lastWakeupCall < 60000) {
        const timeSinceLastCall = Math.round((now - TeslaService.lastWakeupCall)/1000);
        TeslaChargingLogger.writeToLog(`⚠️ RÉVEIL: Tentative ignorée (trop rapprochée, dernière il y a ${timeSinceLastCall}s)`);
        console.log(`TeslaService: Tentative de réveil ignorée (dernière il y a ${timeSinceLastCall}s)`);
        return null;
    }
    
    // Activer le verrou
    TeslaService.wakeupInProgress = true;
    TeslaService.lastWakeupCall = now;
    TeslaChargingLogger.writeToLog("🔒 RÉVEIL: Verrou d'exécution activé");
    
    // NOUVELLE SÉCURITÉ: Timer de secours qui libère le verrou dans tous les cas après 30s
    const safetyTimeout = setTimeout(() => {
        if (TeslaService.wakeupInProgress) {
            TeslaChargingLogger.writeToLog("🔓 RÉVEIL: Libération d'urgence du verrou wake_up après timeout (30s)");
            TeslaService.wakeupInProgress = false;
        }
    }, 30000);
    TeslaChargingLogger.writeToLog("⏱️ RÉVEIL: Timer de sécurité activé (30s)");
    
    try {
        // NOUVEAU: Vérifier si BLE est activé dans la configuration
        const config = configuration_service_1.ConfigurationService.getConfiguration();
        const bleEnabled = config?.tesla?.useBle;
        
        // Si BLE est activé, essayer d'abord le réveil via BLE
        if (bleEnabled) {
            try {
                TeslaChargingLogger.writeToLog("🚗 RÉVEIL: Tentative de réveil via BLE");
                const { TeslaControlBLE } = require('./tesla.ble.service');
                
                // Vérifier si BLE est correctement initialisé
                if (TeslaControlBLE && TeslaControlBLE.config?.vin) {
                    // Tenter le réveil BLE
                    TeslaChargingLogger.writeToLog("🚗 RÉVEIL: Envoi commande wake via BLE");
                    const bleResult = await TeslaControlBLE.wake();
                    
                    if (bleResult) {
                        TeslaChargingLogger.writeToLog("✅ RÉVEIL: Commande wake BLE envoyée avec succès, vérification de l'état");
                        
                        // Attente courte pour laisser la voiture se réveiller
                        TeslaChargingLogger.writeToLog("⏱️ RÉVEIL BLE: Attente de 5 secondes pour activation");
                        await new Promise(resolve => setTimeout(resolve, 5000));
                        
                        // Vérifier si le véhicule est réveillé
                        const isAwake = await TeslaControlBLE.isVehicleAwake();
                        if (isAwake) {
                            TeslaChargingLogger.writeToLog("✅ RÉVEIL: Véhicule réveillé avec succès via BLE");
                            
                            TeslaService.bleConfirmedAwake = Date.now();
                            TeslaChargingLogger.writeToLog("🚀 RÉVEIL: État 'online' confirmé par BLE, ignorera MQTT pendant 60 secondes");

                            // AJOUT D'UN DÉLAI CRITIQUE DE SYNCHRONISATION
                            TeslaChargingLogger.writeToLog("⏰ RÉVEIL: Attente de 09 secondes pour synchronisation avec MQTT");
                            await new Promise(resolve => setTimeout(resolve, 9000)); // 09 secondes
                            TeslaChargingLogger.writeToLog("⏰ RÉVEIL: Fin de l'attente de synchronisation");    

                            // Mise à jour de l'état connu
                            TeslaService.lastKnownVehicleState = "online";
                            TeslaService.lastVehicleStateCheck = now;
                            
                            // Passer à l'étape suivante
                            TeslaService.step = TeslaService.STEP_GET_DATA;
                            TeslaService.next_action_time = now;
                            TeslaChargingLogger.writeToLog("🔄 RÉVEIL: Passage direct à la récupération des données");
                            
                            // Créer et retourner un objet compatible API Tesla
                            const vehicleData = {
                                state: "online",
                                wake_via: "ble",
                                timestamp: now
                            };
                            
                            return vehicleData;
                        } else {
                            TeslaChargingLogger.writeToLog("⚠️ RÉVEIL: Commande BLE envoyée mais véhicule toujours endormi, passage à la méthode API");
                        }
                    } else {
                        TeslaChargingLogger.writeToLog("⚠️ RÉVEIL: Échec du réveil via BLE, passage à la méthode API");
                    }
                } else {
                    TeslaChargingLogger.writeToLog("⚠️ RÉVEIL: BLE activé mais non configuré correctement, utilisation API");
                }
            } catch (bleError) {
                TeslaChargingLogger.writeToLog(`⚠️ RÉVEIL: Erreur BLE - ${bleError.message}, utilisation de l'API comme secours`);
            }
        }
        
        // Si le réveil BLE a échoué ou BLE n'est pas activé, utiliser l'API
        TeslaChargingLogger.writeToLog("🚗 RÉVEIL: Envoi de la commande wake_up via API");
        
        // Initialiser le proxy si nécessaire
        const TeslaProxy = require('./tesla.proxy');
        if (!TeslaProxy.CONFIG.ready) {
            TeslaChargingLogger.writeToLog("🔄 RÉVEIL: Proxy non initialisé, initialisation en cours");
            TeslaProxy.init();
            TeslaChargingLogger.writeToLog("✅ RÉVEIL: Proxy initialisé avec succès");
        } else {
            TeslaChargingLogger.writeToLog("✅ RÉVEIL: Proxy déjà initialisé");
        }
        
        // Récupérer l'identifiant du véhicule
        const vehicleIdOrVin = TeslaService.last_receive_data?.vin || TeslaService.vehicle_id;
        
        if (!vehicleIdOrVin) {
            TeslaChargingLogger.writeToLog("❌ RÉVEIL: Identifiant véhicule manquant, impossible d'envoyer la commande");
            throw new Error("Identifiant véhicule manquant");
        }
        
        // Construction de l'endpoint pour le réveil
        const endpoint = `/api/1/vehicles/${vehicleIdOrVin}/wake_up`;
        TeslaChargingLogger.writeToLog(`🚗 RÉVEIL: Endpoint construit: ${endpoint}`);
        
        // Envoi de la commande de réveil via le proxy
        TeslaChargingLogger.writeToLog(`🚗 RÉVEIL: Tentative pour véhicule ${vehicleIdOrVin.substring(0, 3)}...${vehicleIdOrVin.substring(vehicleIdOrVin.length - 3)}`);
        
        // Mesure du temps de réponse pour diagnostic
        const requestStartTime = Date.now();
        const response = await TeslaProxy.request('POST', endpoint, {}, { jsonContent: true });
        const requestDuration = Date.now() - requestStartTime;
        
        // Log de performance
        TeslaChargingLogger.writeToLog(`📡 RÉVEIL: Réponse reçue en ${requestDuration}ms (statut ${response.status})`);
        
        // Traitement de la réponse
        if (response.status === 200) {
            const vehicleData = response.data.response;
            const state = vehicleData.state;
            
            // Log explicite de l'état reçu
            TeslaChargingLogger.writeToLog(`🚗 RÉVEIL: État actuel "${state}"`);
            console.log(`TeslaService: État véhicule après wake_up: ${state}`);
            
            // Mise à jour de l'état connu
            TeslaService.lastKnownVehicleState = state;
            TeslaService.lastVehicleStateCheck = now;
            TeslaChargingLogger.writeToLog(`💾 RÉVEIL: État mis à jour dans le cache`);
            
            // ATTENTE OBLIGATOIRE après wake_up réussi, AVANT de poursuivre
            TeslaChargingLogger.writeToLog(`⏱️ RÉVEIL: Attente obligatoire de 30 secondes après wake_up API`);
            await new Promise(resolve => setTimeout(resolve, 30000));
            TeslaChargingLogger.writeToLog(`⏱️ RÉVEIL: Fin de l'attente obligatoire après wake_up API`);
            
            if (state === "online") {
                TeslaChargingLogger.writeToLog("✅ RÉVEIL: Véhicule en ligne, succès immédiat");
                TeslaService.step = TeslaService.STEP_GET_DATA;
                TeslaService.next_action_time = now;
                TeslaChargingLogger.writeToLog(`⏱️ RÉVEIL: Passage direct à la récupération des données`);
                
                return vehicleData;
            } 
            else if (state === "waking" || state === "waking_up") {
                TeslaChargingLogger.writeToLog("⏳ RÉVEIL: En cours (état 'waking'), nouvelle vérification dans 5s");
                TeslaService.step = TeslaService.STEP_WAKE_UP;
                TeslaService.next_action_time = now + 5000;
                TeslaChargingLogger.writeToLog(`⏱️ RÉVEIL: Prochaine vérification programmée à ${new Date(TeslaService.next_action_time).toISOString()}`);
                return vehicleData;
            } 
            else {
                // Autres états: offline, asleep, etc.
                TeslaChargingLogger.writeToLog(`⏳ RÉVEIL: État "${state}" non optimal, nouvelle tentative dans 5s`);
                TeslaService.step = TeslaService.STEP_WAKE_UP;
                TeslaService.next_action_time = now + 5000;
                TeslaChargingLogger.writeToLog(`⏱️ RÉVEIL: Prochaine tentative programmée à ${new Date(TeslaService.next_action_time).toISOString()}`);
                return null;
            }
        } else {
            TeslaChargingLogger.writeToLog(`❌ RÉVEIL: Réponse inattendue (${response.status})`);
            console.error(`TeslaService: Réponse inattendue lors du réveil: ${response.status}`);
            
            // Planification d'une nouvelle tentative
            TeslaService.step = TeslaService.STEP_WAKE_UP;
            TeslaService.next_action_time = now + 15000; // 15 secondes
            TeslaChargingLogger.writeToLog(`⏱️ RÉVEIL: Nouvelle tentative dans 15s (${new Date(TeslaService.next_action_time).toISOString()})`);
            return null;
        }
    } catch (error) {
        // Log d'erreur sécurisé
        console.error("WAKE_UP ERROR:", error.message || "Unknown error");
        TeslaChargingLogger.writeToLog(`❌ RÉVEIL ERREUR: ${error.message || "Erreur inconnue"}`);
        
        // Planifier une nouvelle tentative
        TeslaService.step = TeslaService.STEP_WAKE_UP;
        TeslaService.next_action_time = now + 15000; // 15 secondes
        TeslaChargingLogger.writeToLog(`⏱️ RÉVEIL: Nouvelle tentative dans 15s (${new Date(TeslaService.next_action_time).toISOString()})`);
        
        return null;
    } finally {
        // Annuler le timeout de sécurité
        clearTimeout(safetyTimeout);
        TeslaChargingLogger.writeToLog("⏱️ RÉVEIL: Timer de sécurité désactivé");
        
        // CORRECTION CRITIQUE: Libérer immédiatement le verrou en cas d'erreur
        TeslaService.wakeupInProgress = false;
        TeslaChargingLogger.writeToLog("🔓 RÉVEIL: Libération immédiate du verrou wake_up");
    }
}

    ;


/**
 * Calcule l'intervalle optimal de sondage basé sur l'état du véhicule
 * @returns {number} Intervalle en milliseconds jusqu'au prochain sondage
 */
static getNextPollInterval() {
    TeslaChargingLogger.writeToLog("⏱️ INTERVALLE: Calcul de l'intervalle optimal de sondage");
    
    // NOUVEAU: Si les données MQTT sont fraîches, utiliser un intervalle plus long
    const mqttData = TeslaService.getMqttVehicleData();
    const mqttFresh = mqttData && mqttData._timestamp && 
                    (Date.now() - mqttData._timestamp < 10 * 60 * 1000);
    
    if (mqttFresh && TeslaMqttClient?.isConnected) {
        TeslaChargingLogger.writeToLog("⏱️ INTERVALLE: Données MQTT fraîches, intervalle très long (10 min)");
        return 10 * 60 * 1000; // 10 minutes si MQTT est frais
    }
    
    if (TeslaService.chargingMode === 'normal') {
        // En mode normal, sondage très peu fréquent (30 minutes)
        TeslaChargingLogger.writeToLog("⏱️ INTERVALLE: Mode 'normal' détecté, intervalle long (30 min)");
        return 30 * 60 * 1000;
    }
    
    // Si pas encore de données, utiliser l'intervalle par défaut
    if (!TeslaService.last_receive_data || !TeslaService.last_receive_data.drive_state) {
        TeslaChargingLogger.writeToLog(`⏱️ INTERVALLE: Données véhicule insuffisantes, intervalle par défaut (${TeslaService.POLLING_DEFAULT_INTERVAL/1000}s)`);
        return TeslaService.POLLING_DEFAULT_INTERVAL;
    }
    
    // Mode continu - intervalle court
    if (TeslaService.mode === TeslaService.MODE_GET_CONTINUS_DATA) {
        TeslaChargingLogger.writeToLog("⏱️ INTERVALLE: Mode continu actif, intervalle court (15s)");
        return 15 * 1000; // 15 secondes
    }
    
    // En fonction de l'état actuel du véhicule
    if (TeslaService.last_receive_data.drive_state.shift_state) {
        // Voiture en mouvement
        TeslaChargingLogger.writeToLog(`⏱️ INTERVALLE: Véhicule en mouvement (${TeslaService.last_receive_data.drive_state.shift_state}), intervalle conduite (${TeslaService.POLLING_DRIVING_INTERVAL/1000}s)`);
        return TeslaService.POLLING_DRIVING_INTERVAL;
    } else if (TeslaService.last_receive_data.charge_state?.charging_state === "Charging") {
        // Voiture en charge - code existant
    } else if (TeslaService.lastKnownVehicleState === "asleep") {
        // Voiture endormie
        TeslaChargingLogger.writeToLog(`⏱️ INTERVALLE: Véhicule endormi, intervalle sommeil (${TeslaService.POLLING_ASLEEP_INTERVAL/1000}s)`);
        return TeslaService.POLLING_ASLEEP_INTERVAL;
    } else {
        // MODIFIÉ: Voiture stationnée/en ligne - augmenter l'intervalle à 3 minutes minimum
        const newInterval = Math.max(TeslaService.POLLING_ONLINE_INTERVAL, 3 * 60 * 1000);
        const intervalSeconds = newInterval/1000;
        TeslaChargingLogger.writeToLog(`⏱️ INTERVALLE: Véhicule stationné/en ligne (${TeslaService.lastKnownVehicleState || 'état inconnu'}), intervalle optimisé (${intervalSeconds}s)`);
        return newInterval;
    }
}

/**
 * Détecte et gère les déconnexions de session Tesla
 * @param {Error} error - L'erreur à analyser
 * @returns {boolean} true si une erreur de session a été détectée et gérée
 */
static handleSessionTimeout(error) {
    // Log d'entrée pour diagnostic
    TeslaChargingLogger.writeToLog("🔍 SESSION: Analyse d'erreur pour détecter expiration session");
    
    // Vérification de la structure de l'erreur
    const hasErrorResponse = !!(error && error.response);
    const hasErrorData = !!(hasErrorResponse && error.response.data);
    
    TeslaChargingLogger.writeToLog(`🔍 SESSION: Structure erreur - hasResponse=${hasErrorResponse}, hasData=${hasErrorData}`);
    
    // Vérifier si l'erreur est une déconnexion
    if (hasErrorData && 
        ((typeof error.response.data === 'string' && error.response.data.includes("logged out")) || 
         (typeof error.response.data === 'object' && error.response.data.error && 
          error.response.data.error.includes("logged out")))) {
        
        // Extraction de détails pour diagnostic
        const errorStatus = error.response.status || 'inconnu';
        const errorDetails = typeof error.response.data === 'string' 
            ? error.response.data.substring(0, 100) 
            : (error.response.data.error || 'données objet').substring(0, 100);
        
        TeslaChargingLogger.writeToLog(`⚠️ SESSION EXPIRÉE: Statut=${errorStatus}, Détails="${errorDetails}..."`);
        console.error(`TeslaService: Session expirée détectée - Code ${errorStatus}`);
        
        const now = new Date().getTime();
        
        // Limiter les notifications à une par heure
        if (!TeslaService.sessionTimedOut || now - TeslaService.lastSessionTimeoutNotification > 60 * 60 * 1000) {
            // Déterminer le temps écoulé depuis la dernière notification
            let timeSinceLastNotification = "première détection";
            if (TeslaService.lastSessionTimeoutNotification) {
                const minutes = Math.round((now - TeslaService.lastSessionTimeoutNotification) / (60 * 1000));
                timeSinceLastNotification = `${minutes} minutes depuis dernière alerte`;
            }
            
            TeslaChargingLogger.writeToLog(`🚨 SESSION ALERTE: Émission notification (${timeSinceLastNotification})`);
            console.error(`⚠️ VOTRE APPLICATION TESLA EST EXPIRÉE - Veuillez vous reconnecter à https://developer.tesla.com`);
            
            // Mettre à jour l'état de la session
            TeslaService.sessionTimedOut = true;
            TeslaService.lastSessionTimeoutNotification = now;
            
            // Réduire les tentatives d'API pendant la déconnexion
            const nextRetryTime = new Date(now + 15 * 60 * 1000);
            TeslaService.next_action_time = now + 15 * 60 * 1000; // 15 minutes
            
            TeslaChargingLogger.writeToLog(`⏱️ SESSION: Prochaine tentative retardée à ${nextRetryTime.toISOString()} (15 minutes)`);
            console.log(`TeslaService: Réduction fréquence d'appels API - prochaine tentative dans 15 minutes`);
            
            // Notification technique interne
            TeslaService.log(`SESSION EXPIRÉE: Veuillez régénérer le token sur https://developer.tesla.com`);
        } else {
            // Calculer le temps écoulé depuis la dernière notification
            const minutesSinceLastNotification = Math.round((now - TeslaService.lastSessionTimeoutNotification) / (60 * 1000));
            
            TeslaChargingLogger.writeToLog(`🔇 SESSION: Alerte supprimée (dernière il y a ${minutesSinceLastNotification} minutes < 60min)`);
            console.log(`TeslaService: Alerte session supprimée (limitation fréquence)`);
            
            // Réduire les tentatives d'API même sans notification
            const nextRetryTime = new Date(now + 5 * 60 * 1000);
            TeslaService.next_action_time = now + 5 * 60 * 1000; // 5 minutes pour les erreurs suivantes
            
            TeslaChargingLogger.writeToLog(`⏱️ SESSION: Prochaine tentative retardée à ${nextRetryTime.toISOString()} (5 minutes)`);
        }
        
        // Tenter de rafraîchir le token ultérieurement
        TeslaService.mode = TeslaService.MODE_REFRESH_TOKEN;
        TeslaService.step = TeslaService.STEP_CONNECT;
        
        TeslaChargingLogger.writeToLog(`🔄 SESSION: Planification d'un rafraîchissement token à la prochaine exécution`);
        
        return true; // Erreur gérée
    }
    
    // Autres vérifications d'erreurs liées à l'authentification
    if (hasErrorResponse && error.response.status === 401) {
        TeslaChargingLogger.writeToLog(`🔑 SESSION: Erreur d'authentification 401 détectée (potentiellement token expiré)`);
        console.log("TeslaService: Erreur d'authentification 401 détectée - tentative de rafraîchissement token");
        
        // Planifier un rafraîchissement du token
        TeslaService.mode = TeslaService.MODE_REFRESH_TOKEN;
        TeslaService.step = TeslaService.STEP_CONNECT;
        TeslaService.next_action_time = new Date().getTime() + 1000; // 1 seconde
        
        return true; // Erreur gérée
    }
    
    TeslaChargingLogger.writeToLog("✅ SESSION: Pas d'erreur de session détectée");
    return false; // Pas une erreur de session
}

;

/**
 * Récupère les données du véhicule avec optimisation des API et logs détaillés
 */
static async getVehicleData() {
    // PROTECTION ANTI-APPELS TROP FRÉQUENTS
    const now = new Date().getTime();
    if (TeslaService.lastGetVehicleDataCall && now - TeslaService.lastGetVehicleDataCall < 5000) {
        TeslaChargingLogger.writeToLog("⛔ GARANTIE ANNULÉE: getVehicleData déjà appelé récemment");
        return;
    }
    
    // Enregistrer l'horodatage de cet appel
    TeslaService.lastGetVehicleDataCall = now;
    
    // Log initial avec contexte
    const isManualRefresh = TeslaService.mode === TeslaService.MODE_GET_DATA_NOW;
    TeslaChargingLogger.writeToLog(`📊 DONNÉES: Début récupération - Mode=${TeslaService.getModeName(TeslaService.mode)}, État=${TeslaService.lastKnownVehicleState || 'inconnu'}, Rafraîchissement manuel=${isManualRefresh}`);
    
    // NOUVEAU: Utiliser MQTT si disponible et pas en mode rafraîchissement forcé
    const config = configuration_service_1.ConfigurationService.getConfiguration();
    if (config.teslamate?.enabled && 
        TeslaMqttClient.isConnected && 
        TeslaMqttClient.isDataFresh(300000) && // Données de moins de 5 minutes
        !isManualRefresh) {
        
        const mqttData = TeslaService.getMqttVehicleData();
        if (mqttData) {
            // Journaliser l'optimisation
            TeslaChargingLogger.writeToLog("💰 DONNÉES MQTT: Optimisation API - utilisation données temps réel");
            console.log("INFO: Utilisation des données MQTT (temps réel) pour économiser les appels API");
            
            // Mise à jour des données via MQTT
            if (!TeslaService.last_receive_data) TeslaService.last_receive_data = {};
            TeslaChargingLogger.writeToLog("🔄 DONNÉES MQTT: Fusion avec les données existantes");
            
            // Mise à jour intelligente des sous-objets avec les données MQTT
            if (mqttData.charge_state) {
                TeslaChargingLogger.writeToLog(`🔋 DONNÉES MQTT: Mise à jour charge_state - Niveau=${mqttData.charge_state.battery_level || 'N/A'}%, État=${mqttData.charge_state.charging_state || 'N/A'}`);
                TeslaService.last_receive_data.charge_state = {
                    ...TeslaService.last_receive_data.charge_state,
                    ...mqttData.charge_state
                };
            }
            
            if (mqttData.drive_state) {
                const lat = mqttData.drive_state.latitude;
                const lon = mqttData.drive_state.longitude;
                TeslaChargingLogger.writeToLog(`🗺️ DONNÉES MQTT: Mise à jour drive_state - Position=${lat ? lat.toFixed(4) : 'N/A'},${lon ? lon.toFixed(4) : 'N/A'}`);
                TeslaService.last_receive_data.drive_state = {
                    ...TeslaService.last_receive_data.drive_state,
                    ...mqttData.drive_state
                };
            }
            
            if (mqttData.climate_state) {
                TeslaChargingLogger.writeToLog(`🌡️ DONNÉES MQTT: Mise à jour climate_state - Int=${mqttData.climate_state.inside_temp || 'N/A'}°C, Ext=${mqttData.climate_state.outside_temp || 'N/A'}°C`);
                TeslaService.last_receive_data.climate_state = {
                    ...TeslaService.last_receive_data.climate_state,
                    ...mqttData.climate_state
                };
            }
            
            if (mqttData.vehicle_state) {
                TeslaChargingLogger.writeToLog(`🚗 DONNÉES MQTT: Mise à jour vehicle_state - ${Object.keys(mqttData.vehicle_state).length} propriétés`);
                TeslaService.last_receive_data.vehicle_state = {
                    ...TeslaService.last_receive_data.vehicle_state,
                    ...mqttData.vehicle_state
                };
            }
            
            // Ajouter métadonnées
            TeslaService.last_receive_data.timestampMs = now;
            TeslaService.last_receive_data._source = "mqtt";
            TeslaChargingLogger.writeToLog(`🔖 DONNÉES MQTT: Métadonnées ajoutées - Source=mqtt, Timestamp=${now}`);
            
            // Mise à jour des variables d'état
            TeslaService.last_success_get_data = now;
            if (mqttData.state) {
                TeslaChargingLogger.writeToLog(`🚗 DONNÉES MQTT: Mise à jour état véhicule - ${mqttData.state}`);
                TeslaService.lastKnownVehicleState = mqttData.state;
            }
            TeslaService.lastVehicleStateCheck = now;
            
            // Planifier le prochain intervalle
            TeslaService.step = TeslaService.STEP_GET_ONLINE;
            const nextInterval = TeslaService.getNextPollInterval();
            TeslaService.next_action_time = now + nextInterval;
            TeslaChargingLogger.writeToLog(`⏱️ DONNÉES MQTT: Prochaine récupération dans ${Math.round(nextInterval/1000)}s (${new Date(TeslaService.next_action_time).toISOString()})`);
            
            // Vérifier logique de charge si nécessaire
            if (TeslaService.last_receive_data.charge_state?.charging_state === 'Charging') {
                // Journaliser le début/poursuite de charge
                TeslaChargingLogger.writeToLog(`🔌 CHARGE MQTT: Batterie=${TeslaService.last_receive_data.charge_state.battery_level}%, Puissance=${TeslaService.last_receive_data.charge_state.charger_power || 'N/A'}kW`);
                
                // Vérifier si ajustement charge nécessaire
                TeslaService.checkChargingMode();
            }
            
            // Log de succès
            TeslaChargingLogger.writeToLog("✅ DONNÉES MQTT: Mise à jour réussie via MQTT");
            
            // Émettre les données au frontend
            TeslaService.handleVehicleDataReceived();
            
            return;
        } else {
            TeslaChargingLogger.writeToLog("⚠️ DONNÉES MQTT: Aucune donnée MQTT disponible malgré client connecté");
        }
    } else if (config.teslamate?.enabled && !TeslaMqttClient.isConnected) {
        TeslaChargingLogger.writeToLog("⚠️ DONNÉES MQTT: Client MQTT non connecté, utilisera l'API");
    }
    
    // PRÉVALIDATION INTELLIGENTE: Si on sait que le véhicule est endormi, ne pas faire d'appel inutile
    if (TeslaService.lastKnownVehicleState === "asleep" && 
        now - TeslaService.lastVehicleStateCheck < 30000 && // État vérifié il y a moins de 30 secondes
        !isManualRefresh) { // Et pas en mode rafraîchissement manuel
        
        TeslaChargingLogger.writeToLog("⚠️ DONNÉES OPTIMISATION: Véhicule connu endormi, passage directement au réveil");
        TeslaService.step = TeslaService.STEP_WAKE_UP;
        TeslaService.next_action_time = now;
        TeslaChargingLogger.writeToLog(`⏱️ DONNÉES: Prochaine étape=${TeslaService.getStepName(TeslaService.step)} immédiate`);
        return;
    }
    
    // Log standard pour suivi
    TeslaChargingLogger.writeToLog(`📡 DONNÉES: Requête API des données du véhicule`);
    
    // Planifier le prochain essai en cas d'échec
    TeslaService.next_action_time = now + 10 * 1000;
    TeslaChargingLogger.writeToLog(`⏱️ DONNÉES: Prochain essai dans 10s en cas d'échec (${new Date(TeslaService.next_action_time).toISOString()})`);
    
    try {
        // Initialiser le proxy si nécessaire
        const TeslaProxy = require('./tesla.proxy');
        if (!TeslaProxy.CONFIG.ready) {
            TeslaChargingLogger.writeToLog("🔄 DONNÉES: Proxy Tesla non initialisé, initialisation en cours");
            TeslaProxy.init();
            TeslaChargingLogger.writeToLog("✅ DONNÉES: Proxy Tesla initialisé avec succès");
        } else {
            TeslaChargingLogger.writeToLog("✅ DONNÉES: Proxy Tesla déjà initialisé");
        }
        
        // Utiliser VIN de préférence (plus robuste)
        const vehicleIdOrVin = TeslaService.last_receive_data?.vin || TeslaService.vehicle_id;
        
        // Protection contre VIN/ID manquant
        if (!vehicleIdOrVin) {
            TeslaChargingLogger.writeToLog("❌ DONNÉES CRITIQUE: Aucun identifiant véhicule disponible");
            TeslaService.step = TeslaService.STEP_CONNECT;
            TeslaService.next_action_time = now + 2000;
            TeslaChargingLogger.writeToLog(`⏱️ DONNÉES: Retour à l'étape connexion dans 2s (${new Date(TeslaService.next_action_time).toISOString()})`);
            return;
        }
        
        // Log de l'identifiant utilisé (masqué pour confidentialité)
        const maskedId = `${vehicleIdOrVin.substring(0, 3)}...${vehicleIdOrVin.substring(vehicleIdOrVin.length - 3)}`;
        TeslaChargingLogger.writeToLog(`🆔 DONNÉES: Utilisation identifiant véhicule: ${maskedId}`);
        
        // CORRECTION: Utiliser le proxy au lieu de l'API externe
        const endpoint = `/api/1/vehicles/${vehicleIdOrVin}/vehicle_data`;
        
        // DIAGNOSTIC: Log détaillé de l'appel API
        TeslaChargingLogger.writeToLog(`📡 DONNÉES API: Envoi requête via proxy local: ${endpoint}`);
        
        // Mesurer le temps de réponse pour diagnostic
        const startTime = Date.now();
        
        // Exécution de la requête avec le proxy
        const response = await TeslaProxy.request('GET', endpoint);
        
        // Calcul du temps de réponse
        const responseTime = Date.now() - startTime;
        
        // Log de performance
        TeslaChargingLogger.writeToLog(`⏱️ DONNÉES API: Réponse reçue en ${responseTime}ms`);
        
        // Log de succès avec informations clés
        if (response.status === 200) {
            const chargingState = response.data?.response?.charge_state?.charging_state || "Disconnected";
            const batteryLevel = response.data?.response?.charge_state?.battery_level;
            TeslaChargingLogger.writeToLog(`✅ DONNÉES API: Reçues avec succès - Statut=${response.status}, Charge=${chargingState}, Batterie=${batteryLevel || 'N/A'}%`);
            
            // Mise à jour de l'état connu du véhicule
            TeslaService.lastKnownVehicleState = "online";
            TeslaService.lastVehicleStateCheck = now;
            TeslaChargingLogger.writeToLog(`🚗 DONNÉES API: Véhicule marqué comme "online"`);
            
            // Traitement des données reçues
            if (response.data.response && 
                response.data.response.charge_state && 
                response.data.response.charge_state.battery_level > 0) {
                
                TeslaService.log(`Données API valides reçues avec batterie à ${response.data.response.charge_state.battery_level}%`);
                TeslaChargingLogger.writeToLog(`✅ DONNÉES API: Données valides et complètes`);
                
                // Stocker comme meilleures données connues
                TeslaService.last_receive_data = response.data.response;
                TeslaService.last_receive_data.timestampMs = now;
                TeslaService.last_receive_data._source = "api";
                
                TeslaChargingLogger.writeToLog(`💾 DONNÉES API: Stockées avec métadonnées (source=api, timestamp=${now})`);
            } else {
                TeslaService.log("Données API reçues invalides ou incomplètes, conservation des données existantes");
                TeslaChargingLogger.writeToLog("⚠️ DONNÉES API: Invalides ou incomplètes, seul timestamp mis à jour");
                
                // Mettre à jour uniquement le timestamp pour indiquer une tentative récente
                if (!TeslaService.last_receive_data) TeslaService.last_receive_data = {};
                TeslaService.last_receive_data.timestampMs = now;
            }
            
            // Gérer les données de charge si applicable
            if (TeslaService.last_receive_data.charge_state?.charging_state === 'Charging') {
                // Extraire les informations importantes
                const batteryLevel = TeslaService.last_receive_data.charge_state.battery_level;
                const chargerPower = TeslaService.last_receive_data.charge_state.charger_power;
                const chargeAmps = TeslaService.last_receive_data.charge_state.charge_amps;
                
                // Journaliser le début/poursuite de charge
                TeslaChargingLogger.writeToLog(`🔌 CHARGE API: Batterie=${batteryLevel}%, Puissance=${chargerPower}kW, Ampérage=${chargeAmps}A`);
                
                // Ajouter aux données de charge pour graphiques
                const wCharge = TeslaService.last_receive_data.charge_state.charger_power * 1000;
                TeslaService.addToChargeData({
                    timestamp: Math.floor(now / 1000),
                    w_charge: wCharge
                });
                TeslaChargingLogger.writeToLog(`📊 CHARGE API: Données ajoutées au graphique (${wCharge}W)`);
                
                // Marquer l'état de charge
                TeslaService.wasInCharge = true;
                
                // Vérifier si ajustement charge nécessaire
                TeslaChargingLogger.writeToLog(`🔍 CHARGE API: Vérification mode charge "${TeslaService.chargingMode}"`);
                TeslaService.checkChargingMode();
            } else if (TeslaService.wasInCharge) {
                // Extraire l'état actuel
                const currentState = TeslaService.last_receive_data.charge_state.charging_state;
                
                // Fin de charge détectée
                TeslaChargingLogger.writeToLog(`🔌 CHARGE TERMINÉE: État=${currentState}`);
                TeslaService.wasInCharge = false;
                
                // Ajouter entrée à 0W pour marquer la fin de charge
                TeslaService.addToChargeData({
                    timestamp: Math.floor(now / 1000),
                    w_charge: 0
                });
                TeslaChargingLogger.writeToLog(`📊 CHARGE API: Marqueur fin de charge ajouté au graphique (0W)`);
            }
            
            // Définir le prochain intervalle selon le mode
            if (TeslaService.mode === TeslaService.MODE_GET_DATA_NOW) {
                TeslaChargingLogger.writeToLog("🔄 DONNÉES API: Fin du mode rafraîchissement manuel");
                TeslaService.mode = TeslaService.MODE_GET_DATA;
                TeslaService.next_action_time = now + TeslaService.POLLING_MINIMUM_INTERVAL;
                TeslaChargingLogger.writeToLog(`🔄 DONNÉES API: Retour au mode normal, prochaine requête dans ${TeslaService.POLLING_MINIMUM_INTERVAL/1000}s`);
            } else {
                // Intervalle adaptatif selon état
                const nextInterval = TeslaService.getNextPollInterval();
                TeslaService.next_action_time = now + nextInterval;
                TeslaChargingLogger.writeToLog(`⏱️ DONNÉES API: Prochain sondage dans ${Math.round(nextInterval/1000)}s (${new Date(TeslaService.next_action_time).toISOString()})`);
            }
            
            // Enregistrer le temps de succès
            TeslaService.last_success_get_data = now;
            TeslaService.step = TeslaService.STEP_GET_ONLINE;
            
            // Vérifier le mode solaire
            TeslaService.checkModeSolarPlus();
            
            // Émettre les données au frontend
            TeslaService.handleVehicleDataReceived();
        } else if (response.status === 401) {
            // Erreur d'authentification
            TeslaChargingLogger.writeToLog(`🔐 DONNÉES API ERREUR: Authentification (statut ${response.status}), reconnexion nécessaire`);
            
            // Sauvegarder l'état actuel avant reconnexion
            if (TeslaService.mode !== TeslaService.MODE_REFRESH_TOKEN) {
                TeslaChargingLogger.writeToLog(`🔄 DONNÉES API: Sauvegarde mode actuel "${TeslaService.getModeName(TeslaService.mode)}" avant reconnexion`);
                TeslaService.previousMode = TeslaService.mode;
            }
            
            // Forcer reconnexion
            TeslaService.mode = TeslaService.MODE_REFRESH_TOKEN;
            TeslaService.previousStep = TeslaService.step;
            TeslaService.step = TeslaService.STEP_CONNECT;
            TeslaService.next_action_time = now;
            TeslaChargingLogger.writeToLog(`🔄 DONNÉES API: Passage au mode reconnexion immédiat`);
        } else {
            // Autres erreurs
            TeslaChargingLogger.writeToLog(`❓ DONNÉES API ERREUR: Réponse inattendue (statut ${response.status})`);
            TeslaService.next_action_time = now + 60 * 1000; // Réessayer dans 1 minute
            TeslaChargingLogger.writeToLog(`⏱️ DONNÉES API: Nouvelle tentative dans 60s (${new Date(TeslaService.next_action_time).toISOString()})`);
        }
    } catch (error) {
        // Log d'erreur général
        const errorMessage = error.message || 'Erreur inconnue';
        TeslaChargingLogger.writeToLog(`❌ DONNÉES ERREUR: ${errorMessage}`);
        
        // Gestion spécifique des erreurs
        if (TeslaService.handleSessionTimeout(error)) {
            TeslaChargingLogger.writeToLog("🔑 DONNÉES: Erreur de session détectée et gérée");
            return; // L'erreur de session a été gérée
        }
        
        // Gestion spécifique de l'erreur 408 (véhicule endormi)
        if (error.response && error.response.status === 408) {
            TeslaChargingLogger.writeToLog("😴 DONNÉES API: Véhicule endormi (408), passage en mode réveil");
            
            // Mise à jour de l'état connu
            TeslaService.lastKnownVehicleState = "asleep";
            TeslaService.lastVehicleStateCheck = now;
            
            // En mode rafraîchissement manuel, passer directement au réveil
            if (TeslaService.mode === TeslaService.MODE_GET_DATA_NOW) {
                TeslaChargingLogger.writeToLog("🔄 DONNÉES API: Mode rafraîchissement manuel, réveil immédiat");
                TeslaService.step = TeslaService.STEP_WAKE_UP;
                TeslaService.next_action_time = now;
                TeslaService.wake408Attempts = 0;
                return;
            }
            
            // Gestion des tentatives multiples
            if (!TeslaService.wake408Attempts) TeslaService.wake408Attempts = 0;
            TeslaService.wake408Attempts++;
            
            if (TeslaService.wake408Attempts <= 3) {
                // Augmenter progressivement l'intervalle entre les tentatives
                const waitInterval = 10000 * TeslaService.wake408Attempts;
                TeslaChargingLogger.writeToLog(`⏱️ DONNÉES API: Nouvelle tentative dans ${waitInterval/1000}s (essai ${TeslaService.wake408Attempts}/3)`);
                TeslaService.next_action_time = now + waitInterval;
            } else {
                // Après 3 tentatives infructueuses, passer au réveil explicite
                TeslaChargingLogger.writeToLog("🔄 DONNÉES API: Max tentatives atteint (3), passage au réveil explicite");
                TeslaService.step = TeslaService.STEP_WAKE_UP;
                TeslaService.next_action_time = now;
                TeslaService.wake408Attempts = 0;
            }
            return;
        }
        
        // Journaliser les détails de l'erreur pour diagnostic
        if (error.response) {
            TeslaChargingLogger.writeToLog(`❌ DONNÉES API ERREUR HTTP: Statut=${error.response.status}`);
            
            // Extraire les détails de réponse de manière sécurisée
            try {
                const errorDataSample = typeof error.response.data === 'string' 
                    ? error.response.data.substring(0, 100) 
                    : JSON.stringify(error.response.data).substring(0, 100);
                TeslaChargingLogger.writeToLog(`❌ DONNÉES API DÉTAILS: ${errorDataSample}...`);
            } catch (logError) {
                TeslaChargingLogger.writeToLog(`❌ DONNÉES API: Impossible de formater les détails de l'erreur`);
            }
        } else if (error.code) {
            TeslaChargingLogger.writeToLog(`❌ DONNÉES API ERREUR RÉSEAU: Code=${error.code}`);
            
            if (error.code === 'ECONNREFUSED') {
                TeslaChargingLogger.writeToLog("🌐 DONNÉES API: Connexion refusée - Vérifier que le proxy est accessible");
            } else if (error.code === 'ETIMEDOUT') {
                TeslaChargingLogger.writeToLog("⏱️ DONNÉES API: Délai dépassé - Réseau lent ou serveur non disponible");
            }
        }
        
        // Autres erreurs - reconnexion
        TeslaChargingLogger.writeToLog(`🔄 DONNÉES API: Passage en mode reconnexion suite à erreur`);
        TeslaService.mode = TeslaService.MODE_REFRESH_TOKEN;
        TeslaService.step = TeslaService.STEP_CONNECT;
        TeslaService.next_action_time = now + 5000; // 5 secondes
        TeslaChargingLogger.writeToLog(`⏱️ DONNÉES API: Reconnexion dans 5s (${new Date(TeslaService.next_action_time).toISOString()})`);
    } finally {
        // Log de fin
        TeslaChargingLogger.writeToLog("📋 DONNÉES: Fin de la tentative de récupération");
    }
}

;

/**
 * Récupère les données de localisation du véhicule via le proxy local
 * @returns {Promise<Object|null>} Données de localisation ou null en cas d'échec
 */
static async getLocationData() {
    TeslaChargingLogger.writeToLog("🗺️ LOCALISATION: Début récupération des coordonnées GPS");
    
    try {
        // Initialiser le proxy si nécessaire
        const TeslaProxy = require('./tesla.proxy');
        if (!TeslaProxy.CONFIG.ready) {
            TeslaChargingLogger.writeToLog("🔄 LOCALISATION: Proxy Tesla non initialisé, initialisation en cours");
            TeslaProxy.init();
            TeslaChargingLogger.writeToLog("✅ LOCALISATION: Proxy Tesla initialisé avec succès");
        } else {
            TeslaChargingLogger.writeToLog("✅ LOCALISATION: Proxy Tesla déjà initialisé");
        }
        
        // Récupérer l'identifiant du véhicule (VIN de préférence)
        const vehicleIdOrVin = TeslaService.last_receive_data?.vin || TeslaService.vehicle_id;
        
        if (!vehicleIdOrVin) {
            TeslaChargingLogger.writeToLog("❌ LOCALISATION: Identifiant véhicule manquant, impossible de récupérer les coordonnées");
            return null;
        }
        
        // Masquer l'identifiant pour la confidentialité
        const maskedId = `${vehicleIdOrVin.substring(0, 3)}...${vehicleIdOrVin.substring(vehicleIdOrVin.length - 3)}`;
        TeslaChargingLogger.writeToLog(`🆔 LOCALISATION: Utilisation identifiant véhicule ${maskedId}`);
        
        // Construire l'endpoint avec paramètre pour limiter aux données de localisation
        const endpoint = `/api/1/vehicles/${vehicleIdOrVin}/vehicle_data?endpoints=location_data`;
        TeslaChargingLogger.writeToLog(`🗺️ LOCALISATION: Endpoint construit: ${endpoint}`);
        
        // Log avant requête API
        TeslaChargingLogger.writeToLog("📡 LOCALISATION: Envoi requête API via proxy local");
        
        // Mesure du temps de réponse pour diagnostic
        const requestStartTime = Date.now();
        
        // Exécution de la requête via le proxy
        const response = await TeslaProxy.request('GET', endpoint);
        
        // Calcul du temps de réponse
        const requestDuration = Date.now() - requestStartTime;
        TeslaChargingLogger.writeToLog(`⏱️ LOCALISATION: Réponse reçue en ${requestDuration}ms (statut ${response.status})`);
        
        // Analyse de la réponse
        if (response.status === 200 && response.data?.response?.drive_state) {
            const latitude = response.data.response.drive_state.latitude;
            const longitude = response.data.response.drive_state.longitude;
            
            // Vérifier la validité des coordonnées
            if (!latitude || !longitude) {
                TeslaChargingLogger.writeToLog("❌ LOCALISATION: Coordonnées manquantes ou invalides dans la réponse");
                return null;
            }
            
            // Formater les coordonnées pour le log (précision réduite pour confidentialité)
            const latFormatted = latitude.toFixed(4);
            const lonFormatted = longitude.toFixed(4);
            TeslaChargingLogger.writeToLog(`🗺️ LOCALISATION: Coordonnées reçues - Lat: ${latFormatted}, Long: ${lonFormatted}`);
            
            // Mise à jour des données de localisation dans l'objet global
            if (!TeslaService.last_receive_data.drive_state) {
                TeslaChargingLogger.writeToLog("🔄 LOCALISATION: Création de l'objet drive_state car inexistant");
                TeslaService.last_receive_data.drive_state = {};
            }
            
            // Copier les propriétés de localisation
            TeslaService.last_receive_data.drive_state.latitude = latitude;
            TeslaService.last_receive_data.drive_state.longitude = longitude;
            
            // Ajouter d'autres propriétés si présentes dans la réponse
            if (response.data.response.drive_state.heading !== undefined) {
                TeslaService.last_receive_data.drive_state.heading = response.data.response.drive_state.heading;
                TeslaChargingLogger.writeToLog(`🧭 LOCALISATION: Cap mis à jour à ${response.data.response.drive_state.heading}°`);
            }
            
            if (response.data.response.drive_state.speed !== undefined) {
                TeslaService.last_receive_data.drive_state.speed = response.data.response.drive_state.speed;
                TeslaChargingLogger.writeToLog(`🚀 LOCALISATION: Vitesse mise à jour à ${response.data.response.drive_state.speed || 0} km/h`);
            }
            
            // Ajouter un timestamp pour le suivi de fraîcheur
            TeslaService.last_receive_data.drive_state.timestamp = Date.now();
            TeslaChargingLogger.writeToLog("✅ LOCALISATION: Données mises à jour dans le cache avec succès");
            
            // Retourner les coordonnées
            return {
                latitude: latitude,
                longitude: longitude,
                timestamp: Date.now()
            };
        } else {
            // Récupération des détails de l'erreur
            const statusCode = response.status;
            let errorDetails = "";
            
            try {
                if (response.data) {
                    errorDetails = JSON.stringify(response.data).substring(0, 100) + "...";
                }
            } catch (parseError) {
                errorDetails = "Impossible d'extraire les détails de la réponse";
            }
            
            TeslaChargingLogger.writeToLog(`❌ LOCALISATION: Réponse inattendue (${statusCode}) - ${errorDetails}`);
            return null;
        }
    } catch (error) {
        // Log d'erreur sécurisé
        TeslaChargingLogger.logApiError('location_data', 'GET', {
            message: error.message,
            status: error.response?.status
        });
        
        TeslaChargingLogger.writeToLog(`❌ LOCALISATION ERREUR: ${error.message || "Erreur inconnue"}`);
        
        // Gestion spécifique des erreurs
        if (error.response) {
            TeslaChargingLogger.writeToLog(`❌ LOCALISATION: Erreur HTTP ${error.response.status}`);
            
            // Véhicule endormi - planifier un réveil
            if (error.response.status === 408) {
                const now = new Date().getTime();
                TeslaChargingLogger.writeToLog("😴 LOCALISATION: Véhicule endormi (statut 408), mise à jour de l'état");
                
                // Mise à jour de l'état connu
                TeslaService.lastKnownVehicleState = "asleep";
                TeslaService.lastVehicleStateCheck = now;
                
                // Planifier un réveil
                TeslaService.step = TeslaService.STEP_WAKE_UP;
                TeslaService.next_action_time = now + 5000; // 5 secondes
                TeslaChargingLogger.writeToLog(`⏱️ LOCALISATION: Réveil programmé dans 5s (${new Date(TeslaService.next_action_time).toISOString()})`);
            } else if (error.response.status === 401) {
                TeslaChargingLogger.writeToLog("🔑 LOCALISATION: Erreur d'authentification (401), reconnexion nécessaire");
                
                // Forcer une reconnexion
                const now = new Date().getTime();
                TeslaService.step = TeslaService.STEP_CONNECT;
                TeslaService.next_action_time = now + 1000; // 1 seconde
                TeslaChargingLogger.writeToLog(`⏱️ LOCALISATION: Reconnexion programmée dans 1s`);
            }
        } else if (error.code) {
            // Erreurs réseau
            TeslaChargingLogger.writeToLog(`❌ LOCALISATION ERREUR RÉSEAU: ${error.code}`);
            
            if (error.code === 'ECONNREFUSED') {
                TeslaChargingLogger.writeToLog("🌐 LOCALISATION: Connexion refusée - Vérifier que le proxy est accessible");
            } else if (error.code === 'ETIMEDOUT') {
                TeslaChargingLogger.writeToLog("⏱️ LOCALISATION: Délai d'attente dépassé - Réseau lent ou serveur non disponible");
            }
        }
        
        return null;
    } finally {
        TeslaChargingLogger.writeToLog("📋 LOCALISATION: Fin de la tentative de récupération GPS");
    }
}

    ;

/**
 * Ajoute des données de charge au historique et supprime les anciennes entrées
 * @param {Object} value - Objet contenant les données à ajouter (timestamp et w_charge)
 */
static addToChargeData(value) {
    // Validation des données d'entrée
    if (!value || typeof value !== 'object') {
        TeslaChargingLogger.writeToLog("❌ CHARGE DATA: Données invalides, entrée ignorée");
        return;
    }
    
    // Vérifier la présence des propriétés requises
    if (!value.timestamp || value.w_charge === undefined) {
        TeslaChargingLogger.writeToLog("❌ CHARGE DATA: Structure de données incomplète, entrée ignorée");
        return;
    }
    
    // Initialiser le tableau si nécessaire
    if (!TeslaService.chargeData) {
        TeslaChargingLogger.writeToLog("🔄 CHARGE DATA: Initialisation du tableau de données");
        TeslaService.chargeData = [];
    }
    
    // Journaliser l'entrée ajoutée
    const date = new Date(value.timestamp * 1000);
    const formattedTime = date.toLocaleTimeString();
    TeslaChargingLogger.writeToLog(`📊 CHARGE DATA: Ajout entrée ${value.w_charge}W à ${formattedTime}`);
    
    // Ajouter la nouvelle entrée
    TeslaService.chargeData.push(value);
    
    // Compter le nombre d'entrées avant nettoyage
    const beforeCount = TeslaService.chargeData.length;
    
    // Calculer la limite de temps (24 heures)
    const cutoffTime = Math.floor(new Date().getTime() / 1000) - (3600 * 24);
    
    // Supprimer les données trop anciennes (plus de 24h)
    let removedCount = 0;
    while (TeslaService.chargeData.length > 0 && TeslaService.chargeData[0].timestamp < cutoffTime) {
        TeslaService.chargeData.splice(0, 1);
        removedCount++;
    }
    
    // Journaliser le nettoyage si des entrées ont été supprimées
    if (removedCount > 0) {
        TeslaChargingLogger.writeToLog(`🧹 CHARGE DATA: ${removedCount} entrées anciennes supprimées (>24h)`);
        TeslaChargingLogger.writeToLog(`📈 CHARGE DATA: ${beforeCount} → ${TeslaService.chargeData.length} entrées (${removedCount} supprimées)`);
    }
    
    // Log de synthèse pour surveillance de la taille
    if (TeslaService.chargeData.length % 10 === 0) {
        const oldestEntry = TeslaService.chargeData[0];
        const newestEntry = TeslaService.chargeData[TeslaService.chargeData.length - 1];
        
        if (oldestEntry && newestEntry) {
            const oldestDate = new Date(oldestEntry.timestamp * 1000);
            const newestDate = new Date(newestEntry.timestamp * 1000);
            const durationHours = (newestEntry.timestamp - oldestEntry.timestamp) / 3600;
            
            TeslaChargingLogger.writeToLog(`📊 CHARGE DATA: ${TeslaService.chargeData.length} entrées couvrant ${durationHours.toFixed(1)}h (de ${oldestDate.toLocaleTimeString()} à ${newestDate.toLocaleTimeString()})`);
        }
    }
}

    ;

/**
 * Demande un rafraichissement des données immediat avec approche simplifiée "un clic = une données"
 */
static async getImmediateVehicleData() {
    const now = new Date().getTime();
    TeslaChargingLogger.writeToLog("🔄 REFRESH: Début demande rafraîchissement immédiat");
    
    // PROTECTION: Empêcher les exécutions simultanées
    if (TeslaService.refreshInProgress) {
        TeslaChargingLogger.writeToLog("⛔ REFRESH PROTECTION: Rafraîchissement déjà en cours, demande ignorée");
        console.log("TeslaService: Demande de rafraîchissement ignorée (une autre en cours)");
        return;
    }
    
    // PROTECTION: Anti-rebond pour éviter appels trop rapprochés
    if (TeslaService.lastRefreshRequest && now - TeslaService.lastRefreshRequest < 5000) {
        const timeSinceLastRefresh = Math.round((now - TeslaService.lastRefreshRequest)/1000);
        TeslaChargingLogger.writeToLog(`⛔ REFRESH PROTECTION: Demande ignorée (trop rapprochée, dernière il y a ${timeSinceLastRefresh}s)`);
        return;
    }
    
    // Activer le verrou et enregistrer la demande
    TeslaService.refreshInProgress = true;
    TeslaService.lastRefreshRequest = now;
    
    try {
        // 1. VIN manquant : lancer séquence complète
        if (!TeslaService.last_receive_data?.vin) {
            TeslaChargingLogger.writeToLog("❌ REFRESH: VIN manquant, connexion et récupération véhicule nécessaire");
            await TeslaService.connect();
            await TeslaService.getVehicleList();
        }
        
        // 2. Vérifier l'état du véhicule
        TeslaChargingLogger.writeToLog("🔍 REFRESH: Vérification état du véhicule");
        const vehicleState = await TeslaService.checkVehicleOnlineStatus();
        TeslaChargingLogger.writeToLog(`✅ REFRESH: État véhicule = "${vehicleState}"`);
        
        // 3. Si véhicule endormi/offline, réveiller PUIS récupérer données (séquentiel)
        if (vehicleState === "asleep" || vehicleState === "offline") {
            TeslaChargingLogger.writeToLog("😴 REFRESH: Véhicule endormi/offline, réveil nécessaire");
            
            // Forcer le mode manuel temporairement
            const previousMode = TeslaService.mode;
            TeslaService.mode = TeslaService.MODE_GET_DATA_NOW;
            
            // Réveiller le véhicule
            await TeslaService.wakeUp();
            
            // Récupérer les données (un seul appel, après réveil)
            TeslaChargingLogger.writeToLog("📊 REFRESH: Récupération données après réveil");
            await TeslaService.getVehicleData();
            
            // Restaurer le mode précédent
            TeslaService.mode = previousMode;
        } 
        // 4. Si véhicule déjà en ligne, récupérer les données directement
        else {
            TeslaChargingLogger.writeToLog("🚗 REFRESH: Véhicule déjà en ligne, récupération directe des données");
            
            // Forcer le mode manuel temporairement
            const previousMode = TeslaService.mode;
            TeslaService.mode = TeslaService.MODE_GET_DATA_NOW;
            
            // Récupérer les données (un seul appel)
            await TeslaService.getVehicleData();
            
            // Restaurer le mode précédent
            TeslaService.mode = previousMode;
        }
        
        TeslaChargingLogger.writeToLog("✅ REFRESH: Rafraîchissement des données terminé");
    } catch (error) {
        TeslaChargingLogger.writeToLog(`❌ REFRESH ERREUR: ${error.message}`);
    } finally {
        // Garantir la libération du verrou
        TeslaService.refreshInProgress = false;
        TeslaChargingLogger.writeToLog("🔓 REFRESH: Verrou de rafraîchissement libéré");
    }
}

    ;
/**
 * Activez le mode de récupération continue des données (polling fréquent)
 * Ce mode est utilisé pour les sessions de conduite active ou d'analyse en temps réel
 */
static setContinuousMode() {
    const now = new Date().getTime();
    TeslaChargingLogger.writeToLog("🔄 MODE CONTINU: Activation du mode de récupération continue des données");
    
    // Sauvegarder le mode précédent pour référence
    const previousMode = TeslaService.mode;
    const previousStep = TeslaService.step;
    
    // Définir le nouveau mode
    TeslaService.mode = TeslaService.MODE_GET_CONTINUS_DATA;
    TeslaChargingLogger.writeToLog(`🔄 MODE CONTINU: Mode passé de ${TeslaService.getModeName(previousMode)} à ${TeslaService.getModeName(TeslaService.mode)}`);
    
    // Adapter l'étape selon l'état actuel
    if (TeslaService.step === TeslaService.STEP_GET_ONLINE) {
        TeslaService.step = TeslaService.STEP_WAKE_UP;
        TeslaChargingLogger.writeToLog(`🔄 MODE CONTINU: Étape modifiée de GET_ONLINE à WAKE_UP pour garantir un véhicule actif`);
    } else {
        TeslaService.step = TeslaService.STEP_GET_DATA;
        TeslaChargingLogger.writeToLog(`🔄 MODE CONTINU: Étape définie à GET_DATA pour récupération immédiate`);
    }
    
    // Programmer une exécution immédiate
    TeslaService.next_action_time = now;
    TeslaChargingLogger.writeToLog(`⏱️ MODE CONTINU: Prochaine action programmée immédiatement (${new Date(TeslaService.next_action_time).toISOString()})`);
    
    // Vérifier l'état du véhicule pour information
    const knownState = TeslaService.lastKnownVehicleState || 'inconnu';
    TeslaChargingLogger.writeToLog(`🚗 MODE CONTINU: État actuel connu du véhicule: "${knownState}"`);
    
    // Créer une notification système
    console.log(`TeslaService: Mode continu activé, récupération fréquente des données (intervalle: ${15}s)`);
    
    // Informations sur l'intervalle de sondage
    TeslaChargingLogger.writeToLog(`⏱️ MODE CONTINU: Intervalle de sondage réduit à 15 secondes`);
    
    return true;
}
    ;

    /**
     * Set as continuous mode
     */
/**
 * Revient au mode de récupération normal des données (polling standard)
 * Ce mode est utilisé pour l'économie d'API et la réduction de la consommation d'énergie
 */
static setNormalMode() {
    const now = new Date().getTime();
    TeslaChargingLogger.writeToLog("🔄 MODE NORMAL: Activation du mode de récupération standard des données");
    
    // Sauvegarder le mode précédent pour référence
    const previousMode = TeslaService.mode;
    const previousStep = TeslaService.step;
    
    // Définir le nouveau mode
    TeslaService.mode = TeslaService.MODE_GET_DATA;
    TeslaChargingLogger.writeToLog(`🔄 MODE NORMAL: Mode passé de ${TeslaService.getModeName(previousMode)} à ${TeslaService.getModeName(TeslaService.mode)}`);
    
    // Définir la prochaine étape
    TeslaService.step = TeslaService.STEP_GET_DATA;
    TeslaChargingLogger.writeToLog(`🔄 MODE NORMAL: Étape définie à ${TeslaService.getStepName(TeslaService.step)} (GET_DATA)`);
    
    // Programmer une exécution immédiate
    TeslaService.next_action_time = now;
    TeslaChargingLogger.writeToLog(`⏱️ MODE NORMAL: Prochaine action programmée immédiatement (${new Date(TeslaService.next_action_time).toISOString()})`);
    
    // Déterminer le prochain intervalle de polling
    const nextInterval = TeslaService.getNextPollInterval();
    const intervalMinutes = Math.round(nextInterval / (60 * 1000) * 10) / 10;
    TeslaChargingLogger.writeToLog(`⏱️ MODE NORMAL: Prochain intervalle de polling sera de ${intervalMinutes} minutes`);
    
    // Créer une notification système
    console.log(`TeslaService: Mode normal activé, récupération standard des données (intervalle adaptatif)`);
    
    return true;
}

 ;

/**
 * La montre passe en mode affichage de l'heure donc on reprend la lecture de l'état en mode normal
 * Cette méthode est appelée lorsqu'un dispositif d'affichage revient à l'écran d'horloge
 */
static displayScreenHour() {
    TeslaChargingLogger.writeToLog("🕒 ÉCRAN: Gestion du retour à l'affichage de l'heure");
    
    // Log de l'état actuel pour diagnostic
    const currentMode = TeslaService.getModeName(TeslaService.mode);
    TeslaChargingLogger.writeToLog(`🕒 ÉCRAN: Mode actuel = "${currentMode}"`);
    
    // Vérifier si nous sommes en mode continu
    if (TeslaService.mode === TeslaService.MODE_GET_CONTINUS_DATA) {
        TeslaChargingLogger.writeToLog("🔄 ÉCRAN: Détection mode continu actif, passage en mode normal");
        console.log("TeslaService: Affichage revenu à l'écran d'horloge, désactivation du mode continu");
        
        // Sauvegarder le mode actuel avant changement
        const previousMode = TeslaService.mode;
        const previousStep = TeslaService.step;
        
        // Définir le nouveau mode
        TeslaService.setNormalMode();
        
        // Journaliser le changement
        TeslaChargingLogger.writeToLog(`🔄 ÉCRAN: Mode modifié de "${TeslaService.getModeName(previousMode)}" à "${TeslaService.getModeName(TeslaService.mode)}"`);
        TeslaChargingLogger.writeToLog(`🔄 ÉCRAN: Étape modifiée de "${TeslaService.getStepName(previousStep)}" à "${TeslaService.getStepName(TeslaService.step)}"`);
        
        // Obtenir le prochain intervalle prévu
        const nextInterval = TeslaService.getNextPollInterval();
        const nextIntervalMinutes = Math.round(nextInterval / (60 * 1000) * 10) / 10;
        
        // Log détaillé des informations de planification
        const nextTime = new Date(TeslaService.next_action_time).toISOString();
        TeslaChargingLogger.writeToLog(`⏱️ ÉCRAN: Prochain intervalle adaptatif = ${nextIntervalMinutes} minutes`);
        TeslaChargingLogger.writeToLog(`⏱️ ÉCRAN: Prochaine action planifiée à ${nextTime}`);
        
        // Notification de succès
        TeslaChargingLogger.writeToLog("✅ ÉCRAN: Retour au mode normal réussi");
    } else {
        // Pas de changement nécessaire
        TeslaChargingLogger.writeToLog("ℹ️ ÉCRAN: Déjà en mode normal, aucun changement nécessaire");
    }
}

;

/**
 * Ouvre le frunk (capot avant) via le proxy local
 * @returns {Promise<boolean>} true si la commande a été exécutée avec succès
 */
static async frunkOpen() {
    return TeslaService.executeVehicleCommand({
        commandName: "FRUNK",
        commandType: "frunkOpen",
        endpoint: "actuate_trunk",
        payload: { which_trunk: "front" },
        icon: "🚘",
        lockVar: "lastFrunkOpenCall",
        onSuccess: (response) => {
            TeslaChargingLogger.writeToLog("✅ FRUNK: Capot avant ouvert avec succès");
            
            // Mettre à jour le cache si nécessaire
            if (!TeslaService.last_receive_data) {
                TeslaChargingLogger.writeToLog("🔄 FRUNK: Initialisation du cache de données");
                TeslaService.last_receive_data = {};
            }
            
            if (!TeslaService.last_receive_data.vehicle_state) {
                TeslaChargingLogger.writeToLog("🔄 FRUNK: Initialisation du cache vehicle_state");
                TeslaService.last_receive_data.vehicle_state = {};
            }
            
            // Mise à jour des propriétés d'état du véhicule pour refléter l'ouverture
            TeslaService.last_receive_data.vehicle_state.ft = 1; // front trunk = 1 (ouvert)
            TeslaService.last_receive_data.vehicle_state.frunk_open = true;
            TeslaChargingLogger.writeToLog("💾 FRUNK: État mis à jour dans le cache (frunk_open=true)");
            
            // Log pour diagnostic
            console.log("TeslaService: Frunk ouvert avec succès");
            
            // Programmer une vérification des données pour confirmer le changement
            TeslaService.step = TeslaService.STEP_GET_DATA;
            TeslaService.next_action_time = new Date().getTime() + 5000; // 5 secondes
            TeslaChargingLogger.writeToLog("⏱️ FRUNK: Vérification programmée dans 5s");
        }
    });
}
    
    ;

/**
 * Ouvre la trappe de charge via le proxy local
 * @returns {Promise<boolean>} true si la commande a été exécutée avec succès
 */
static async openChargeDoor() {
    return TeslaService.executeVehicleCommand({
        commandName: "TRAPPE CHARGE",
        commandType: "openChargeDoor",
        endpoint: "charge_port_door_open",
        icon: "🔌",
        lockVar: "lastOpenChargeDoorCall",
        onSuccess: (response) => {
            TeslaChargingLogger.writeToLog("✅ TRAPPE: Trappe de charge ouverte avec succès");
            
            // Mettre à jour le cache si nécessaire
            if (!TeslaService.last_receive_data) {
                TeslaChargingLogger.writeToLog("🔄 TRAPPE: Initialisation du cache de données");
                TeslaService.last_receive_data = {};
            }
            
            if (!TeslaService.last_receive_data.charge_state) {
                TeslaChargingLogger.writeToLog("🔄 TRAPPE: Initialisation du cache charge_state");
                TeslaService.last_receive_data.charge_state = {};
            }
            
            // Mise à jour des propriétés d'état de charge
            TeslaService.last_receive_data.charge_state.charge_port_door_open = true;
            TeslaService.last_receive_data.charge_state.charge_port_latch = "Open";
            TeslaService.last_receive_data.charge_state.charge_port_cold_weather_mode = false;
            
            TeslaChargingLogger.writeToLog("💾 TRAPPE: État du port de charge mis à jour dans le cache");
            
            // Log pour diagnostic
            console.log("TeslaService: Trappe de charge ouverte avec succès");
            
            // Programmer une vérification des données pour confirmer le changement
            TeslaService.step = TeslaService.STEP_GET_DATA;
            TeslaService.next_action_time = new Date().getTime() + 5000; // 5 secondes
            TeslaChargingLogger.writeToLog("⏱️ TRAPPE: Vérification programmée dans 5s");
        }
    });
}

    ;

/**
 * Ferme la trappe de charge via le proxy local
 * @returns {Promise<boolean>} true si la commande a été exécutée avec succès
 */
static async closeChargeDoor() {
    return TeslaService.executeVehicleCommand({
        commandName: "TRAPPE CHARGE",
        commandType: "closeChargeDoor",
        endpoint: "charge_port_door_close",
        icon: "🔌",
        lockVar: "lastCloseChargeDoorCall",
        onSuccess: (response) => {
            TeslaChargingLogger.writeToLog("✅ TRAPPE: Trappe de charge fermée avec succès");
            
            // Mettre à jour le cache si nécessaire
            if (!TeslaService.last_receive_data) {
                TeslaChargingLogger.writeToLog("🔄 TRAPPE: Initialisation du cache de données");
                TeslaService.last_receive_data = {};
            }
            
            if (!TeslaService.last_receive_data.charge_state) {
                TeslaChargingLogger.writeToLog("🔄 TRAPPE: Initialisation du cache charge_state");
                TeslaService.last_receive_data.charge_state = {};
            }
            
            // Mise à jour des propriétés d'état de charge
            TeslaService.last_receive_data.charge_state.charge_port_door_open = false;
            TeslaService.last_receive_data.charge_state.charge_port_latch = "Engaged";
            TeslaChargingLogger.writeToLog("💾 TRAPPE: État du port de charge mis à jour dans le cache");
            
            // Log pour diagnostic
            console.log("TeslaService: Trappe de charge fermée avec succès");
            
            // Programmer une vérification des données pour confirmer le changement
            TeslaService.step = TeslaService.STEP_GET_DATA;
            TeslaService.next_action_time = new Date().getTime() + 5000; // 5 secondes
            TeslaChargingLogger.writeToLog("⏱️ TRAPPE: Vérification programmée dans 5s");
        }
    });
}
    
    ;

/**
 * Déverrouille les portes du véhicule via le proxy local
 * @returns {Promise<boolean>} true si la commande a été exécutée avec succès
 */
static async doorUnlock() {
    return TeslaService.executeVehicleCommand({
        commandName: "DÉVERROUILLAGE",
        commandType: "doorUnlock",
        endpoint: "door_unlock",
        icon: "🔓",
        lockVar: "lastDoorUnlockCall",
        onSuccess: (response) => {
            TeslaChargingLogger.writeToLog("✅ DÉVERROUILLAGE: Portes déverrouillées avec succès");
            
            // Mettre à jour le cache si nécessaire
            if (!TeslaService.last_receive_data) {
                TeslaChargingLogger.writeToLog("🔄 DÉVERROUILLAGE: Initialisation du cache de données");
                TeslaService.last_receive_data = {};
            }
            
            if (!TeslaService.last_receive_data.vehicle_state) {
                TeslaChargingLogger.writeToLog("🔄 DÉVERROUILLAGE: Initialisation du cache vehicle_state");
                TeslaService.last_receive_data.vehicle_state = {};
            }
            
            // Mise à jour des propriétés d'état du véhicule
            TeslaService.last_receive_data.vehicle_state.locked = false;
            TeslaService.last_receive_data.vehicle_state.df = 0; // porte conducteur 
            TeslaService.last_receive_data.vehicle_state.pf = 0; // porte passager
            TeslaService.last_receive_data.vehicle_state.dr = 0; // porte arrière gauche
            TeslaService.last_receive_data.vehicle_state.pr = 0; // porte arrière droite
            
            TeslaChargingLogger.writeToLog("💾 DÉVERROUILLAGE: État mis à jour dans le cache (locked=false)");
            
            // Log pour diagnostic
            console.log("TeslaService: Véhicule déverrouillé avec succès");
            
            // Programmer une vérification des données pour confirmer le changement
            TeslaService.step = TeslaService.STEP_GET_DATA;
            TeslaService.next_action_time = new Date().getTime() + 5000; // 5 secondes
            TeslaChargingLogger.writeToLog("⏱️ DÉVERROUILLAGE: Vérification programmée dans 5s");
        }
    });
}

    ;

/**
 * Ferme les fenêtres via le proxy local
 * @returns {Promise<boolean>} true si la commande a été exécutée avec succès
 */
static async ventClose() {
    return TeslaService.executeVehicleCommand({
        commandName: "FERMETURE FENÊTRES",
        commandType: "ventClose",
        endpoint: "window_control",
        payload: { 
            command: "close",
            lat: 0,
            lon: 0
        },
        icon: "🪟",
        lockVar: "lastVentCloseCall",
        onSuccess: (response) => {
            TeslaChargingLogger.writeToLog("✅ FENÊTRES: Fermeture réussie");
            
            // Mettre à jour le cache si nécessaire
            if (!TeslaService.last_receive_data) {
                TeslaChargingLogger.writeToLog("🔄 FENÊTRES: Initialisation du cache de données");
                TeslaService.last_receive_data = {};
            }
            
            if (!TeslaService.last_receive_data.vehicle_state) {
                TeslaChargingLogger.writeToLog("🔄 FENÊTRES: Initialisation du cache vehicle_state");
                TeslaService.last_receive_data.vehicle_state = {};
            }
            
            // Mise à jour des propriétés d'état du véhicule
            TeslaService.last_receive_data.vehicle_state.fd_window = 0;
            TeslaService.last_receive_data.vehicle_state.fp_window = 0;
            TeslaService.last_receive_data.vehicle_state.rd_window = 0;
            TeslaService.last_receive_data.vehicle_state.rp_window = 0;
            TeslaService.last_receive_data.vehicle_state.windows_open = false;
            TeslaChargingLogger.writeToLog("💾 FENÊTRES: État mis à jour dans le cache (windows_open=false)");
            
            // Log pour diagnostic
            console.log("TeslaService: Fenêtres fermées avec succès");
            
            // Programmer une vérification des données pour confirmer le changement
            TeslaService.step = TeslaService.STEP_GET_DATA;
            TeslaService.next_action_time = new Date().getTime() + 5000; // 5 secondes
            TeslaChargingLogger.writeToLog("⏱️ FENÊTRES: Vérification programmée dans 5s");
        }
    });
}
    ;

/**
 * Exécute une commande sur le véhicule avec gestion complète des erreurs et du réveil
 * @param {Object} options - Options de la commande
 * @param {string} options.commandName - Nom de la commande pour les logs (ex: "CHARGE", "CLIMAT")
 * @param {string} options.commandType - Type de commande pour file d'attente (ex: "startCharge", "stopClimatisation")
 * @param {string} options.endpoint - Point d'accès API (ex: "charge_start", "auto_conditioning_stop")
 * @param {Object} options.payload - Données à envoyer (défaut: {})
 * @param {string} options.icon - Icône pour les logs (ex: "🔌", "🌡️")
 * @param {string} options.lockVar - Variable pour le verrou (ex: "lastStartChargeCall")
 * @param {Function} options.onSuccess - Fonction à exécuter en cas de succès (optionnel)
 * @returns {Promise<boolean>} true si la commande a été exécutée avec succès
 */
static async executeVehicleCommand(options) {
    // Destructurer les options avec valeurs par défaut
    const {
        commandName, 
        commandType,
        endpoint,
        payload = {},
        icon = "📡",
        lockVar,
        onSuccess
    } = options;
    
    // Validation des paramètres requis
    if (!commandName || !commandType || !endpoint || !lockVar) {
        TeslaChargingLogger.writeToLog(`❌ ERREUR PARAMÈTRES: executeVehicleCommand appelé avec paramètres incomplets`);
        console.error(`TeslaService: Commande ${commandName || 'inconnue'} annulée - paramètres incomplets`);
        return false;
    }
    
    // Horodatage et identifiant unique de verrou
    const now = new Date().getTime();
    const lockId = `${commandType}-${now}`; // Identifiant unique pour ce verrou
    const lastCall = TeslaService[lockVar] || 0;
    
    // Log détaillé du début d'exécution
    TeslaChargingLogger.writeToLog(`${icon} ${commandName}: Début traitement commande ${endpoint}`);
    TeslaChargingLogger.writeToLog(`${icon} ${commandName}: Payload: ${JSON.stringify(payload)}`);
    
    // Vérifier si une commande est déjà en cours et l'ajouter au log
    const isLocked = !!TeslaService[lockVar];
    if (isLocked) {
        const lockAge = Math.round((now - lastCall)/1000);
        TeslaChargingLogger.writeToLog(`🔒 ${commandName}: Verrou existant détecté (âge: ${lockAge}s)`);
    }
    
    // Timer de sécurité pour garantir la libération du verrou
    let safetyTimeoutId = null;
    
    try {
        // Protection anti-appels multiples (anti-rebond)
        if (lastCall && now - lastCall < 10000) {
            const secondsSinceLastCall = Math.round((now - lastCall) / 1000);
            TeslaChargingLogger.writeToLog(`⛔ PROTECTION: Demande de ${commandName} ignorée (trop rapprochée, dernier appel il y a ${secondsSinceLastCall}s)`);
            console.log(`TeslaService: Commande ${commandName} ignorée (déjà appelée il y a ${secondsSinceLastCall}s)`);
            return false;
        }
        
        // NOUVEAU: Marquer le début de l'opération pour le verrou
        TeslaService[lockVar] = lockId;
        TeslaChargingLogger.writeToLog(`🔒 ${commandName}: Verrou activé avec ID ${lockId.substring(0, 10)}...`);
        
        // Timer de sécurité pour garantir la libération du verrou
        safetyTimeoutId = setTimeout(() => {
            TeslaChargingLogger.writeToLog(`⏰ ${commandName}: LIBÉRATION FORCÉE du verrou après délai de sécurité`);
            TeslaService[lockVar] = null;
        }, 60000); // 60 secondes
        
        TeslaChargingLogger.writeToLog(`⏱️ ${commandName}: Timer de sécurité activé (60s)`);
        
        // Log de l'exécution de la commande
        TeslaChargingLogger.writeToLog(`${icon} ${commandName}: Demande de commande ${endpoint}`);
        
        // S'assurer que le proxy est initialisé
        const TeslaProxy = require('./tesla.proxy');
        if (!TeslaProxy.CONFIG.ready) {
            TeslaChargingLogger.writeToLog(`${icon} ${commandName}: Initialisation du proxy Tesla`);
            TeslaProxy.init();
            TeslaChargingLogger.writeToLog(`✅ ${commandName}: Proxy Tesla initialisé avec succès`);
        } else {
            TeslaChargingLogger.writeToLog(`✅ ${commandName}: Proxy Tesla déjà initialisé`);
        }
        
        // MODIFICATION MAJEURE: Vérification de l'état du véhicule
        TeslaChargingLogger.writeToLog(`${icon} ${commandName}: Vérification état avant envoi`);
        
        const startCheckTime = Date.now();
        
        // 1. NOUVEAU: Vérifier d'abord les données MQTT (plus rapide et ne cause pas de conflit)
        const mqttData = TeslaService.getMqttVehicleData();
        
        let vehicleState = "unknown";
        
        // MODIFICATION: Vérifier d'abord si un réveil BLE récent a été confirmé (priorité maximale)
        if (TeslaService.bleConfirmedAwake && (Date.now() - TeslaService.bleConfirmedAwake < 60000)) {
            vehicleState = "online";
            TeslaChargingLogger.writeToLog(`🔄 ${commandName}: Utilisation état "online" forcé suite à un réveil BLE récent`);
        }
        // Si données MQTT fraîches disponibles, les utiliser sans appel API
        else if (mqttData && mqttData._timestamp && (Date.now() - mqttData._timestamp < 30000)) {
            vehicleState = mqttData.vehicle_state?.state || mqttData.state;
            
            // Reconnaissance de l'état de charge comme "online"
            if (mqttData.charge_state?.charging_state === "Charging") {
                vehicleState = "online";
            }
            
            TeslaChargingLogger.writeToLog(`📡 ${commandName}: État véhicule via MQTT: "${vehicleState}" (données fraîches)`);
        } 
        // 2. Utiliser le cache d'état s'il est récent
        else if (TeslaService.lastKnownVehicleState && TeslaService.lastVehicleStateCheck && 
                 (Date.now() - TeslaService.lastVehicleStateCheck < 15000)) {
            vehicleState = TeslaService.lastKnownVehicleState;
            TeslaChargingLogger.writeToLog(`🔍 ${commandName}: État véhicule via cache récent: "${vehicleState}"`);
        }
        // 3. Seulement si MQTT et cache indisponibles, vérifier via API
        else {
            try {
                vehicleState = await TeslaService.checkVehicleOnlineStatus();
            } catch (stateError) {
                // En cas d'erreur de vérification d'état, utiliser le dernier état connu
                vehicleState = TeslaService.lastKnownVehicleState || "unknown";
                TeslaChargingLogger.writeToLog(`⚠️ ${commandName}: Erreur vérification état, utilisation état cache: "${vehicleState}"`);
            }
        }
        
        // Log du temps de vérification
        const checkDuration = Date.now() - startCheckTime;
        TeslaChargingLogger.writeToLog(`⏱️ ${commandName}: Vérification état terminée en ${checkDuration}ms - État: "${vehicleState}"`);
        
        // Vérifier si le véhicule est online/charging pour envoyer la commande directement
        if (vehicleState !== "online" && vehicleState !== "charging" && vehicleState !== "driving") {
            TeslaChargingLogger.writeToLog(`⚠️ ${commandName}: Véhicule "${vehicleState}", réveil nécessaire avant envoi`);
            console.log(`TeslaService: ${commandName} - Véhicule ${vehicleState}, réveil programmé`);
            
            // Stocker la commande en attente pour l'exécuter après le réveil
            TeslaService.pendingCommand = commandType;
            TeslaChargingLogger.writeToLog(`📝 ${commandName}: Commande "${commandType}" ajoutée à la file d'attente`);
            
            TeslaService.step = TeslaService.STEP_WAKE_UP;
            TeslaService.next_action_time = now;
            TeslaChargingLogger.writeToLog(`⏰ ${commandName}: Réveil programmé immédiatement`);
            setImmediate(TeslaService.checkGetNewData);
            
            // Désactiver le timer de sécurité
            if (safetyTimeoutId) {
                clearTimeout(safetyTimeoutId);
                TeslaChargingLogger.writeToLog(`⏱️ ${commandName}: Timer de sécurité désactivé`);
            }
            
            // Libérer le verrou car on va planifier un réveil à la place
            TeslaService[lockVar] = null;
            TeslaChargingLogger.writeToLog(`🔓 ${commandName}: Libération verrou après opération terminée`);
            
            return false;
        }
        
        TeslaChargingLogger.writeToLog(`✅ ${commandName}: Véhicule actif (${vehicleState}), exécution immédiate`);
        
        // Récupération de l'identifiant véhicule
        const vehicleIdOrVin = TeslaService.last_receive_data?.vin || TeslaService.vehicle_id;
        
        // Validation de l'identifiant véhicule
        if (!vehicleIdOrVin) {
            TeslaChargingLogger.writeToLog(`❌ ${commandName}: Identifiant véhicule manquant`);
            console.error(`TeslaService: ${commandName} impossible - identifiant véhicule manquant`);
            return false;
        }
        
        // Construction de l'endpoint pour le proxy
        const commandEndpoint = `/api/1/vehicles/${vehicleIdOrVin}/command/${endpoint}`;
        TeslaChargingLogger.writeToLog(`🔗 ${commandName}: Endpoint construit: ${commandEndpoint}`);

        TeslaChargingLogger.writeToLog(`${icon} ${commandName}: Vérifie si BLE est activé`);
        const config = configuration_service_1.ConfigurationService.getConfiguration();
        if (config?.tesla?.useBle) {
            try {
                // Mapper la commande API vers la commande BLE équivalente
                let bleCommand = null;
                let bleArgs = [];
                
                // Convertir endpoint et payload en commande BLE appropriée
                switch(endpoint) {
                    case 'charge_start':
                        bleCommand = 'startCharging';
                        break;
                    case 'charge_stop':
                        bleCommand = 'stopCharging';
                        break;
                    case 'set_charging_amps':
                        bleCommand = 'setChargingAmps';
                        bleArgs = [payload.charging_amps];
                        break;
                    case 'charge_limit_soc':
                    case 'set_charge_limit':
                        bleCommand = 'setChargeLimit';
                        bleArgs = [payload.percent];
                        break;
                    case 'charge_port_door_open':
                        bleCommand = 'openChargePort';
                        break;
                    case 'charge_port_door_close':
                        bleCommand = 'closeChargePort';
                        break;
                    case 'door_lock':
                        bleCommand = 'lock';
                        break;
                    case 'door_unlock':
                        bleCommand = 'unlock';
                        break;
                    case 'auto_conditioning_start':
                        bleCommand = 'climateOn';
                        break;
                    case 'auto_conditioning_stop':
                        bleCommand = 'climateOff';
                        break;
                    case 'set_temps':
                        bleCommand = 'setTemperature';
                        bleArgs = [payload.driver_temp];
                        break;
                    case 'actuate_trunk':
                        if (payload.which_trunk === 'front') {
                            // Le frunk n'est pas supporté directement par BLE
                            bleCommand = null;
                        } else if (payload.which_trunk === 'rear') {
                            // Le coffre non plus, ignorer le BLE
                            bleCommand = null;
                        }
                        break;
                    case 'window_control':
                        if (payload.command === 'vent') {
                            bleCommand = 'ventWindows';
                        } else if (payload.command === 'close') {
                            bleCommand = 'closeWindows';
                        }
                        break;
                    default:
                        bleCommand = null; // Commande non supportée en BLE
                        TeslaChargingLogger.writeToLog(`${icon} ${commandName}: Commande ${endpoint} non supportée en BLE`);
                }
                
                // Si commande BLE mappée trouvée, l'exécuter
                if (bleCommand && typeof TeslaControlBLE[bleCommand] === 'function') {
                    TeslaChargingLogger.writeToLog(`${icon} ${commandName}: Tentative d'exécution via BLE (${bleCommand})`);
                    
                    // Exécution de la commande BLE
                    const bleResult = await TeslaControlBLE[bleCommand](...bleArgs);
                    
                    TeslaChargingLogger.writeToLog(`${icon} ${commandName}: Commande BLE exécutée avec succès`);
                    
                    // Si BLE réussi, ne pas appeler l'API
                    if (bleResult) {
                        TeslaChargingLogger.writeToLog(`${icon} ${commandName}: BLE réussi, pas d'appel API nécessaire`);
                        
                        // Simuler la réponse de l'API pour que le reste du code fonctionne
                        const simulatedResponse = {
                            status: 200,
                            data: {
                                response: {
                                    result: true,
                                    reason: "",
                                    _from_ble: true
                                }
                            }
                        };
                        
                        // Continuer avec le traitement comme si c'était une réponse API
                        return simulatedResponse;
                    }
                } else {
                    TeslaChargingLogger.writeToLog(`${icon} ${commandName}: Aucune commande BLE correspondante disponible`);
                }
            } catch (bleError) {
                // Échec BLE, on continu avec l'API
                TeslaChargingLogger.writeToLog(`${icon} ${commandName}: Échec BLE: ${bleError.message}, fallback vers API`);
            }
        }
        
        // Envoi de la commande avec le proxy
        TeslaChargingLogger.writeToLog(`📡 ${commandName}: Envoi commande via proxy local`);
        const requestStartTime = Date.now();
        const response = await TeslaProxy.request('POST', commandEndpoint, payload, { jsonContent: true });
        const requestDuration = Date.now() - requestStartTime;
        
        // Log du résultat
        TeslaChargingLogger.writeToLog(`⏱️ ${commandName}: Réponse reçue en ${requestDuration}ms (statut ${response.status})`);
        
        // Analyse de la réponse
        if (response.status === 200) {
            const vehicleResponse = response.data.response;
            
            if (vehicleResponse.result) {
                // Commande acceptée par le véhicule
                TeslaChargingLogger.writeToLog(`✅ ${commandName}: Commande ${endpoint} acceptée par le véhicule`);
                console.log(`TeslaService: ${commandName} exécuté avec succès`);
                
                // Vérifier l'état plus rapidement après une commande
                TeslaService.next_action_time = now + 15000; // 15 secondes
                TeslaService.step = TeslaService.STEP_GET_DATA;
                TeslaChargingLogger.writeToLog(`⏱️ ${commandName}: Vérification des données planifiée dans 15s`);
                
                // Exécution du callback de succès personnalisé si fourni
                if (onSuccess && typeof onSuccess === 'function') {
                    try {
                        TeslaChargingLogger.writeToLog(`🔄 ${commandName}: Exécution callback onSuccess`);
                        onSuccess(vehicleResponse);
                        TeslaChargingLogger.writeToLog(`✅ ${commandName}: Callback exécuté avec succès`);
                    } catch (callbackError) {
                        TeslaChargingLogger.writeToLog(`⚠️ ${commandName}: Erreur dans callback onSuccess - ${callbackError.message}`);
                        console.error(`TeslaService: Erreur dans callback ${commandName} - ${callbackError.message}`);
                    }
                }
                
                return true;
            } else {
                // Commande refusée par le véhicule
                const reason = vehicleResponse.reason || "Raison inconnue";
                TeslaChargingLogger.writeToLog(`❌ ${commandName}: Commande ${endpoint} refusée - ${reason}`);
                console.error(`TeslaService: ${commandName} refusé par le véhicule - ${reason}`);
                return false;
            }
        } else {
            // Réponse inattendue
            TeslaChargingLogger.writeToLog(`❌ ${commandName}: Réponse inattendue - Statut=${response.status}`);
            console.error(`TeslaService: ${commandName} échec - Statut ${response.status}`);
            return false;
        }
    } catch (error) {
        // Log des erreurs
        TeslaChargingLogger.writeToLog(`❌ ${commandName}: Erreur - ${error.message || "Erreur inconnue"}`);
        console.error(`TeslaService: ${commandName} erreur - ${error.message || "Erreur inconnue"}`);
        
        // Traitement spécifique pour les véhicules endormis (code 408)
        if (error.response && error.response.status === 408) {
            TeslaChargingLogger.writeToLog(`🔄 ${commandName}: Véhicule endormi, tentative de réveil programmée`);
            console.log(`TeslaService: ${commandName} - Véhicule endormi, réveil programmé`);
            
            // Mettre à jour l'état connu
            TeslaService.lastKnownVehicleState = "asleep";
            TeslaService.lastVehicleStateCheck = now;
            
            // Stocker la commande en attente et programmer le réveil
            TeslaService.pendingCommand = commandType;
            TeslaChargingLogger.writeToLog(`📝 ${commandName}: Commande "${commandType}" ajoutée à la file d'attente`);
            
            TeslaService.step = TeslaService.STEP_WAKE_UP;
            TeslaService.next_action_time = now;
            TeslaChargingLogger.writeToLog(`⏰ ${commandName}: Réveil programmé immédiatement`);
        }
        
        return false;
    } finally {
        // Annuler le timeout de sécurité si présent
        if (safetyTimeoutId) {
            clearTimeout(safetyTimeoutId);
            TeslaChargingLogger.writeToLog(`⏱️ ${commandName}: Timer de sécurité désactivé`);
        }
        
        // Libérer le verrou
        if (TeslaService[lockVar] === lockId) {
            TeslaService[lockVar] = null;
            TeslaChargingLogger.writeToLog(`🔓 ${commandName}: Libération verrou après opération terminée`);
        }
        
        // Log de fin d'exécution
        TeslaChargingLogger.writeToLog(`📋 ${commandName}: Fin du traitement de la commande`);
    }
}

;

/**
 * Démarre la charge du véhicule avec vérification préalable de son état
 * @returns {Promise<boolean>} true si la commande a été exécutée avec succès
 */
static async startCharge() {
    TeslaChargingLogger.writeToLog("🔌 CHARGE START: Début de la séquence de démarrage de charge");
    
    try {
        // NOUVEAU: Vérification GLOBALE d'opération en cours
        if (TeslaService.chargingOperationInProgress) {
            TeslaChargingLogger.writeToLog("🔒🔌 CHARGE START: Une opération de charge est déjà en cours, demande ignorée");
            return false;
        }
        
        // NOUVEAU: Vérification du délai global entre opérations
        const now = new Date().getTime();
        if (TeslaService.lastChargingOperation && now - TeslaService.lastChargingOperation < 30000) {
            const secondsAgo = Math.round((now - TeslaService.lastChargingOperation) / 1000);
            TeslaChargingLogger.writeToLog(`⏱️🔌 CHARGE START: Dernière opération il y a ${secondsAgo}s, minimum 30s requis`);
            return false;
        }

        // NOUVEAU: Activer le verrou global
        TeslaService.chargingOperationInProgress = true;
        TeslaService.lastChargingOperation = now;
        
        // NOUVEAU: Timer de sécurité pour libérer le verrou
        TeslaService.chargingOperationTimeout = setTimeout(() => {
            TeslaChargingLogger.writeToLog("⏰🔌 CHARGE START: Libération automatique du verrou global (sécurité)");
            TeslaService.chargingOperationInProgress = false;
        }, 60000); // 60 secondes maximum pour une opération de charge
        
        // PROTECTION: Vérification d'une demande récente de démarrage
        const MIN_START_INTERVAL = 30000; // 30 secondes minimum entre demandes
        
        if (TeslaService.lastChargeStartRequestTime && 
            now - TeslaService.lastChargeStartRequestTime < MIN_START_INTERVAL) {
            
            const secondsSinceLastRequest = Math.round((now - TeslaService.lastChargeStartRequestTime)/1000);
            TeslaChargingLogger.writeToLog(`⛔ CHARGE START: Demande ignorée - dernière requête il y a ${secondsSinceLastRequest}s (min. 30s)`);
            return false;
        }
        
        // États valides pour démarrer la charge
        const validStartStates = ["Disconnected", "Stopped", "Complete", "NoPower"];
        const currentState = TeslaService.last_receive_data.charge_state.charging_state;
        
        // Si déjà en charge, ne rien faire
        if (currentState === "Charging" || currentState === "Starting") {
            TeslaChargingLogger.writeToLog(`✅ CHARGE START: Déjà en charge (${currentState}), aucune action nécessaire`);
            return true;
        }
        
        // Si branché et dans un état valide pour démarrer
        if (validStartStates.includes(currentState)) {
            TeslaChargingLogger.writeToLog(`✅ CHARGE START: Véhicule branché et prêt pour la charge (état: ${currentState})`);
            
            // CORRECTION: Vérifier l'état du véhicule (online/asleep) directement
            const vehicleState = await TeslaService.checkVehicleOnlineStatus();
            
            // Si le véhicule n'est pas en ligne, le réveiller de façon synchrone
            if (vehicleState !== "online") {
                TeslaChargingLogger.writeToLog(`⏰ CHARGE START: Véhicule ${vehicleState}, réveil nécessaire`);
                
                // ATTENTION: Cette méthode wakeUp() intègre déjà un délai de 30 secondes
                await TeslaService.wakeUp();
                
                // IMPORTANT: Le wakeUp inclut déjà une attente de 30 secondes, ne pas en ajouter une autre ici!
                TeslaChargingLogger.writeToLog(`⏱️ CHARGE START: Attente terminée après réveil, véhicule supposé en ligne`);
            }
            
            // MAINTENANT envoyer directement la commande sans passer par la file d'attente
            const result = await TeslaService.executeVehicleCommand({
                commandName: "CHARGE START",
                commandType: "startCharge",
                endpoint: "charge_start",
                icon: "🔋",
                lockVar: "lastStartChargeCall",
                onSuccess: (response) => {
                    TeslaChargingLogger.writeToLog("✅ CHARGE START: Démarrage réussi");
                    
                    // Mettre à jour l'état local
                    if (!TeslaService.last_receive_data) TeslaService.last_receive_data = {};
                    if (!TeslaService.last_receive_data.charge_state) TeslaService.last_receive_data.charge_state = {};
                    TeslaService.last_receive_data.charge_state.charging_state = "Starting";
                    
                    // Programmer une vérification des données
                    TeslaService.step = TeslaService.STEP_GET_DATA;
                    TeslaService.next_action_time = new Date().getTime() + 5000; // 5 secondes
                }
            });
            
            if (result) {
                // ATTENTE OBLIGATOIRE après la commande de démarrage de charge
                TeslaChargingLogger.writeToLog(`⏱️ CHARGE START: Attente obligatoire de 15 secondes après démarrage de charge`);
                await new Promise(resolve => setTimeout(resolve, 15000));
                TeslaChargingLogger.writeToLog(`⏱️ CHARGE START: Fin de l'attente obligatoire après charge`);
            }
            
            TeslaChargingLogger.writeToLog(`📋 CHARGE START: Fin de la séquence de démarrage`);
            return result;
        } else {
            TeslaChargingLogger.writeToLog(`❌ CHARGE START: État actuel "${currentState}" incompatible avec le démarrage de charge`);
            return false;
        }
    } catch (error) {
        TeslaChargingLogger.writeToLog(`❌ CHARGE START ERREUR: ${error.message}`);
        console.error(`Tesla: Erreur lors du démarrage de charge - ${error.message}`);
        return false;
    } finally {
        // NOUVEAU: Libérer le verrou global dans tous les cas
        clearTimeout(TeslaService.chargingOperationTimeout);
        TeslaService.chargingOperationInProgress = false;
        TeslaChargingLogger.writeToLog("🔓🔌 CHARGE START: Verrou global libéré");
    }
}
  
    ;

/**
 * Arrête la charge du véhicule avec vérification préalable de son état
 * @returns {Promise<boolean>} true si la commande a été exécutée avec succès
 */
static async stopCharge() {
    TeslaChargingLogger.writeToLog("🔌 CHARGE STOP: Début de la séquence d'arrêt de charge");
    
    try {
        // NOUVEAU: Vérification GLOBALE d'opération en cours
        if (TeslaService.chargingOperationInProgress) {
            TeslaChargingLogger.writeToLog("🔒🔌 CHARGE STOP: Une opération de charge est déjà en cours, demande ignorée");
            return false;
        }
        
        // NOUVEAU: Vérification du délai global entre opérations
        const now = new Date().getTime();
        if (TeslaService.lastChargingOperation && now - TeslaService.lastChargingOperation < 30000) {
            const secondsAgo = Math.round((now - TeslaService.lastChargingOperation) / 1000);
            TeslaChargingLogger.writeToLog(`⏱️🔌 CHARGE STOP: Dernière opération il y a ${secondsAgo}s, minimum 30s requis`);
            return false;
        }
        
        // NOUVEAU: Activer le verrou global
        TeslaService.chargingOperationInProgress = true;
        TeslaService.lastChargingOperation = now;
        
        // NOUVEAU: Timer de sécurité pour libérer le verrou
        TeslaService.chargingOperationTimeout = setTimeout(() => {
            TeslaChargingLogger.writeToLog("⏰🔌 CHARGE STOP: Libération automatique du verrou global (sécurité)");
            TeslaService.chargingOperationInProgress = false;
        }, 60000);

        // Vérifier si une charge est effectivement en cours
        const currentChargeState = TeslaService.last_receive_data?.charge_state?.charging_state;
        if (currentChargeState !== 'Charging' && currentChargeState !== 'Starting') {
            TeslaChargingLogger.writeToLog(`⚠️ CHARGE STOP: Le véhicule n'est pas en charge (état: ${currentChargeState || 'inconnu'}), commande ignorée`);
            console.log(`TeslaService: Arrêt charge ignoré - pas en cours (${currentChargeState || 'état inconnu'})`);
            return true; // Considéré comme un succès puisque l'état désiré est déjà atteint
        }
        
        // Exécuter la commande via la méthode généralisée
        return TeslaService.executeVehicleCommand({
            commandName: "CHARGE STOP",
            commandType: "stopCharge",
            endpoint: "charge_stop",
            icon: "🔌",
            lockVar: "lastStopChargeCall",
            onSuccess: (response) => {
                TeslaChargingLogger.writeToLog("✅ CHARGE STOP: Commande d'arrêt acceptée par le véhicule");
                
                // Mettre à jour immédiatement le cache pour refléter le nouvel état
                if (TeslaService.last_receive_data && TeslaService.last_receive_data.charge_state) {
                    const previousState = TeslaService.last_receive_data.charge_state.charging_state;
                    TeslaService.last_receive_data.charge_state.charging_state = "Stopped";
                    TeslaChargingLogger.writeToLog(`🔄 CHARGE STOP: État de charge mis à jour dans le cache (${previousState || 'inconnu'} → Stopped)`);
                }
                
                // Mettre à jour l'indicateur d'état de charge immédiatement
                const wasChargingBefore = TeslaService.wasInCharge;
                TeslaService.wasInCharge = false;
                TeslaChargingLogger.writeToLog(`🔄 CHARGE STOP: Indicateur wasInCharge mis à jour (${wasChargingBefore ? 'désactivé' : 'déjà inactif'})`);
                
                // Ajouter une entrée finale à 0W aux données de charge pour marquer la fin
                const now = Math.floor(Date.now() / 1000);
                TeslaService.addToChargeData({
                    timestamp: now,
                    w_charge: 0
                });
                TeslaChargingLogger.writeToLog(`📊 CHARGE STOP: Entrée finale (0W) ajoutée aux données de charge`);
                
                // Si le mode de charge n'est pas normal, retourner en mode normal
                if (TeslaService.chargingMode !== 'normal') {
                    const previousMode = TeslaService.chargingMode;
                    TeslaService.chargingMode = 'normal';
                    
                    // Sauvegarder le changement dans la configuration
                    const config = configuration_service_1.ConfigurationService.getConfiguration();
                    config.tesla.chargingMode = 'normal';
                    configuration_service_1.ConfigurationService.saveConfiguration(config);
                    
                    TeslaChargingLogger.writeToLog(`🔌 CHARGE STOP: Mode de charge réinitialisé (${previousMode} → normal)`);
                }
                
                // Programmation d'une vérification rapide pour confirmer l'arrêt effectif
                TeslaService.step = TeslaService.STEP_GET_DATA;
                TeslaService.next_action_time = Date.now() + 10000; // 10 secondes
                TeslaChargingLogger.writeToLog(`⏱️ CHARGE STOP: Vérification de confirmation programmée dans 10s`);
            }
        });
    } catch (error) {
        TeslaChargingLogger.writeToLog(`❌ CHARGE STOP ERREUR: ${error.message}`);
        console.error(`TeslaService: Erreur lors de l'arrêt de la charge: ${error.message}`);
        return false;
    } finally {
        // NOUVEAU: Libérer le verrou dans tous les cas
        clearTimeout(TeslaService.chargingOperationTimeout);
        TeslaService.chargingOperationInProgress = false;
        TeslaChargingLogger.writeToLog("🔓🔌 CHARGE STOP: Verrou global libéré");
        TeslaChargingLogger.writeToLog("📋 CHARGE STOP: Fin de la séquence d'arrêt");
    }
}

;

// À ajouter à la classe TeslaService

/**
 * Compare et affiche les différences entre API et MQTT de façon structurée
 * Cette méthode est utile pour le diagnostic des problèmes de données
 */
static compareDataSources() {
    TeslaChargingLogger.writeToLog("🔍 COMPARAISON: Début analyse comparative des sources de données");
    console.log("============= COMPARAISON TESLA API vs MQTT =============");
    
    try {
        // 1. Récupérer données API
        const apiData = this.last_receive_data || {};
        
        // 2. Récupérer données MQTT
        const mqttData = TeslaService.getMqttVehicleData();
        
        // Log de la présence des données
        TeslaChargingLogger.writeToLog(`🔍 COMPARAISON: Données API ${Object.keys(apiData).length > 0 ? 'présentes' : 'ABSENTES'}, Données MQTT ${Object.keys(mqttData).length > 0 ? 'présentes' : 'ABSENTES'}`);
        
        // 3. Créer tableaux comparatifs
        console.log("\n=== SOURCE ET MÉTADONNÉES ===");
        const apiTimestamp = apiData._timestamp || apiData.timestampMs || 0;
        const mqttTimestamp = mqttData._timestamp || 0;
        
        const apiAge = apiTimestamp ? Math.round((Date.now() - apiTimestamp)/1000) : "N/A";
        const mqttAge = mqttTimestamp ? Math.round((Date.now() - mqttTimestamp)/1000) : "N/A";
        
        // Log des métadonnées
        TeslaChargingLogger.writeToLog(`🕒 COMPARAISON MÉTADONNÉES: API (age=${apiAge}s), MQTT (age=${mqttAge}s)`);
        
        console.table({
            "API Tesla": {
                timestamp: apiTimestamp ? new Date(apiTimestamp).toISOString() : "N/A",
                source: apiData._source || "api",
                age_seconds: apiAge
            },
            "MQTT": {
                timestamp: mqttTimestamp ? new Date(mqttTimestamp).toISOString() : "N/A",
                source: "mqtt",
                age_seconds: mqttAge,
                last_update: Tesla.lastUpdateTime ? new Date(TeslaMqttClient.lastUpdateTime).toISOString() : "N/A",
                is_connected: TeslaMqttClient.isConnected ? "Oui" : "Non"
            }
        });
        
        // Analyse de fraîcheur
        const freshApi = apiAge !== "N/A" && apiAge < 300; // Moins de 5 minutes
        const freshMqtt = mqttAge !== "N/A" && mqttAge < 300; // Moins de 5 minutes
        
        TeslaChargingLogger.writeToLog(`🕒 COMPARAISON FRAÎCHEUR: API ${freshApi ? 'récente' : 'OBSOLÈTE'}, MQTT ${freshMqtt ? 'récente' : 'OBSOLÈTE'}`);
        
        console.log("\n=== ÉTAT DE CHARGE ===");
        const apiChargeData = {
            battery_level: apiData.charge_state?.battery_level || "N/A",
            usable_battery_level: apiData.charge_state?.usable_battery_level || "N/A",
            charging_state: apiData.charge_state?.charging_state || "N/A",
            charge_limit_soc: apiData.charge_state?.charge_limit_soc || "N/A",
            charge_amps: apiData.charge_state?.charge_amps || "N/A",
            charge_power: apiData.charge_state?.charger_power || "N/A"
        };
        
        const mqttChargeData = {
            battery_level: mqttData.charge_state?.battery_level || "N/A",
            usable_battery_level: mqttData.charge_state?.usable_battery_level || "N/A",
            charging_state: mqttData.charge_state?.charging_state || "N/A",
            charge_limit_soc: mqttData.charge_state?.charge_limit_soc || "N/A",
            charge_amps: mqttData.charge_state?.charge_amps || "N/A",
            charge_power: mqttData.charge_state?.charger_power || "N/A"
        };
        
        // Comparer les données de charge et identifier les divergences
        const chargeDifferences = [];
        Object.keys(apiChargeData).forEach(key => {
            if (apiChargeData[key] !== mqttChargeData[key] && 
                apiChargeData[key] !== "N/A" && 
                mqttChargeData[key] !== "N/A") {
                chargeDifferences.push(`${key}: API=${apiChargeData[key]}, MQTT=${mqttChargeData[key]}`);
            }
        });
        
        if (chargeDifferences.length > 0) {
            TeslaChargingLogger.writeToLog(`⚠️ COMPARAISON CHARGE: ${chargeDifferences.length} différences détectées - ${chargeDifferences.join('; ')}`);
        } else {
            TeslaChargingLogger.writeToLog("✅ COMPARAISON CHARGE: Données cohérentes entre API et MQTT");
        }
        
        console.table({
            "API Tesla": apiChargeData,
            "MQTT": mqttChargeData
        });
        
        console.log("\n=== CLIMAT ===");
        const apiClimateData = {
            inside_temp: apiData.climate_state?.inside_temp || "N/A",
            outside_temp: apiData.climate_state?.outside_temp || "N/A",
            is_climate_on: apiData.climate_state?.is_climate_on || "N/A",
            is_preconditioning: apiData.climate_state?.is_preconditioning || "N/A",
            fan_status: apiData.climate_state?.fan_status || "N/A"
        };
        
        const mqttClimateData = {
            inside_temp: mqttData.climate_state?.inside_temp || "N/A",
            outside_temp: mqttData.climate_state?.outside_temp || "N/A",
            is_climate_on: mqttData.climate_state?.is_climate_on || "N/A",
            is_preconditioning: mqttData.climate_state?.is_preconditioning || "N/A",
            fan_status: mqttData.climate_state?.fan_status || "N/A"
        };
        
        // Comparer les données de climat
        const climateDifferences = [];
        Object.keys(apiClimateData).forEach(key => {
            if (apiClimateData[key] !== mqttClimateData[key] &&
                apiClimateData[key] !== "N/A" &&
                mqttClimateData[key] !== "N/A") {
                climateDifferences.push(`${key}: API=${apiClimateData[key]}, MQTT=${mqttClimateData[key]}`);
            }
        });
        
        if (climateDifferences.length > 0) {
            TeslaChargingLogger.writeToLog(`⚠️ COMPARAISON CLIMAT: ${climateDifferences.length} différences détectées - ${climateDifferences.join('; ')}`);
        } else {
            TeslaChargingLogger.writeToLog("✅ COMPARAISON CLIMAT: Données cohérentes entre API et MQTT");
        }
        
        console.table({
            "API Tesla": apiClimateData,
            "MQTT": mqttClimateData
        });
        
        console.log("\n=== ÉTAT VÉHICULE ===");
        const apiVehicleData = {
            state: apiData.state || "N/A",
            vehicle_name: apiData.display_name || "N/A",
            odometer: apiData.vehicle_state?.odometer || "N/A",
            locked: apiData.vehicle_state?.locked || "N/A",
            sentry_mode: apiData.vehicle_state?.sentry_mode || "N/A",
            windows_open: apiData.vehicle_state?.windows_open || "N/A"
        };
        
        const mqttVehicleData = {
            state: mqttData.state || "N/A",
            vehicle_name: mqttData.display_name || "N/A",
            odometer: mqttData.vehicle_state?.odometer || "N/A",
            locked: mqttData.vehicle_state?.locked || "N/A",
            sentry_mode: mqttData.vehicle_state?.sentry_mode || "N/A",
            windows_open: mqttData.vehicle_state?.windows_open || "N/A"
        };
        
        // Comparer les données d'état du véhicule
        const vehicleDifferences = [];
        Object.keys(apiVehicleData).forEach(key => {
            if (apiVehicleData[key] !== mqttVehicleData[key] &&
                apiVehicleData[key] !== "N/A" &&
                mqttVehicleData[key] !== "N/A") {
                vehicleDifferences.push(`${key}: API=${apiVehicleData[key]}, MQTT=${mqttVehicleData[key]}`);
            }
        });
        
        if (vehicleDifferences.length > 0) {
            TeslaChargingLogger.writeToLog(`⚠️ COMPARAISON VÉHICULE: ${vehicleDifferences.length} différences détectées - ${vehicleDifferences.join('; ')}`);
        } else {
            TeslaChargingLogger.writeToLog("✅ COMPARAISON VÉHICULE: Données cohérentes entre API et MQTT");
        }
        
        console.table({
            "API Tesla": apiVehicleData,
            "MQTT": mqttVehicleData
        });
        
        // Analyse globale des types de données disponibles
        console.log("\n=== DISPONIBILITÉ DES MODULES DE DONNÉES ===");
        const dataModules = ['charge_state', 'climate_state', 'drive_state', 'vehicle_state', 'gui_settings'];
        
        const moduleAvailability = {};
        dataModules.forEach(module => {
            moduleAvailability[module] = {
                'API': apiData[module] ? "✓" : "✗",
                'MQTT': mqttData[module] ? "✓" : "✗",
            };
        });
        
        console.table(moduleAvailability);
        
        // Log des modules disponibles
        const apiModules = dataModules.filter(m => apiData[m]).join(', ');
        const mqttModules = dataModules.filter(m => mqttData[m]).join(', ');
        TeslaChargingLogger.writeToLog(`📊 COMPARAISON MODULES: API [${apiModules || 'aucun'}], MQTT [${mqttModules || 'aucun'}]`);
        
        // Déterminer la meilleure source de données selon fraîcheur et complétude
        let bestSource = "api"; // Par défaut
        
        if (freshMqtt && !freshApi) {
            bestSource = "mqtt";
            TeslaChargingLogger.writeToLog("📊 ANALYSE: MQTT recommandé car plus récent que l'API");
        } else if (freshApi && !freshMqtt) {
            bestSource = "api";
            TeslaChargingLogger.writeToLog("📊 ANALYSE: API recommandé car plus récente que MQTT");
        } else if (freshApi && freshMqtt) {
            // Les deux sont récents, comparer la complétude
            const apiModuleCount = dataModules.filter(m => apiData[m]).length;
            const mqttModuleCount = dataModules.filter(m => mqttData[m]).length;
            
            if (mqttModuleCount > apiModuleCount) {
                bestSource = "mqtt";
                TeslaChargingLogger.writeToLog("📊 ANALYSE: MQTT recommandé car plus complet que l'API");
            } else {
                bestSource = "api";
                TeslaChargingLogger.writeToLog("📊 ANALYSE: API recommandé car plus complet ou égal à MQTT");
            }
        } else {
            // Aucune source n'est récente, prioriser celle avec le plus de données
            const apiModuleCount = dataModules.filter(m => apiData[m]).length;
            const mqttModuleCount = dataModules.filter(m => mqttData[m]).length;
            
            if (mqttModuleCount > apiModuleCount) {
                bestSource = "mqtt";
                TeslaChargingLogger.writeToLog("⚠️ ANALYSE: MQTT recommandé par défaut mais données obsolètes");
            } else {
                bestSource = "api";
                TeslaChargingLogger.writeToLog("⚠️ ANALYSE: API recommandé par défaut mais données obsolètes");
            }
        }
        
        console.log(`\n=== RECOMMANDATION ===\nUtiliser la source: ${bestSource.toUpperCase()}\n`);
        
        // Sauvegarder dans un fichier pour analyse ultérieure
        this.dumpCompleteData();
        
        TeslaChargingLogger.writeToLog("✅ COMPARAISON: Analyse comparative des sources de données terminée");
        return bestSource;
        
    } catch (error) {
        TeslaChargingLogger.writeToLog(`❌ COMPARAISON ERREUR: ${error.message}`);
        console.error("Erreur lors de la comparaison des sources de données:", error.message);
        return "api"; // Valeur par défaut en cas d'erreur
    }
}

;
  
/**
 * Sauvegarde les structures complètes de données pour diagnostic
 * Cette méthode est utilisée pour l'analyse et le débogage approfondi
 */
static dumpCompleteData() {
    TeslaChargingLogger.writeToLog("📦 DUMP: Démarrage de la sauvegarde des données complètes");
    
    try {
        const fs = require('fs');
        const path = require('path');
        const logDir = path.join(__dirname, '../logs/tesla-data');
        
        // Créer le dossier logs/tesla-data s'il n'existe pas (structure plus organisée)
        if (!fs.existsSync(logDir)) {
            TeslaChargingLogger.writeToLog(`📂 DUMP: Création du répertoire ${logDir}`);
            fs.mkdirSync(logDir, { recursive: true });
        }
        
        // Générer un timestamp formaté pour le nom de fichier
        const timestamp = new Date().toISOString()
            .replace(/:/g, '-')          // Remplacer les : par des - pour Windows
            .replace(/\..+/, '')         // Supprimer les millisecondes
            .replace('T', '_');          // Remplacer T par _ pour lisibilité
        
        TeslaChargingLogger.writeToLog(`⏱️ DUMP: Utilisation du timestamp ${timestamp} pour les fichiers`);
        
        // Récupération des données API
        const apiData = this.last_receive_data || {};
        
        // Récupération des données MQTT
        const mqttData = TeslaService.getMqttVehicleData();
        
        // Ajout de métadonnées aux données exportées
        const apiExport = {
            _meta: {
                exportTime: new Date().toISOString(),
                dataAge: apiData._timestamp ? Math.round((Date.now() - apiData._timestamp)/1000) + 's' : 'inconnu',
                source: apiData._source || 'api'
            },
            data: apiData
        };
        
        const mqttExport = {
            _meta: {
                exportTime: new Date().toISOString(),
                dataAge: mqttData._timestamp ? Math.round((Date.now() - mqttData._timestamp)/1000) + 's' : 'inconnu',
                source: 'mqtt',
                clientConnected: TeslaMqttClient.isConnected || false,
                lastUpdateTime: TeslaMqttClient.lastUpdateTime || null
            },
            data: mqttData
        };
        
        // Chemin des fichiers
        const apiFilePath = path.join(logDir, `tesla-api-data-${timestamp}.json`);
        const mqttFilePath = path.join(logDir, `tesla-mqtt-data-${timestamp}.json`);
        
        // Écriture des fichiers avec formatage JSON pour lisibilité
        TeslaChargingLogger.writeToLog(`💾 DUMP API: Écriture des données API dans ${apiFilePath}`);
        fs.writeFileSync(apiFilePath, JSON.stringify(apiExport, null, 2));
        
        TeslaChargingLogger.writeToLog(`💾 DUMP MQTT: Écriture des données MQTT dans ${mqttFilePath}`);
        fs.writeFileSync(mqttFilePath, JSON.stringify(mqttExport, null, 2));
        
        // Créer un fichier de résumé pour référence rapide
        const summaryData = {
            timestamp: new Date().toISOString(),
            apiDataAvailable: Object.keys(apiData).length > 0,
            mqttDataAvailable: Object.keys(mqttData).length > 0,
            batteryLevel: {
                api: apiData.charge_state?.battery_level || null,
                mqtt: mqttData.charge_state?.battery_level || null
            },
            chargingState: {
                api: apiData.charge_state?.charging_state || null,
                mqtt: mqttData.charge_state?.charging_state || null
            },
            usableBatteryLevel: {
                api: apiData.charge_state?.usable_battery_level || null,
                mqtt: mqttData.charge_state?.usable_battery_level || null 
            },
            location: {
                api: apiData.drive_state ? {
                    lat: apiData.drive_state.latitude?.toFixed(4) || null,
                    lon: apiData.drive_state.longitude?.toFixed(4) || null
                } : null,
                mqtt: mqttData.drive_state ? {
                    lat: mqttData.drive_state.latitude?.toFixed(4) || null,
                    lon: mqttData.drive_state.longitude?.toFixed(4) || null
                } : null
            },
            temperatures: {
                api: {
                    inside: apiData.climate_state?.inside_temp || null,
                    outside: apiData.climate_state?.outside_temp || null
                },
                mqtt: {
                    inside: mqttData.climate_state?.inside_temp || null,
                    outside: mqttData.climate_state?.outside_temp || null
                }
            },
            vehicleState: {
                api: {
                    locked: apiData.vehicle_state?.locked || null,
                    sentryMode: apiData.vehicle_state?.sentry_mode || null
                },
                mqtt: {
                    locked: mqttData.vehicle_state?.locked || null,
                    sentryMode: mqttData.vehicle_state?.sentry_mode || null
                }
            }
        };
        
        const summaryFilePath = path.join(logDir, `tesla-summary-${timestamp}.json`);
        TeslaChargingLogger.writeToLog(`📊 DUMP RÉSUMÉ: Écriture du résumé dans ${summaryFilePath}`);
        fs.writeFileSync(summaryFilePath, JSON.stringify(summaryData, null, 2));
        
        // Journaliser les informations sur les fichiers exportés
        console.log(`TeslaService: Données sauvegardées dans le dossier ${logDir}`);
        console.log(`- API: ${path.basename(apiFilePath)}`);
        console.log(`- MQTT: ${path.basename(mqttFilePath)}`);
        console.log(`- Résumé: ${path.basename(summaryFilePath)}`);
        
        TeslaChargingLogger.writeToLog("✅ DUMP: Sauvegarde des données complète terminée avec succès");
        
        // Retourner les chemins des fichiers pour éventuel traitement ultérieur
        return {
            directory: logDir,
            files: {
                api: apiFilePath,
                mqtt: mqttFilePath,
                summary: summaryFilePath
            }
        };
    } catch (error) {
        // Gestion des erreurs
        TeslaChargingLogger.writeToLog(`❌ DUMP ERREUR: ${error.message}`);
        console.error("Erreur lors de la sauvegarde des données:", error.message);
        
        // Si erreur de permission, donner des conseils spécifiques
        if (error.code === 'EACCES') {
            TeslaChargingLogger.writeToLog("🔒 DUMP ERREUR PERMISSION: Vérifiez les droits d'accès au dossier logs");
            console.error("Erreur de permission: Vérifiez les droits d'accès au dossier logs");
        }
        
        // Si erreur d'espace disque
        if (error.code === 'ENOSPC') {
            TeslaChargingLogger.writeToLog("💾 DUMP ERREUR STOCKAGE: Espace disque insuffisant");
            console.error("Erreur: Espace disque insuffisant pour sauvegarder les données");
        }
        
        return null;
    }
}

;

/**
 * Démarre la climatisation avec vérification préalable de l'état du véhicule
 * @returns {Promise<boolean>} true si la commande a été exécutée avec succès
 */
static async startClimatisation() {
    TeslaChargingLogger.writeToLog("🌡️ CLIMAT: Début séquence démarrage climatisation");
    
    try {
        // Vérifier si la climatisation est déjà active
        const isClimateOn = TeslaService.last_receive_data?.climate_state?.is_climate_on;
        if (isClimateOn === true) {
            TeslaChargingLogger.writeToLog("⚠️ CLIMAT: La climatisation est déjà active, commande ignorée");
            console.log("TeslaService: Démarrage climatisation ignoré - déjà active");
            return true; // Considéré comme un succès puisque l'état désiré est déjà atteint
        }
        
        // Exécuter la commande via la méthode généralisée
        return TeslaService.executeVehicleCommand({
            commandName: "CLIMAT",
            commandType: "startClimatisation",
            endpoint: "auto_conditioning_start",
            icon: "🌡️",
            lockVar: "lastStartClimateCall",
            onSuccess: (response) => {
                TeslaChargingLogger.writeToLog("✅ CLIMAT: Climatisation démarrée avec succès");
                
                // Mettre à jour immédiatement le cache pour refléter le nouvel état
                if (!TeslaService.last_receive_data) {
                    TeslaChargingLogger.writeToLog("🔄 CLIMAT: Initialisation du cache de données");
                    TeslaService.last_receive_data = {};
                }
                
                if (!TeslaService.last_receive_data.climate_state) {
                    TeslaChargingLogger.writeToLog("🔄 CLIMAT: Initialisation du cache climate_state");
                    TeslaService.last_receive_data.climate_state = {};
                }
                
                // Mise à jour des propriétés d'état de la climatisation
                TeslaService.last_receive_data.climate_state.is_climate_on = true;
                TeslaService.last_receive_data.climate_state.is_preconditioning = true;
                TeslaService.last_receive_data.climate_state.fan_status = 4; // Valeur approximative pour indiquer le fonctionnement
                
                TeslaChargingLogger.writeToLog("💾 CLIMAT: État de la climatisation mis à jour dans le cache");
                
                // Log pour diagnostic
                console.log("TeslaService: Climatisation démarrée avec succès");
                
                // Programmer une vérification des données pour confirmer le changement
                TeslaService.step = TeslaService.STEP_GET_DATA;
                TeslaService.next_action_time = new Date().getTime() + 5000; // 5 secondes
                TeslaChargingLogger.writeToLog("⏱️ CLIMAT: Vérification programmée dans 5s");
            }
        });
    } catch (error) {
        TeslaChargingLogger.writeToLog(`❌ CLIMAT ERREUR: ${error.message}`);
        console.error(`TeslaService: Erreur lors du démarrage de la climatisation: ${error.message}`);
        return false;
    } finally {
        TeslaChargingLogger.writeToLog("📋 CLIMAT: Fin de la séquence de démarrage climatisation");
    }
}

;

/**
 * Arrête la climatisation avec vérification préalable de l'état du véhicule
 * @returns {Promise<boolean>} true si la commande a été exécutée avec succès
 */
static async stopClimatisation() {
    TeslaChargingLogger.writeToLog("🌡️ CLIMAT STOP: Début séquence arrêt climatisation");
    
    try {
        // Vérifier si la climatisation est déjà inactive
        const isClimateOn = TeslaService.last_receive_data?.climate_state?.is_climate_on;
        if (isClimateOn === false) {
            TeslaChargingLogger.writeToLog("⚠️ CLIMAT STOP: La climatisation est déjà inactive, commande ignorée");
            console.log("TeslaService: Arrêt climatisation ignoré - déjà inactive");
            return true; // Considéré comme un succès puisque l'état désiré est déjà atteint
        }
        
        // Exécuter la commande via la méthode généralisée
        return TeslaService.executeVehicleCommand({
            commandName: "CLIMAT STOP",
            commandType: "stopClimatisation",
            endpoint: "auto_conditioning_stop",
            icon: "🌡️",
            lockVar: "lastStopClimateCall",
            onSuccess: (response) => {
                TeslaChargingLogger.writeToLog("✅ CLIMAT STOP: Climatisation arrêtée avec succès");
                
                // Mettre à jour immédiatement le cache pour refléter le nouvel état
                if (!TeslaService.last_receive_data) {
                    TeslaChargingLogger.writeToLog("🔄 CLIMAT STOP: Initialisation du cache de données");
                    TeslaService.last_receive_data = {};
                }
                
                if (!TeslaService.last_receive_data.climate_state) {
                    TeslaChargingLogger.writeToLog("🔄 CLIMAT STOP: Initialisation du cache climate_state");
                    TeslaService.last_receive_data.climate_state = {};
                }
                
                // Mise à jour des propriétés d'état de la climatisation
                const previousState = TeslaService.last_receive_data.climate_state.is_climate_on;
                TeslaService.last_receive_data.climate_state.is_climate_on = false;
                TeslaService.last_receive_data.climate_state.is_preconditioning = false;
                TeslaService.last_receive_data.climate_state.fan_status = 0;
                
                TeslaChargingLogger.writeToLog(`💾 CLIMAT STOP: État mis à jour dans le cache (is_climate_on: ${previousState} → false)`);
                
                // Log pour diagnostic
                console.log("TeslaService: Climatisation arrêtée avec succès");
                
                // Programmer une vérification des données pour confirmer le changement
                TeslaService.step = TeslaService.STEP_GET_DATA;
                TeslaService.next_action_time = new Date().getTime() + 5000; // 5 secondes
                TeslaChargingLogger.writeToLog("⏱️ CLIMAT STOP: Vérification programmée dans 5s");
            }
        });
    } catch (error) {
        TeslaChargingLogger.writeToLog(`❌ CLIMAT STOP ERREUR: ${error.message}`);
        console.error(`TeslaService: Erreur lors de l'arrêt de la climatisation: ${error.message}`);
        return false;
    } finally {
        TeslaChargingLogger.writeToLog("📋 CLIMAT STOP: Fin de la séquence d'arrêt climatisation");
    }
}

    ;
 
/**
 * Active le mode sentinelle du véhicule via le proxy local
 * @returns {Promise<boolean>} true si la commande a été exécutée avec succès
 */
static async startSentinel() {
    TeslaChargingLogger.writeToLog("👁️ SENTINEL: Début séquence d’activation du mode sentinelle");

    try {
        // Vérifier si le mode sentinelle est déjà actif
        const isSentryOn = TeslaService.last_receive_data?.vehicle_state?.sentry_mode;
        if (isSentryOn === true) {
            TeslaChargingLogger.writeToLog("⚠️ SENTINEL: Mode sentinelle déjà actif, commande ignorée");
            console.log("TeslaService: Activation sentinelle ignorée - déjà actif");
            return true;
        }

        return TeslaService.executeVehicleCommand({
            commandName: "SENTINELLE",
            commandType: "startSentinel",
            endpoint: "set_sentry_mode",
            payload: { on: true },
            icon: "👁️",
            lockVar: "lastStartSentinelCall",
            onSuccess: (response) => {
                TeslaChargingLogger.writeToLog("✅ SENTINEL: Mode sentinelle activé avec succès");

                // Initialiser le cache si nécessaire
                if (TeslaService.last_receive_data && !TeslaService.last_receive_data.vehicle_state) {
                    TeslaService.last_receive_data.vehicle_state = {};
                }

                if (TeslaService.last_receive_data?.vehicle_state) {
                    const prevState = TeslaService.last_receive_data.vehicle_state.sentry_mode;
                    TeslaService.last_receive_data.vehicle_state.sentry_mode = true;
                    TeslaChargingLogger.writeToLog(`🔄 SENTINEL: État mis à jour (sentry_mode: ${prevState} → true)`);
                }

                // Programmer une vérification des données pour confirmer l’activation
                TeslaService.step = TeslaService.STEP_GET_DATA;
                TeslaService.next_action_time = Date.now() + 5000; // 5 secondes
                TeslaChargingLogger.writeToLog("⏱️ SENTINEL: Vérification programmée dans 5s");
            }
        });
    } catch (error) {
        TeslaChargingLogger.writeToLog(`❌ SENTINEL ERREUR: ${error.message}`);
        console.error(`TeslaService: Erreur lors de l’activation du mode sentinelle: ${error.message}`);
        return false;
    } finally {
        TeslaChargingLogger.writeToLog("📋 SENTINEL: Fin de la séquence d’activation du mode sentinelle");
    }
}

;

/**
 * Définit l'ampérage de charge du véhicule
 * @param {number} amps - Ampérage de charge (5-32)
 * @returns {Promise<boolean>} true si la commande a été exécutée avec succès
 */
static async setChargeAmps(amps) {
    // Début de l'exécution de setChargeAmps
    TeslaChargingLogger.writeToLog("🔄 SET CHARGE AMPS: Début de l'exécution de setChargeAmps");

    // 1. Validation de l'ampérage fourni
    TeslaChargingLogger.writeToLog("SET CHARGE AMPS: Validation de l'ampérage fourni");
    if (typeof amps !== 'number' || amps < 5 || amps > 32) {
        TeslaChargingLogger.writeToLog(`❌ SET CHARGE AMPS: Ampérage invalide (${amps}). Valeur doit être entre 5 et 32A.`);
        return false;
    }

    // 2. Arrondir l'ampérage au nombre entier
    const originalAmps = amps;
    amps = Math.round(amps);
    TeslaChargingLogger.writeToLog(`SET CHARGE AMPS: Arrondi de l'ampérage de ${originalAmps} à ${amps}A`);

    // 3. Vérifier l'ampérage ACTUEL (charger_actual_current) plutôt que la consigne (charge_amps)
    TeslaChargingLogger.writeToLog("SET CHARGE AMPS: Vérification de l'ampérage actuel dans les données reçues");
    const currentActualAmps = TeslaService.last_receive_data?.charge_state?.charger_actual_current;
    const currentSetAmps = TeslaService.last_receive_data?.charge_state?.charge_amps;
    
    TeslaChargingLogger.writeToLog(`SET CHARGE AMPS: Ampérage actuel=${currentActualAmps}A, consigne actuelle=${currentSetAmps}A, nouvelle consigne=${amps}A`);
    
    // Ne pas ignorer si l'ampérage réel est différent de la consigne demandée
    if (currentActualAmps === amps) {
        TeslaChargingLogger.writeToLog(`🔌 SET CHARGE AMPS: Ampérage réel déjà à ${amps}A, requête ignorée`);
        return true;
    }

    // 4. Mise à jour du timestamp du dernier changement
    TeslaChargingLogger.writeToLog("SET CHARGE AMPS: Mise à jour du timestamp du dernier changement d'ampérage");
    TeslaService.lastChangeChargingAmps = new Date().getTime();
    TeslaChargingLogger.writeToLog(`SET CHARGE AMPS: Timestamp mis à jour à ${TeslaService.lastChangeChargingAmps}`);

    // 5. Exécution de la commande véhicule pour définir l'ampérage
    TeslaChargingLogger.writeToLog("SET CHARGE AMPS: Exécution de la commande pour définir l'ampérage");
    return TeslaService.executeVehicleCommand({
        commandName: "AMPÉRAGE CHARGE",
        commandType: "setChargeAmps",
        endpoint: "set_charging_amps",
        payload: { charging_amps: amps },
        icon: "🔌",
        lockVar: "lastSetChargeAmpsCall",
        onSuccess: (response) => {
            TeslaChargingLogger.writeToLog("SET CHARGE AMPS - onSuccess: Réception de la réponse du véhicule");

            // 5.1. Mise à jour du cache avec la nouvelle valeur d'ampérage
            if (TeslaService.last_receive_data && TeslaService.last_receive_data.charge_state) {
                TeslaService.last_receive_data.charge_state.charge_amps = amps;
                TeslaService.last_receive_data.charge_state.charge_current_request = amps;
                TeslaChargingLogger.writeToLog(`🔌 SET CHARGE AMPS - onSuccess: Cache mis à jour avec l'ampérage ${amps}A`);
            } else {
                TeslaChargingLogger.writeToLog("SET CHARGE AMPS - onSuccess: Données de charge non initialisées dans le cache");
            }

            // 5.2. Programmation de la prochaine vérification des données pour confirmer le changement
            TeslaChargingLogger.writeToLog("SET CHARGE AMPS - onSuccess: Programmation de la vérification des données");
            TeslaService.step = TeslaService.STEP_GET_DATA;
            TeslaService.next_action_time = new Date().getTime() + 5000; // 5 secondes
            TeslaChargingLogger.writeToLog(`SET CHARGE AMPS - onSuccess: Prochaine action planifiée dans 5 secondes à ${TeslaService.next_action_time}`);
        }
    });
}

    
     /**
     * Obtenir les dernières données reçues sur le véhicule (avec le timestampMs qui indique quand ces données sont arrivées)
     */
     static getLastReceiveData() {
        // Démarrage de l'exécution de getLastReceiveData
        TeslaChargingLogger.writeToLog("GET LAST RECEIVE DATA: Début de l'exécution de getLastReceiveData");
    
        // Mise à jour du chargingMode dans les données reçues
        TeslaChargingLogger.writeToLog("GET LAST RECEIVE DATA: Mise à jour du 'chargingMode' dans last_receive_data");
        TeslaService.last_receive_data.chargingMode = TeslaService.chargingMode;
        TeslaChargingLogger.writeToLog(`GET LAST RECEIVE DATA: 'chargingMode' mis à jour à ${TeslaService.chargingMode}`);
    
        // Retour des données mises à jour
        TeslaChargingLogger.writeToLog("GET LAST RECEIVE DATA: Retour des données mises à jour");
        return TeslaService.last_receive_data;
    }
    
    ;
    /**
     * Remove bad caracteres of a token
     */
    static removeTokenBadCaracteres(token) {
        var newToken = "";
        for (let i = 0; i < token.length; i++) {
            if (token.charAt(i) != String.fromCharCode(10) && token.charAt(i) != String.fromCharCode(13) && token.charAt(i) != " ") {
                newToken += token.charAt(i);
            }
        }
        return newToken;
    }
    /**
     * Log information
     */
    static log(text) {
        // LOG FUNCTION: Début de l'exécution de la fonction log
        console.info("LOG FUNCTION: Début de l'exécution de log");
    
        // Création de l'horodatage et construction du message de log
        let d = new Date();
        const logMessage = d.toLocaleTimeString() + " - " + text;
        console.info("LOG FUNCTION: Message construit: " + logMessage);
    
        // Affichage du message final dans la console
        console.log(logMessage);
    
        // LOG FUNCTION: Fin de l'exécution de la fonction log
        console.info("LOG FUNCTION: Fin de l'exécution de log");
    }
    
    ;
/**
 * Set the Tesla charging mode and save it to configuration
 * @param {string} teslaChargingMode - Mode de charge (normal, solar, solarPlus, etc.)
 */
static async setMode(teslaChargingMode) {
    // Début de l'exécution de setMode avec le mode spécifié
    TeslaChargingLogger.writeToLog(`MODE CHARGE: Début de l'exécution de setMode avec le mode "${teslaChargingMode}"`);

    // Mise à jour du mode de charge dans TeslaService
    TeslaChargingLogger.writeToLog("MODE CHARGE: Mise à jour de TeslaService.chargingMode");
    TeslaService.chargingMode = teslaChargingMode;
    TeslaChargingLogger.writeToLog(`MODE CHARGE: TeslaService.chargingMode mis à jour à "${teslaChargingMode}"`);

    // ÉTAPE 1: Synchronisation des tokens
    const TeslaProxy = require('./tesla.proxy');
    await TeslaProxy.checkTokenConsistency();
    TeslaChargingLogger.writeToLog("✅ TOKEN: Cohérence des tokens vérifiée");

    // ÉTAPE 2: FORCER la relecture du fichier config.json
    const fs = require('fs');
    const path = require('path');
    const configPath = path.join(__dirname, '../assets/conf/config.json');
    const configFromFile = JSON.parse(fs.readFileSync(configPath, 'utf8'));
    configFromFile.tesla.token = TeslaProxy.CONFIG.auth.refreshToken;
    
    // ÉTAPE 3: Mise à jour du mode de charge
    configFromFile.tesla.chargingMode = teslaChargingMode;
    
    // ÉTAPE 4: Sauvegarde directe pour éviter tout problème de cache
    fs.writeFileSync(configPath, JSON.stringify(configFromFile, null, 2));
    TeslaChargingLogger.writeToLog(`✅ CONFIG: Configuration sauvegardée avec token ${configFromFile.tesla.token.substring(0,8)}... et mode ${teslaChargingMode}`);

    // ÉTAPE 5: Notification au ConfigurationService qu'il doit recharger
    try {
        // Tenter de notifier ConfigurationService de recharger son cache
        configuration_service_1.ConfigurationService.reloadConfiguration();
    } catch (e) {
        // Cette fonction n'existe peut-être pas, ce n'est pas grave
    }

    TeslaChargingLogger.writeToLog(`MODE CHARGE: Changement de mode en "${teslaChargingMode}" confirmé et enregistré`);
}


;

/**
 * Ouvre le coffre (trunk) via le proxy local
 * @returns {Promise<boolean>} true si la commande a été exécutée avec succès
 */
static async trunkOpen() {
    // Début de l'exécution de trunkOpen
    TeslaChargingLogger.writeToLog("TRUNK OPEN: Début de l'exécution de trunkOpen");

    return TeslaService.executeVehicleCommand({
        commandName: "OUVERTURE COFFRE",
        commandType: "trunkOpen",
        endpoint: "actuate_trunk",
        payload: { which_trunk: "rear" },
        icon: "🚘",
        lockVar: "lastTrunkOpenCall",
        onSuccess: (response) => {
            // Confirmation de l'ouverture réussie du coffre
            TeslaChargingLogger.writeToLog("✅ COFFRE: Coffre arrière ouvert avec succès");

            // Vérification et mise à jour du cache pour vehicle_state
            TeslaChargingLogger.writeToLog("TRUNK OPEN: Vérification de l'existence de vehicle_state dans last_receive_data");
            if (TeslaService.last_receive_data && !TeslaService.last_receive_data.vehicle_state) {
                TeslaChargingLogger.writeToLog("TRUNK OPEN: vehicle_state non défini, initialisation d'un objet vide");
                TeslaService.last_receive_data.vehicle_state = {};
            }

            if (TeslaService.last_receive_data?.vehicle_state) {
                TeslaChargingLogger.writeToLog("TRUNK OPEN: Mise à jour de vehicle_state pour indiquer l'ouverture du coffre");
                TeslaService.last_receive_data.vehicle_state.rt = 1; // rear trunk = 1 (ouvert)
                TeslaService.last_receive_data.vehicle_state.trunk_open = true;
                TeslaChargingLogger.writeToLog("TRUNK OPEN: Cache mis à jour avec vehicle_state.rt=1 et trunk_open=true");
            } else {
                TeslaChargingLogger.writeToLog("TRUNK OPEN: Alerte - vehicle_state reste indéfini après initialisation");
            }

            // Programmation de la vérification des données pour confirmer le changement
            TeslaChargingLogger.writeToLog("TRUNK OPEN: Programmation de la vérification des données pour confirmer le changement");
            TeslaService.step = TeslaService.STEP_GET_DATA;
            TeslaService.next_action_time = new Date().getTime() + 5000; // 5 secondes
            TeslaChargingLogger.writeToLog(`TRUNK OPEN: Prochaine action planifiée à ${TeslaService.next_action_time}`);
        }
    });
}


;

/**
 * Ouvre partiellement les fenêtres via le proxy local (mode ventilation)
 * @returns {Promise<boolean>} true si la commande a été exécutée avec succès
 */
static async ventOpen() {
    // Début de l'exécution de ventOpen
    TeslaChargingLogger.writeToLog("VENT OPEN: Début de l'exécution de ventOpen");

    return TeslaService.executeVehicleCommand({
        commandName: "VENTILATION",
        commandType: "ventOpen",
        endpoint: "window_control",
        payload: { 
            command: "vent",
            lat: 0,
            lon: 0
        },
        icon: "🪟",
        lockVar: "lastVentOpenCall",
        onSuccess: (response) => {
            // Confirmation de l'activation de la ventilation
            TeslaChargingLogger.writeToLog("✅ FENÊTRES: Ventilation activée avec succès");

            // Vérification et initialisation du cache pour vehicle_state
            TeslaChargingLogger.writeToLog("VENT OPEN: Vérification de l'existence de vehicle_state dans last_receive_data");
            if (TeslaService.last_receive_data && !TeslaService.last_receive_data.vehicle_state) {
                TeslaChargingLogger.writeToLog("VENT OPEN: vehicle_state non défini, initialisation d'un objet vide");
                TeslaService.last_receive_data.vehicle_state = {};
            }

            if (TeslaService.last_receive_data?.vehicle_state) {
                // Mise à jour du cache pour indiquer l'ouverture partielle des fenêtres
                TeslaChargingLogger.writeToLog("VENT OPEN: Mise à jour du cache pour l'ouverture partielle des fenêtres");
                TeslaService.last_receive_data.vehicle_state.fd_window = 1;
                TeslaService.last_receive_data.vehicle_state.fp_window = 1;
                TeslaService.last_receive_data.vehicle_state.rd_window = 1;
                TeslaService.last_receive_data.vehicle_state.rp_window = 1;
                TeslaService.last_receive_data.vehicle_state.windows_open = true;
                TeslaChargingLogger.writeToLog("VENT OPEN: Cache mis à jour (fd_window, fp_window, rd_window, rp_window = 1 et windows_open=true)");
            } else {
                TeslaChargingLogger.writeToLog("VENT OPEN: Alerte - vehicle_state demeure indéfini après initialisation");
            }

            // Programmation de la vérification des données pour confirmer le changement
            TeslaChargingLogger.writeToLog("VENT OPEN: Programmation de la vérification des données pour confirmer le changement");
            TeslaService.step = TeslaService.STEP_GET_DATA;
            TeslaService.next_action_time = new Date().getTime() + 5000; // 5 secondes
            TeslaChargingLogger.writeToLog(`VENT OPEN: Prochaine action planifiée à ${TeslaService.next_action_time}`);
        }
    });
}


;

/**
 * Désactive le mode sentinelle du véhicule via le proxy local
 * @returns {Promise<boolean>} true si la commande a été exécutée avec succès
 */
static async stopSentinel() {
    // Début de l'exécution de stopSentinel
    TeslaChargingLogger.writeToLog("SENTINEL STOP: Début de l'exécution de stopSentinel");

    return TeslaService.executeVehicleCommand({
        commandName: "SENTINELLE",
        commandType: "stopSentinel",
        endpoint: "set_sentry_mode",
        payload: { on: false },
        icon: "👁️",
        lockVar: "lastStopSentinelCall",
        onSuccess: (response) => {
            // Confirmation de la désactivation du mode sentinelle
            TeslaChargingLogger.writeToLog("✅ SENTINELLE: Mode sentinelle désactivé avec succès");

            // Vérification et mise à jour du cache pour vehicle_state
            TeslaChargingLogger.writeToLog("SENTINEL STOP: Vérification du cache pour vehicle_state");
            if (TeslaService.last_receive_data && !TeslaService.last_receive_data.vehicle_state) {
                TeslaChargingLogger.writeToLog("SENTINEL STOP: vehicle_state non défini, initialisation d'un objet vide");
                TeslaService.last_receive_data.vehicle_state = {};
            }
            
            if (TeslaService.last_receive_data?.vehicle_state) {
                TeslaChargingLogger.writeToLog("SENTINEL STOP: Mise à jour de vehicle_state pour indiquer la désactivation du mode sentinelle");
                TeslaService.last_receive_data.vehicle_state.sentry_mode = false;
                TeslaChargingLogger.writeToLog("SENTINEL STOP: vehicle_state.sentry_mode mis à false");
            } else {
                TeslaChargingLogger.writeToLog("SENTINEL STOP: Alerte - vehicle_state demeure indéfini après initialisation");
            }
            
            // Programmation de la vérification des données pour confirmer le changement
            TeslaChargingLogger.writeToLog("SENTINEL STOP: Programmation de la vérification des données pour confirmer la désactivation");
            TeslaService.step = TeslaService.STEP_GET_DATA;
            TeslaService.next_action_time = new Date().getTime() + 5000; // 5 secondes
            TeslaChargingLogger.writeToLog(`SENTINEL STOP: Prochaine action planifiée à ${TeslaService.next_action_time}`);
        }
    });
}


;

/**
 * Vérifie si le véhicule Tesla est à la maison en utilisant uniquement les coordonnées GPS
 * @returns {Promise<boolean>} true si le véhicule est à la maison
 */
static async isTeslaAtHome() {
    // Log de début d'exécution
    TeslaChargingLogger.writeToLog("🏠 IS TESLA AT HOME: Vérification si le véhicule est à la maison");
    
    try {
        // Récupérer les données MQTT de position (priorité donnée aux données MQTT)
        const mqttData = TeslaService.getMqttVehicleData();
        
        // Vérifier la disponibilité des coordonnées GPS 
        const latitude = mqttData?.drive_state?.latitude || TeslaService.last_receive_data?.drive_state?.latitude;
        const longitude = mqttData?.drive_state?.longitude || TeslaService.last_receive_data?.drive_state?.longitude;
        
        TeslaChargingLogger.writeToLog(`🗺️ IS TESLA AT HOME: Coordonnées GPS: lat=${latitude}, lon=${longitude}`);
        
        if (!latitude || !longitude) {
            TeslaChargingLogger.writeToLog("⚠️ IS TESLA AT HOME: Coordonnées GPS non disponibles");
            return false;
        }
        
        // Récupérer les coordonnées du domicile depuis la configuration
        const config = configuration_service_1.ConfigurationService.getConfiguration();
        const homeLat = config.tesla.homeLatitude;
        const homeLon = config.tesla.homeLongitude;
        
        // Vérifier si les coordonnées du domicile sont configurées
        if (!homeLat || !homeLon || (homeLat === 0 && homeLon === 0)) {
            TeslaChargingLogger.writeToLog("⚠️ IS TESLA AT HOME: Coordonnées du domicile non configurées");
            return false;
        }
        
        // Calcul de la distance en km en utilisant la formule haversine
        const R = 6371; // Rayon de la Terre en km
        const dLat = (homeLat - latitude) * Math.PI / 180;
        const dLon = (homeLon - longitude) * Math.PI / 180;
        
        const a = Math.sin(dLat/2) * Math.sin(dLat/2) +
                  Math.cos(latitude * Math.PI / 180) * Math.cos(homeLat * Math.PI / 180) * 
                  Math.sin(dLon/2) * Math.sin(dLon/2);
        
        const c = 2 * Math.atan2(Math.sqrt(a), Math.sqrt(1-a));
        const distance = R * c;
        
        // Distance maximale pour considérer le véhicule "à la maison" (500m)
        const isHome = distance < 0.5;
        
        TeslaChargingLogger.writeToLog(`🏠 IS TESLA AT HOME: Distance = ${distance.toFixed(3)}km, Véhicule ${isHome ? "à la maison" : "pas à la maison"}`);
        
        return isHome;
    } catch (error) {
        TeslaChargingLogger.writeToLog(`❌ IS TESLA AT HOME: Erreur - ${error.message}`);
        return false; // Par défaut, considérer le véhicule comme absent en cas d'erreur
    }
}

;
static addChargeValue(enphaseArray) {
    // Début de l'exécution de addChargeValue
    //TeslaChargingLogger.writeToLog("ADD CHARGE VALUE: Début de l'exécution de addChargeValue");
    let currentIndex = 0;
    //TeslaChargingLogger.writeToLog(`ADD CHARGE VALUE: Initialisation de currentIndex à ${currentIndex}`);

    // Initialisation de w_charge à 0 pour chaque élément de enphaseArray
    for (let i = 0; i < enphaseArray.length; i++) {
        enphaseArray[i].w_charge = 0;
        //TeslaChargingLogger.writeToLog(`ADD CHARGE VALUE: enphaseArray[${i}].w_charge initialisé à 0`);
    }

    // Parcours de enphaseArray à partir du second élément
    for (let i = 1; i < enphaseArray.length; i++) {
        let timestamp = enphaseArray[i - 1].timestamp;
        //TeslaChargingLogger.writeToLog(`ADD CHARGE VALUE: Traitement de enphaseArray[${i}], utilisant le timestamp de enphaseArray[${i - 1}]: ${timestamp}`);

        // Avancer dans chargeData jusqu'à trouver un timestamp supérieur ou égal à celui recherché
        while (currentIndex < TeslaService.chargeData.length && TeslaService.chargeData[currentIndex].timestamp < timestamp) {
            //TeslaChargingLogger.writeToLog(`ADD CHARGE VALUE: chargeData[${currentIndex}].timestamp (${TeslaService.chargeData[currentIndex].timestamp}) < ${timestamp}, incrémentation de currentIndex`);
            currentIndex++;
        }

        // Si currentIndex dépasse la longueur de chargeData, retourner enphaseArray
        if (currentIndex >= TeslaService.chargeData.length) {
            //TeslaChargingLogger.writeToLog(`ADD CHARGE VALUE: currentIndex (${currentIndex}) >= longueur de chargeData, retour de enphaseArray`);
            return enphaseArray;
        }

        // Comparaison des timestamps pour déterminer l'affectation de w_charge
        if (TeslaService.chargeData[currentIndex].timestamp < enphaseArray[i].timestamp) {
            //TeslaChargingLogger.writeToLog(`ADD CHARGE VALUE: chargeData[${currentIndex}].timestamp (${TeslaService.chargeData[currentIndex].timestamp}) < enphaseArray[${i}].timestamp (${enphaseArray[i].timestamp}), affectation de w_charge`);
            enphaseArray[i].w_charge = TeslaService.chargeData[currentIndex].w_charge;
        } else {
            //TeslaChargingLogger.writeToLog(`ADD CHARGE VALUE: chargeData[${currentIndex}].timestamp (${TeslaService.chargeData[currentIndex].timestamp}) >= enphaseArray[${i}].timestamp (${enphaseArray[i].timestamp}), aucune affectation de w_charge`);
        }
    }

    // Fin de l'exécution de addChargeValue, retour de enphaseArray modifié
    //TeslaChargingLogger.writeToLog("ADD CHARGE VALUE: Fin de l'exécution de addChargeValue, retour de enphaseArray");
    return enphaseArray;
}

static setConfiguration(configuration, attribModified, from) {
    // Début de l'exécution de setConfiguration
    TeslaChargingLogger.writeToLog(`CONFIGURATION: Début de setConfiguration, attribModified = ${attribModified}, source = ${from}`);

    // Vérifier si l'attribut modifié est critique nécessitant une reconnexion
    if (attribModified.startsWith('"tesla.token') ||
        attribModified.startsWith('"tesla.useTesla') ||
        attribModified.startsWith('"tesla.chargingMode')) {
        
        TeslaChargingLogger.writeToLog(`CONFIGURATION: Changement critique détecté (${attribModified}), reconnexion en cours`);
        TeslaService.connect();
    } 
    // Pour les autres changements liés à Tesla, journaliser sans reconnexion
    else if (attribModified.startsWith('"tesla.')) {
        TeslaChargingLogger.writeToLog(`CONFIGURATION: Paramètre ${attribModified} modifié, aucune reconnexion nécessaire`);
    }

    // Fin de l'exécution de setConfiguration
    TeslaChargingLogger.writeToLog("CONFIGURATION: Fin de l'exécution de setConfiguration");
}

/**
 * Effectue un log périodique des données clés du véhicule (API vs MQTT)
 * Permet de diagnostiquer les différences entre sources de données
 */
static logPeriodicStatus() {
    TeslaChargingLogger.writeToLog("📊 PERIODIC: Début du log périodique des données véhicule");
    
    try {
        // Récupérer les données API
        const apiData = TeslaService.last_receive_data || {};
        const apiAge = apiData._timestamp ? Math.round((Date.now() - apiData._timestamp)/1000) : "N/A";
        
        // Récupérer les données MQTT
        const mqttData = TeslaService.getMqttVehicleData();
        const mqttAge = mqttData._timestamp ? Math.round((Date.now() - mqttData._timestamp)/1000) : "N/A";
        
        // Formater le résumé
        let summary = [
            `📋 SYNTHÈSE DONNÉES VÉHICULE (${new Date().toISOString()})`,
            `🔋 Batterie: API=${apiData.charge_state?.battery_level || "N/A"}%, MQTT=${mqttData.charge_state?.battery_level || "N/A"}%`,
            `⚡ État charge: API=${apiData.charge_state?.charging_state || "N/A"}, MQTT=${mqttData.charge_state?.charging_state || "N/A"}`,
            `🌡️ Temp int/ext: API=${apiData.climate_state?.inside_temp || "N/A"}°C/${apiData.climate_state?.outside_temp || "N/A"}°C, MQTT=${mqttData.climate_state?.inside_temp || "N/A"}°C/${mqttData.climate_state?.outside_temp || "N/A"}°C`,
            `🚗 État véhicule: API=${apiData.state || "N/A"}, MQTT=${mqttData.state || "N/A"}`,
            `⌚ Âge données: API=${apiAge}s, MQTT=${mqttAge}s`,
            `📍 Source utilisée: ${TeslaService.getVehicleStatus().data_source}`
        ];
        
        // Log concis si les données correspondent entre API et MQTT
        if (apiData.charge_state?.battery_level === mqttData.charge_state?.battery_level &&
            apiData.charge_state?.charging_state === mqttData.charge_state?.charging_state) {
            TeslaChargingLogger.writeToLog(summary.join("\n"));
        } else {
            // Log plus détaillé en cas de différence
            summary.push(`⚠️ DIFFÉRENCES DÉTECTÉES ENTRE API ET MQTT`);
            summary.push(`🔄 Mode actuel: ${TeslaService.getModeName(TeslaService.mode)}, Étape: ${TeslaService.getStepName(TeslaService.step)}`);
            TeslaChargingLogger.writeToLog(summary.join("\n"));
        }
    } catch (error) {
        TeslaChargingLogger.writeToLog(`❌ PERIODIC LOG ERROR: ${error.message}`);
    }
}

/**
 * Synchronise intelligemment les données MQTT et API pour garantir la fraîcheur
 * MQTT prime sur API pour tous les champs qu'il couvre, sans perdre les autres champs API
 */
static syncDataSources() {
    TeslaChargingLogger.writeToLog("🔄 DATA SYNC: Début synchronisation des sources de données");
    
    try {
        // CORRECTION: Vérifier et initialiser this.last_receive_data si undefined
        if (!this.last_receive_data) {
            this.last_receive_data = {};
            TeslaChargingLogger.writeToLog("⚠️ DATA SYNC: last_receive_data était undefined, initialisation effectuée");
        }
        
        // Récupérer les données MQTT avec protection nullish
        const mqttData = TeslaService.getMqttVehicleData() || {};
        
        // Vérifier si les données MQTT sont présentes et fraîches (moins de 10 minutes)
        const mqttFresh = mqttData && 
                        mqttData._timestamp && 
                        (Date.now() - mqttData._timestamp) < 10 * 60 * 1000;
        
        if (mqttFresh) {
            TeslaChargingLogger.writeToLog("📡 DATA SYNC: Données MQTT fraîches disponibles, application prioritaire");
            
            // 1. STRATÉGIE: Conserver la structure API mais écraser toutes les valeurs MQTT disponibles
            
            // Traitement des données de charge
            if (mqttData.charge_state) {
                // CORRECTION: S'assurer que charge_state existe
                if (!this.last_receive_data.charge_state) {
                    this.last_receive_data.charge_state = {};
                    TeslaChargingLogger.writeToLog("🔄 DATA SYNC: Création de l'objet charge_state");
                }
                
                // Écraser toutes les valeurs de charge disponibles
                Object.keys(mqttData.charge_state).forEach(key => {
                    this.last_receive_data.charge_state[key] = mqttData.charge_state[key];
                });
                
                TeslaChargingLogger.writeToLog(`🔋 DATA SYNC: ${Object.keys(mqttData.charge_state).length} attributs de charge écrasés par MQTT`);
                
                // NOUVEAU: Synchronisation spécifique entre charger_actual_current et charge_amps
                if (mqttData.charge_state.charger_actual_current && 
                    this.last_receive_data.charge_state.charge_amps !== mqttData.charge_state.charger_actual_current) {
                    
                    const previousAmps = this.last_receive_data.charge_state.charge_amps;
                    this.last_receive_data.charge_state.charge_amps = mqttData.charge_state.charger_actual_current;
                    this.last_receive_data.charge_state.charge_current_request = mqttData.charge_state.charger_actual_current;
                    
                    TeslaChargingLogger.writeToLog(`🔄 CHARGE SYNC: Synchronisation charge_amps avec charger_actual_current (${previousAmps}A → ${mqttData.charge_state.charger_actual_current}A)`);
                }

                // Calcul de charge_rate (en miles/heure) à partir de la puissance
                if (mqttData.charge_state?.charger_power && !mqttData.charge_state?.charge_rate) {
                    // Constante d'efficacité de la Tesla (Wh/mile)
                    const EFFICIENCY_WH_PER_MILE = 255;
                    
                    // Puissance en kW convertie en W
                    const powerInWatts = mqttData.charge_state.charger_power * 1000;
                    
                    // Calcul du taux de charge en miles/heure (W ÷ (Wh/mile) = miles/h)
                    const chargeRateMilesPerHour = powerInWatts / EFFICIENCY_WH_PER_MILE;
                    
                    // Mise à jour de charge_rate dans la structure de données (uniquement en miles/h)
                    this.last_receive_data.charge_state.charge_rate = chargeRateMilesPerHour;
                    
                    TeslaChargingLogger.writeToLog(`🔄 CHARGE RATE SYNC: Calcul depuis puissance ${mqttData.charge_state.charger_power}kW → ${chargeRateMilesPerHour.toFixed(1)}mi/h`);
                }
                
                // NOUVEAU: Synchronisation bidirectionnelle des valeurs d'autonomie
                const milesToKm = 1.60934;
                const kmToMiles = 1 / milesToKm;
                
                // 1. Si MQTT a est_battery_range_km, calculer est_battery_range en miles (pas battery_range)
                if (mqttData.charge_state.est_battery_range_km) {
                    this.last_receive_data.charge_state.est_battery_range = 
                        Math.round((mqttData.charge_state.est_battery_range_km * kmToMiles) * 100) / 100;
                    
                    TeslaChargingLogger.writeToLog(`🔄 RANGE SYNC: KM→MI ${mqttData.charge_state.est_battery_range_km}km → ${this.last_receive_data.charge_state.est_battery_range}mi`);
                }

                // Si MQTT a également battery_range
                if (mqttData.charge_state.battery_range) {
                    this.last_receive_data.charge_state.battery_range = mqttData.charge_state.battery_range;
                    TeslaChargingLogger.writeToLog(`🔄 RANGE SYNC: Copie directe battery_range: ${mqttData.charge_state.battery_range}mi`);
                }
                
                // 2. Si MQTT a ideal_battery_range_km, calculer ideal_battery_range en miles
                if (mqttData.charge_state.ideal_battery_range_km) {
                    this.last_receive_data.charge_state.ideal_battery_range = 
                        Math.round((mqttData.charge_state.ideal_battery_range_km * kmToMiles) * 100) / 100;
                        
                    TeslaChargingLogger.writeToLog(`🔄 RANGE SYNC: KM→MI idéal ${mqttData.charge_state.ideal_battery_range_km}km → ${this.last_receive_data.charge_state.ideal_battery_range}mi`);
                }
                
                // 3. Si API a battery_range mais pas est_battery_range_km, calculer
                if (this.last_receive_data.charge_state.battery_range && !this.last_receive_data.charge_state.est_battery_range_km) {
                    this.last_receive_data.charge_state.est_battery_range_km = 
                        Math.round((this.last_receive_data.charge_state.battery_range * milesToKm) * 100) / 100;
                        
                    TeslaChargingLogger.writeToLog(`🔄 RANGE SYNC: MI→KM ${this.last_receive_data.charge_state.battery_range}mi → ${this.last_receive_data.charge_state.est_battery_range_km}km`);
                }
                
                // 4. Si API a ideal_battery_range mais pas ideal_battery_range_km, calculer
                if (this.last_receive_data.charge_state.ideal_battery_range && !this.last_receive_data.charge_state.ideal_battery_range_km) {
                    this.last_receive_data.charge_state.ideal_battery_range_km = 
                        Math.round((this.last_receive_data.charge_state.ideal_battery_range * milesToKm) * 100) / 100;
                        
                    TeslaChargingLogger.writeToLog(`🔄 RANGE SYNC: MI→KM idéal ${this.last_receive_data.charge_state.ideal_battery_range}mi → ${this.last_receive_data.charge_state.ideal_battery_range_km}km`);
                }
                
                // 5. Synchroniser également time_to_full_charge si disponible en MQTT
                if (mqttData.charge_state.time_to_full_charge !== undefined &&
                    this.last_receive_data.charge_state.time_to_full_charge !== mqttData.charge_state.time_to_full_charge) {
                    
                    const previousTime = this.last_receive_data.charge_state.time_to_full_charge;
                    this.last_receive_data.charge_state.time_to_full_charge = mqttData.charge_state.time_to_full_charge;
                    
                    TeslaChargingLogger.writeToLog(`🔄 TIME SYNC: Temps restant de charge mis à jour (${previousTime || 'N/A'}h → ${mqttData.charge_state.time_to_full_charge}h)`);
                }
                
                // NOUVEAU: Conversion de time_to_full_charge (heures) en minutes_to_full_charge (minutes)
                if (this.last_receive_data.charge_state.time_to_full_charge !== undefined) {
                    // Convertir les heures en minutes pour minutes_to_full_charge
                    const hoursToMinutes = this.last_receive_data.charge_state.time_to_full_charge * 60;
                    this.last_receive_data.charge_state.minutes_to_full_charge = Math.round(hoursToMinutes);
                    
                    TeslaChargingLogger.writeToLog(`🔄 TIME SYNC: Conversion hours→minutes: ${this.last_receive_data.charge_state.time_to_full_charge}h → ${this.last_receive_data.charge_state.minutes_to_full_charge}min`);
                }
            }
            
            // Traitement des données de climat
            if (mqttData.climate_state) {
                // CORRECTION: S'assurer que climate_state existe
                if (!this.last_receive_data.climate_state) {
                    this.last_receive_data.climate_state = {};
                    TeslaChargingLogger.writeToLog("🔄 DATA SYNC: Création de l'objet climate_state");
                }
                
                // Écraser toutes les valeurs de climat disponibles
                Object.keys(mqttData.climate_state).forEach(key => {
                    this.last_receive_data.climate_state[key] = mqttData.climate_state[key];
                });
                
                TeslaChargingLogger.writeToLog(`🌡️ DATA SYNC: ${Object.keys(mqttData.climate_state).length} attributs de climat écrasés par MQTT`);
            }
            
            // Traitement des données de conduite
            if (mqttData.drive_state) {
                // CORRECTION: S'assurer que drive_state existe
                if (!this.last_receive_data.drive_state) {
                    this.last_receive_data.drive_state = {};
                    TeslaChargingLogger.writeToLog("🔄 DATA SYNC: Création de l'objet drive_state");
                }
                
                // Écraser toutes les valeurs de conduite disponibles
                Object.keys(mqttData.drive_state).forEach(key => {
                    this.last_receive_data.drive_state[key] = mqttData.drive_state[key];
                });
                
                TeslaChargingLogger.writeToLog(`🚗 DATA SYNC: ${Object.keys(mqttData.drive_state).length} attributs de conduite écrasés par MQTT`);
            }
            
            // Traitement des données d'état du véhicule
            if (mqttData.vehicle_state) {
                // CORRECTION: S'assurer que vehicle_state existe
                if (!this.last_receive_data.vehicle_state) {
                    this.last_receive_data.vehicle_state = {};
                    TeslaChargingLogger.writeToLog("🔄 DATA SYNC: Création de l'objet vehicle_state");
                }
                
                // Écraser toutes les valeurs d'état véhicule disponibles
                Object.keys(mqttData.vehicle_state).forEach(key => {
                    this.last_receive_data.vehicle_state[key] = mqttData.vehicle_state[key];
                });
                
                TeslaChargingLogger.writeToLog(`🚗 DATA SYNC: ${Object.keys(mqttData.vehicle_state).length} attributs d'état véhicule écrasés par MQTT`);
            }
            
            // Copier les autres attributs de premier niveau
            if (mqttData.state) {
                this.last_receive_data.state = mqttData.state;
                TeslaChargingLogger.writeToLog(`🚗 DATA SYNC: État global synchronisé: ${mqttData.state}`);
            }
            
            // CORRECTION: Mise à jour des métadonnées pour traçabilité
            this.last_receive_data._mqtt_timestamp = mqttData._timestamp;
            this.last_receive_data._source = "mixed (mqtt priority)";
            this.last_receive_data._timestamp = Date.now();
            
            TeslaChargingLogger.writeToLog("✅ DATA SYNC: Données synchronisées avec priorité MQTT");
        } else {
            // Pas de données MQTT fraîches, conserver les données API
            TeslaChargingLogger.writeToLog("ℹ️ DATA SYNC: Aucune donnée MQTT fraîche, conservation API uniquement");
        }
        
        return this.last_receive_data;
    } catch (error) {
        // CORRECTION: Ajout d'un bloc try-catch pour gérer les erreurs potentielles
        TeslaChargingLogger.writeToLog(`❌ DATA SYNC ERROR: ${error.message}`);
        console.error("Erreur lors de la synchronisation des données:", error);
        
        // Assurer qu'on a toujours un objet à retourner
        return this.last_receive_data || {};
    }
}
   
/**
 * Vérifie les conditions pour le mode Solar Plus et lance la charge si les conditions sont remplies
 * Cette méthode utilise les coordonnées GPS pour vérifier si le véhicule est à la maison
 * @returns {Promise<void>}
 */
static async checkModeSolarPlus() {
    // PROTECTION CRITIQUE: Empêcher tout appel trop rapproché
    const now = new Date().getTime();
    
    // Initialiser lastCheckModeSolarPlusTime si non défini
    if (!TeslaService.lastCheckModeSolarPlusTime) {
        TeslaService.lastCheckModeSolarPlusTime = 0;
    }
    
    // Bloquer complètement les appels trop rapprochés (15 secondes minimum)
    if (now - TeslaService.lastCheckModeSolarPlusTime < 15000) {
        // Ne pas même logger pour éviter de polluer les logs
        return;
    }
    
    // NOUVEAU: Vérifier si une opération de charge est en cours
    if (TeslaService.chargingOperationInProgress) {
        TeslaChargingLogger.writeToLog("CHECK MODE SOLAR PLUS: Une opération de charge est en cours, vérification reportée");
        return;
    }
    
    // Enregistrer l'heure de cette vérification AVANT de commencer le traitement
    TeslaService.lastCheckModeSolarPlusTime = now;
    
    // Début de l'exécution de checkModeSolarPlus
    TeslaChargingLogger.writeToLog("CHECK MODE SOLAR PLUS: Début de l'exécution de checkModeSolarPlus");
    
    // Vérifier si le véhicule est à la maison
    const isAtHome = await TeslaService.isTeslaAtHome();
    TeslaChargingLogger.writeToLog(`🏠 CHECK MODE SOLAR PLUS: Véhicule à la maison: ${isAtHome ? 'Oui' : 'Non'}`);
    
    // Si véhicule pas à la maison, ne pas démarrer de charge solaire
    if (!isAtHome) {
        TeslaChargingLogger.writeToLog("CHECK MODE SOLAR PLUS: Véhicule non détecté à la maison, charge solaire ignorée");
        return;
    }
    
    // Vérifier si le mode de charge est 'solarStart' ou 'solarPlus'
    if (TeslaService.chargingMode !== 'solarStart' && TeslaService.chargingMode !== 'solarPlus') {
        TeslaChargingLogger.writeToLog(`CHECK MODE SOLAR PLUS: Mode de charge actuel '${TeslaService.chargingMode}' n'est pas compatible, aucune action requise`);
        return;
    }

    // Vérifier immédiatement l'état de charge
    const chargingState = TeslaService.last_receive_data?.charge_state?.charging_state;
    const isPluggedIn = TeslaService.last_receive_data?.charge_state?.plugged_in === true;
    
    TeslaChargingLogger.writeToLog(`CHECK MODE SOLAR PLUS: État de charge actuel="${chargingState || 'inconnu'}", Branché: ${isPluggedIn ? 'Oui' : 'Non'}`);
    
    // NOUVELLE VÉRIFICATION: Si le véhicule n'est pas branché, ne pas tenter de charge
    if (!isPluggedIn) {
        TeslaChargingLogger.writeToLog("CHECK MODE SOLAR PLUS: Véhicule non branché, impossible de démarrer la charge");
        return;
    }
    
    // Si le véhicule est déjà en charge, utiliser checkChargingMode pour l'ajustement
    if (chargingState === "Charging" || chargingState === "Starting") {
        TeslaChargingLogger.writeToLog("CHECK MODE SOLAR PLUS: Véhicule déjà en charge, redirection vers l'ajustement d'ampérage");
        return await TeslaService.checkChargingMode(true);
    }

    // Récupérer la configuration et utiliser adaptChargeFrequency comme délai de cooldown
    const config = configuration_service_1.ConfigurationService.getConfiguration();
    const cooldownSeconds = Math.max(config.tesla.adaptChargeFrequency || 3600, 900); // Minimum 15 min (900s)

    // Vérifier si la dernière tentative de démarrage RÉUSSIE est trop récente
    if (TeslaService.lastSolarPlusStartChargeTime && 
        now - TeslaService.lastSolarPlusStartChargeTime < cooldownSeconds * 1000) {
        const secondsSinceLastChange = Math.round((now - TeslaService.lastSolarPlusStartChargeTime)/1000);
        TeslaChargingLogger.writeToLog(`CHECK MODE SOLAR PLUS: Dernier démarrage de charge réussi trop récent (il y a ${secondsSinceLastChange}s, cooldown: ${cooldownSeconds}s), aucune action requise`);
        return;
    }
    
    // NOUVEAU: Vérifier si la dernière tentative ÉCHOUÉE est trop récente (15 minutes)
    if (TeslaService.lastSolarPlusFailedStartTime && 
        now - TeslaService.lastSolarPlusFailedStartTime < 15 * 60 * 1000) {
        const secondsSinceLastFailure = Math.round((now - TeslaService.lastSolarPlusFailedStartTime)/1000);
        TeslaChargingLogger.writeToLog(`CHECK MODE SOLAR PLUS: Dernier échec de démarrage trop récent (il y a ${secondsSinceLastFailure}s, cooldown: 15min), aucune action requise`);
        return;
    }

    // Vérifier si la surproduction est suffisante pour démarrer la charge
    TeslaChargingLogger.writeToLog("CHECK MODE SOLAR PLUS: Vérification de la surproduction pour démarrer la charge");
    let overProduction = enphase_data_service_1.EnphaseDataService.getOverproduction(config.tesla.adaptChargeFrequency);
    TeslaChargingLogger.writeToLog(`CHECK MODE SOLAR PLUS: Surproduction obtenue: ${overProduction}W`);

    const requiredOverProduction = config.tesla.minAmps * 220 - config.tesla.chargeOffset;
    TeslaChargingLogger.writeToLog(`CHECK MODE SOLAR PLUS: Surproduction requise: ${requiredOverProduction}W`);

    if (overProduction && overProduction >= requiredOverProduction) {
        TeslaChargingLogger.writeToLog(`CHECK MODE SOLAR PLUS: Surproduction suffisante (${overProduction}W >= ${requiredOverProduction}W)`);

        // Vérifier si la limite de charge est supérieure au niveau de batterie utilisable
        const chargeLimit = TeslaService.last_receive_data?.charge_state?.charge_limit_soc;
        const usableBatteryLevel = TeslaService.last_receive_data?.charge_state?.usable_battery_level;
        
        TeslaChargingLogger.writeToLog(`CHECK MODE SOLAR PLUS: Limite de charge: ${chargeLimit}%, Niveau actuel: ${usableBatteryLevel}%`);

        if (chargeLimit > usableBatteryLevel) {
            TeslaChargingLogger.writeToLog("CHECK MODE SOLAR PLUS: Conditions remplies pour démarrer la charge en mode solarStart/solarPlus");
            
            // NOUVEAU: Vérification finale avant démarrage
            if (TeslaService.chargingOperationInProgress) {
                TeslaChargingLogger.writeToLog("CHECK MODE SOLAR PLUS: Une opération de charge a démarré entre temps, annulation");
                return;
            }
            
            // Lancer le démarrage de charge
            TeslaChargingLogger.writeToLog("CHECK MODE SOLAR PLUS: Démarrage de la charge");
            const ret = await TeslaService.startCharge();
            
            if (ret) {
                // Mettre à jour le timestamp seulement si la charge a réellement démarré
                TeslaService.lastSolarPlusStartChargeTime = now;
                TeslaChargingLogger.writeToLog("CHECK MODE SOLAR PLUS: Démarrage de charge réussi et timestamp mis à jour");
                
                // Réinitialiser le timestamp d'échec
                TeslaService.lastSolarPlusFailedStartTime = 0;
                
                // Mettre à jour l'état local immédiatement pour éviter les demandes multiples
                if (!TeslaService.last_receive_data) TeslaService.last_receive_data = {};
                if (!TeslaService.last_receive_data.charge_state) TeslaService.last_receive_data.charge_state = {};
                TeslaService.last_receive_data.charge_state.charging_state = "Starting";
                
                // Planifier un ajustement rapide de l'ampérage après le démarrage
                setTimeout(async () => {
                    TeslaChargingLogger.writeToLog("CHECK MODE SOLAR PLUS: Ajustement de l'ampérage après démarrage");
                    // Passer true pour forcer l'ajustement même si une vérification récente a été faite
                    await TeslaService.checkChargingMode(true);
                }, 15000); // 15 secondes après le démarrage
            } else {
                // NOUVEAU: Enregistrer l'échec pour le cooldown
                TeslaService.lastSolarPlusFailedStartTime = now;
                TeslaChargingLogger.writeToLog("CHECK MODE SOLAR PLUS: Échec du démarrage de la charge, cooldown de 15 minutes activé");
            }
            
            TeslaChargingLogger.writeToLog("CHECK MODE SOLAR PLUS: Programmation de la récupération des données dans 30 secondes");
            TeslaService.next_action_time = now + 30000;
        } else {
            TeslaChargingLogger.writeToLog(`CHECK MODE SOLAR PLUS: Charge non démarrée car la limite de charge (${chargeLimit}%) est inférieure ou égale au niveau de batterie utilisable (${usableBatteryLevel}%)`);
        }
    } else {
        TeslaChargingLogger.writeToLog(`CHECK MODE SOLAR PLUS: Surproduction insuffisante pour démarrer la charge (${overProduction}W < ${requiredOverProduction}W)`);
    }
}

}
exports.TeslaService = TeslaService;
/** Le token pour acceder à l'API */
TeslaService.access_token = '';
/** L'id du vehicule. */
TeslaService.vehicle_id = '';
/** Les dernière données recues. */
TeslaService.last_receive_data = {};
TeslaService.nbWakeUp = 0;
TeslaService.wasInCharge = false;
TeslaService.MODE_INIT = 0;
TeslaService.MODE_GET_DATA = 10;
TeslaService.MODE_GET_DATA_NOW = 11;
TeslaService.MODE_WAIT_ONLINE = 20;
TeslaService.MODE_GET_CONTINUS_DATA = 30;
TeslaService.MODE_REFRESH_TOKEN = 40;
TeslaService.mode = TeslaService.MODE_INIT;
TeslaService.previousMode = TeslaService.mode;
TeslaService.STEP_CONNECT = 0;
TeslaService.STEP_GET_VEHICLES = 10;
TeslaService.STEP_GET_ONLINE = 20;
TeslaService.STEP_WAKE_UP = 30;
TeslaService.STEP_GET_DATA = 40;
TeslaService.step = TeslaService.STEP_CONNECT;
TeslaService.previousStep = TeslaService.step;
TeslaService.next_action_time = 0;
TeslaService.last_success_get_data = 0;
TeslaService.max_time_between_get_data = 6 * 60 * 60 * 1000; // 6 heures
TeslaService.chargingMode = "normal";
TeslaService.lastChangeChargingAmps = 0;
TeslaService.lastSolarPlusStartChargeTime = 0;
TeslaService.chargeData = [];
TeslaService.stepInProgress = false;

// Intervalles recommandés par TeslaMate (en millisecondes)
TeslaService.POLLING_ASLEEP_INTERVAL = 180 * 1000;     // 3 minutes (comme POLLING_ASLEEP_INTERVAL=180)
TeslaService.POLLING_CHARGING_INTERVAL = 180 * 1000;   // 3 minutes (comme POLLING_CHARGING_INTERVAL=180)
TeslaService.POLLING_DRIVING_INTERVAL = 120 * 1000;    // 2 minutes (comme POLLING_DRIVING_INTERVAL=120)
TeslaService.POLLING_ONLINE_INTERVAL = 180 * 1000;     // 3 minutes (comme POLLING_ONLINE_INTERVAL=180)
TeslaService.POLLING_DEFAULT_INTERVAL = 180 * 1000;    // 3 minutes (comme POLLING_DEFAULT_INTERVAL=180)
TeslaService.POLLING_MINIMUM_INTERVAL = 120 * 1000;    // 2 minutes (comme POLLING_MINIMUM_INTERVAL=120)

// Variables pour la gestion des sessions
TeslaService.sessionTimedOut = false;
TeslaService.lastSessionTimeoutNotification = 0;

;

/**
 * Planificateur principal avec gestion asynchrone complète
 * @returns {Promise<void>}
 */
TeslaService.checkGetNewData = async function() {
    // Protection contre les exécutions simultanées
    if (TeslaService.stepInProgress) {
        TeslaChargingLogger.writeToLog("CHECK GET NEW DATA: Exécution déjà en cours, abandon de l'appel");
        return;
    }
    
    const now = Date.now();
    /* JLO TeslaChargingLogger.writeToLog(`CHECK GET NEW DATA: Timestamp actuel = ${now}`);*/
    
    // PROTECTION AMÉLIORÉE:
    // 1. Mode normal = pas d'exécution automatique
    // 2. Mode solaire = respect de la fréquence configurée
    if (TeslaService.mode !== TeslaService.MODE_GET_DATA_NOW) {
        if (TeslaService.chargingMode === 'normal') {
            /* JLO TeslaChargingLogger.writeToLog("CHECK GET NEW DATA: Mode normal détecté, aucune exécution automatique");*/
            return;
        } else if (TeslaService.chargingMode === 'solar' || 
                   TeslaService.chargingMode === 'solarPlus' || 
                   TeslaService.chargingMode === 'adapt' ||
                   TeslaService.chargingMode === 'solarStart') {
            const config = configuration_service_1.ConfigurationService.getConfiguration().tesla;
            
            // Initialiser lastSolarModeCheck si nécessaire
            if (!TeslaService.lastSolarModeCheck) {
                TeslaService.lastSolarModeCheck = 0;
                TeslaChargingLogger.writeToLog("CHECK GET NEW DATA: Initialisation de lastSolarModeCheck à 0");
            }
            
            // Appliquer la fréquence configurée (minimum 10 secondes)
            const solarCheckInterval = Math.max(config.adaptChargeFrequency * 1000, 10000);
            /* JLO TeslaChargingLogger.writeToLog(`CHECK GET NEW DATA: Intervalle de vérification solaire = ${solarCheckInterval} ms`);*/
            
            // Ne pas continuer si le délai configuré n'est pas écoulé
            if (now - TeslaService.lastSolarModeCheck < solarCheckInterval) {
                /* JLO TeslaChargingLogger.writeToLog(`CHECK GET NEW DATA: Délai de vérification solaire non écoulé (${now - TeslaService.lastSolarModeCheck} ms écoulés)`);*/
                return;
            }
            
            // Mise à jour du timestamp de dernière vérification solaire
            TeslaService.lastSolarModeCheck = now;
            TeslaChargingLogger.writeToLog(`CHECK GET NEW DATA: lastSolarModeCheck mis à jour à ${now}`);
            TeslaChargingLogger.writeToLog(`⏱️ MODE SOLAIRE: Vérification selon fréquence configurée (${config.adaptChargeFrequency}s)`);
            
            // CORRECTION: Vérification des modes de charge et appel des fonctions appropriées
            if (TeslaService.last_receive_data?.charge_state?.charging_state === 'Charging') {
                TeslaChargingLogger.writeToLog("🔌 CHECK GET NEW DATA: Véhicule en charge détecté en mode solaire");
                
                // Distinguer les modes pour appeler la bonne fonction
                if (TeslaService.chargingMode === 'solar' || TeslaService.chargingMode === 'adapt') {
                    TeslaChargingLogger.writeToLog("🔋 CHECK GET NEW DATA: Mode solar/adapt, appel à checkChargingMode()");
                    TeslaService.checkChargingMode();
                }
                else if (TeslaService.chargingMode === 'solarPlus' || TeslaService.chargingMode === 'solarStart') {
                    TeslaChargingLogger.writeToLog("🔋 CHECK GET NEW DATA: Mode solarPlus/solarStart, appel à checkModeSolarPlus()");
                    TeslaService.checkModeSolarPlus();
                }
            } else {
                TeslaChargingLogger.writeToLog("ℹ️ CHECK GET NEW DATA: Véhicule non en charge, pas d'appel à checkChargingMode()");
            }
        }
    }
    
    // PROTECTION: Respecter le délai entre tentatives
    if (TeslaService.next_action_time && now < TeslaService.next_action_time) {
        TeslaChargingLogger.writeToLog(`CHECK GET NEW DATA: Délai entre tentatives non écoulé (prochain appel prévu à ${TeslaService.next_action_time})`);
        return;
    }
    
    // GESTION DU MODE RAFRAÎCHISSEMENT MANUEL
    if (TeslaService.mode === TeslaService.MODE_GET_DATA_NOW) {
        TeslaChargingLogger.writeToLog("🔄 RAFRAÎCHISSEMENT MANUEL: Exécution immédiate demandée par l'utilisateur");
        if (!TeslaService.lastManualRefreshTime) {
            TeslaService.lastManualRefreshTime = now;
            TeslaChargingLogger.writeToLog(`RAFRAÎCHISSEMENT MANUEL: lastManualRefreshTime initialisé à ${now}`);
        } else if (now - TeslaService.lastManualRefreshTime > 30000) {
            TeslaChargingLogger.writeToLog("⏱️ TIMEOUT: Retour automatique au mode normal après 30s");
            TeslaService.mode = TeslaService.MODE_GET_DATA;
            TeslaService.lastManualRefreshTime = null;
        }
    }
    
    // SÉCURITÉ: Libération des verrous bloqués après un certain temps
    if (TeslaService.wakeupInProgress && TeslaService.lastWakeupCall && 
        now - TeslaService.lastWakeupCall > 60000) {
        TeslaChargingLogger.writeToLog("🔓 SÉCURITÉ: Libération automatique du verrou wake_up bloqué");
        TeslaService.wakeupInProgress = false;
    }
    
    // Activer le verrou pour empêcher les exécutions parallèles
    TeslaService.stepInProgress = true;
    TeslaChargingLogger.writeToLog("CHECK GET NEW DATA: Verrou activé (stepInProgress = true)");
    
    try {
        // Log d'exécution initiale
        TeslaChargingLogger.writeToLog(`🔍 EXÉCUTION: Étape = ${TeslaService.getStepName(TeslaService.step)}, Mode = ${TeslaService.getModeName(TeslaService.mode)}`);
        
        // Vérification via MQTT pour optimiser les transitions d'état
        const mqttData = TeslaService.getMqttVehicleData();
        TeslaChargingLogger.writeToLog(`CHECK GET NEW DATA: Données MQTT récupérées = ${JSON.stringify(mqttData)}`);
        
        // Mise à jour de l'état du véhicule si MQTT indique "online"
        if (mqttData?.vehicle_state?.state === "online" || mqttData?.state === "online") {
            if (TeslaService.lastKnownVehicleState !== "online") {
                TeslaChargingLogger.writeToLog("📡 MQTT: Véhicule détecté en ligne, mise à jour de l'état");
                TeslaService.lastKnownVehicleState = "online";
                TeslaService.lastVehicleStateCheck = now;
            }
        }
        
        switch (TeslaService.step) {
            case TeslaService.STEP_CONNECT:
                TeslaChargingLogger.writeToLog("CHECK GET NEW DATA: Étape CONNECT détectée, appel à connect()");
                await TeslaService.connect();
                break;
                
            case TeslaService.STEP_GET_VEHICLES:
                TeslaChargingLogger.writeToLog("CHECK GET NEW DATA: Étape GET_VEHICLES détectée, appel à getVehicleList()");
                await TeslaService.getVehicleList();
                break;
                
            case TeslaService.STEP_GET_ONLINE:
                TeslaChargingLogger.writeToLog("CHECK GET NEW DATA: Étape GET_ONLINE détectée");
                // OPTIMISATION: Passage direct aux données si MQTT indique "online" en mode manuel
                if (mqttData?.vehicle_state?.state === "online" && TeslaService.mode === TeslaService.MODE_GET_DATA_NOW) {
                    TeslaChargingLogger.writeToLog("📡 OPTIMISATION: MQTT indique véhicule en ligne, passage direct à GET_DATA");
                    TeslaService.step = TeslaService.STEP_GET_DATA;
                    TeslaService.next_action_time = now;
                    TeslaService.stepInProgress = false;
                    setImmediate(TeslaService.checkGetNewData);
                    return;
                }
                
                // Vérifier l'état du véhicule via API
                TeslaChargingLogger.writeToLog("CHECK GET NEW DATA: Appel à checkVehicleOnlineStatus()");
                const state = await TeslaService.checkVehicleOnlineStatus();
                TeslaChargingLogger.writeToLog(`CHECK GET NEW DATA: État du véhicule obtenu = ${state}`);
                
                // Transition en mode manuel selon l'état du véhicule
                if (TeslaService.mode === TeslaService.MODE_GET_DATA_NOW) {
                    if (state === "online") {
                        TeslaChargingLogger.writeToLog("CHECK GET NEW DATA: En mode manuel, état 'online' détecté, passage à GET_DATA");
                        TeslaService.step = TeslaService.STEP_GET_DATA;
                        TeslaService.next_action_time = now;
                        TeslaService.stepInProgress = false;
                        setImmediate(TeslaService.checkGetNewData);
                        return;
                    } else if (state === "asleep" || state === "offline") {
                        TeslaChargingLogger.writeToLog("CHECK GET NEW DATA: En mode manuel, état 'asleep/offline' détecté, passage à WAKE_UP");
                        TeslaService.step = TeslaService.STEP_WAKE_UP;
                        TeslaService.next_action_time = now;
                        TeslaService.stepInProgress = false;
                        setImmediate(TeslaService.checkGetNewData);
                        return;
                    }
                }
                break;
                
            case TeslaService.STEP_WAKE_UP:
                TeslaChargingLogger.writeToLog("CHECK GET NEW DATA: Étape WAKE_UP détectée");
                // Vérifier via MQTT si le véhicule est actif (online, charging, driving)
                if (mqttData?.vehicle_state?.state === "online" || 
                    mqttData?.state === "online" ||
                    mqttData?.vehicle_state?.state === "charging" ||
                    mqttData?.state === "charging" ||
                    mqttData?.charge_state?.charging_state === "Charging") {
                    
                    TeslaChargingLogger.writeToLog("📡 DÉTECTION: Véhicule déjà actif selon MQTT, réveil inutile évité");
                    TeslaService.lastKnownVehicleState = "online";
                    TeslaService.lastVehicleStateCheck = now;
                    TeslaService.step = TeslaService.STEP_GET_DATA;
                    TeslaService.next_action_time = now + 2000; // Court délai pour stabilisation
                    break;
                }
                
                // Protection anti-rebond pour les réveils trop fréquents
                if (TeslaService.lastWakeupCall && now - TeslaService.lastWakeupCall < 60000) {
                    const elapsedSeconds = Math.round((now - TeslaService.lastWakeupCall) / 1000);
                    TeslaChargingLogger.writeToLog(`⏱️ ATTENTE: Dernier réveil il y a ${elapsedSeconds}s, délai requis`);
                    
                    if (TeslaService.mode === TeslaService.MODE_GET_DATA_NOW) {
                        if (elapsedSeconds < 30) {
                            TeslaChargingLogger.writeToLog("⏳ RÉVEIL EN COURS: Attente prolongée pour permettre le réveil complet");
                            TeslaService.next_action_time = now + 15000; // Attendre 15 secondes supplémentaires
                        } else {
                            TeslaChargingLogger.writeToLog("⏩ RÉESSAI: Tentative de récupération des données après délai suffisant");
                            TeslaService.step = TeslaService.STEP_GET_DATA;
                            TeslaService.next_action_time = now + 3000;
                        }
                    } else {
                        TeslaService.next_action_time = now + 60000; // 60s entre tentatives en mode normal
                    }
                    break;
                }
                
                TeslaChargingLogger.writeToLog("CHECK GET NEW DATA: Appel à wakeUp()");
                await TeslaService.wakeUp();
                break;
                
            case TeslaService.STEP_GET_DATA:
                TeslaChargingLogger.writeToLog("CHECK GET NEW DATA: Étape GET_DATA détectée");
                // S'assurer que le véhicule n'est pas en train d'être réveillé
                const isActiveWakeup = TeslaService.lastWakeupCall && now - TeslaService.lastWakeupCall < 30000;
                
                // Si l'état connu est "asleep" ou "offline" et qu'une vérification récente a eu lieu, rediriger si nécessaire
                if ((TeslaService.lastKnownVehicleState === "asleep" || 
                     TeslaService.lastKnownVehicleState === "offline") && 
                    now - TeslaService.lastVehicleStateCheck < 30000 &&
                    !isActiveWakeup) {
                    
                    if (TeslaService.mode === TeslaService.MODE_GET_DATA_NOW) {
                        TeslaChargingLogger.writeToLog("🔀 REDIRECTION: Véhicule non disponible, passage à WAKE_UP");
                        TeslaService.step = TeslaService.STEP_WAKE_UP;
                        TeslaService.next_action_time = now;
                        TeslaService.stepInProgress = false;
                        setImmediate(TeslaService.checkGetNewData);
                        return;
                    }
                }
                
                // Vérifier via MQTT avant d'appeler l'API
                if ((TeslaService.lastKnownVehicleState === "asleep" || 
                     TeslaService.lastKnownVehicleState === "offline") &&
                    (mqttData?.vehicle_state?.state === "online" || mqttData?.state === "online")) {
                    
                    TeslaChargingLogger.writeToLog("📡 OPTIMISATION: Véhicule signalé en ligne via MQTT, mise à jour de l'état");
                    TeslaService.lastKnownVehicleState = "online";
                    TeslaService.lastVehicleStateCheck = now;
                }
                
                TeslaChargingLogger.writeToLog("CHECK GET NEW DATA: Appel à getVehicleData()");
                await TeslaService.getVehicleData();
                break;
                
            default:
                TeslaChargingLogger.writeToLog(`❓ ÉTAPE INCONNUE: ${TeslaService.step}`);
                TeslaService.step = TeslaService.STEP_GET_ONLINE;
                TeslaService.next_action_time = now;
        }
    } catch (error) {
        TeslaChargingLogger.writeToLog(`❌ ERREUR EXÉCUTION: ${error.message}`);
        
        // En cas d'erreur, annuler le mode rafraîchissement manuel
        if (TeslaService.mode === TeslaService.MODE_GET_DATA_NOW) {
            TeslaService.mode = TeslaService.MODE_GET_DATA;
            TeslaService.lastManualRefreshTime = null;
            TeslaChargingLogger.writeToLog("🛑 ANNULATION: Mode rafraîchissement manuel annulé suite à une erreur");
        }
        
        // Planifier un réessai après 30 secondes
        TeslaService.next_action_time = now + 30000;
    } finally {
        // Libérer le verrou d'exécution
        TeslaService.stepInProgress = false;
        TeslaChargingLogger.writeToLog("CHECK GET NEW DATA: Fin de l'exécution, verrou relâché");
    }
};


;


TeslaService.getStepName = function(step) {
    // Début de l'exécution de getStepName
    TeslaChargingLogger.writeToLog(`GET STEP NAME: Début de getStepName avec step=${step}`);
    
    let stepName;
    switch (step) {
        case TeslaService.STEP_CONNECT:
            stepName = "CONNECT";
            break;
        case TeslaService.STEP_GET_VEHICLES:
            stepName = "GET_VEHICLES";
            break;
        case TeslaService.STEP_GET_ONLINE:
            stepName = "GET_ONLINE";
            break;
        case TeslaService.STEP_WAKE_UP:
            stepName = "WAKE_UP";
            break;
        case TeslaService.STEP_GET_DATA:
            stepName = "GET_DATA";
            break;
        default:
            stepName = `INCONNU(${step})`;
            break;
    }
    
    // Fin de l'exécution de getStepName
    TeslaChargingLogger.writeToLog(`GET STEP NAME: Fin de getStepName, résultat = ${stepName}`);
    return stepName;
};


TeslaService.getModeName = function(mode) {
    // Début de l'exécution de getModeName
    TeslaChargingLogger.writeToLog(`GET MODE NAME: Début de getModeName avec mode=${mode}`);

    let modeName;
    switch (mode) {
        case TeslaService.MODE_INIT:
            modeName = "INIT";
            break;
        case TeslaService.MODE_GET_DATA:
            modeName = "GET_DATA";
            break;
        case TeslaService.MODE_GET_DATA_NOW:
            modeName = "GET_DATA_NOW";
            break;
        case TeslaService.MODE_WAIT_ONLINE:
            modeName = "WAIT_ONLINE";
            break;
        case TeslaService.MODE_GET_CONTINUS_DATA:
            modeName = "CONTINUOUS";
            break;
        case TeslaService.MODE_REFRESH_TOKEN:
            modeName = "REFRESH_TOKEN";
            break;
        default:
            modeName = `INCONNU(${mode})`;
            break;
    }

    // Fin de l'exécution de getModeName
    TeslaChargingLogger.writeToLog(`GET MODE NAME: Fin de getModeName, résultat = ${modeName}`);
    return modeName;
};


/**
 * Force l'obtention du VIN au démarrage ou en cas d'erreurs
 */
TeslaService.ensureVehicleIdentifiers = function() {
    // Début de l'exécution de ensureVehicleIdentifiers
    TeslaChargingLogger.writeToLog("ENSURE VEHICLE IDENTIFIERS: Début de l'exécution de ensureVehicleIdentifiers");

    // Vérifier la présence du VIN et de l'identifiant du véhicule
    if (!TeslaService.last_receive_data?.vin || !TeslaService.vehicle_id) {
        TeslaChargingLogger.writeToLog("ENSURE VEHICLE IDENTIFIERS: VIN ou identifiant du véhicule manquant, lancement de la séquence de récupération");
        TeslaService.step = TeslaService.STEP_CONNECT;
        TeslaChargingLogger.writeToLog(`ENSURE VEHICLE IDENTIFIERS: Changement d'étape à ${TeslaService.getStepName(TeslaService.step)}`);
        TeslaService.next_action_time = new Date().getTime(); // Exécution immédiate
        TeslaChargingLogger.writeToLog("ENSURE VEHICLE IDENTIFIERS: Next action time mis à jour pour une exécution immédiate");
    } else {
        TeslaChargingLogger.writeToLog("ENSURE VEHICLE IDENTIFIERS: VIN et identifiant du véhicule présents, aucune action requise");
    }

    // Fin de l'exécution de ensureVehicleIdentifiers
    TeslaChargingLogger.writeToLog("ENSURE VEHICLE IDENTIFIERS: Fin de l'exécution de ensureVehicleIdentifiers");
};


TeslaService.init = async function () {
    // Marqueurs de version et logs
    console.log("========== MARQUEUR VERSION MODIFIÉE UNIFIÉE ==========");
    TeslaChargingLogger.writeToLog("🚨🚨🚨 VERSION UNIFIÉE DÉTECTÉE");
    TeslaChargingLogger.writeToLog("INIT: Début de l'initialisation de TeslaService");
    console.log('Init of TeslaService');
    
    try {
        // Nettoyage périodique des logs si disponible
        if (typeof TeslaChargingLogger.schedulePeriodicCleanup === 'function') {
            TeslaChargingLogger.schedulePeriodicCleanup();
        }

        // 1. Initialiser le proxy Tesla si nécessaire
        TeslaChargingLogger.writeToLog("INIT: Vérification du proxy Tesla");
        const TeslaProxy = require('./tesla.proxy');
        if (!TeslaProxy.CONFIG.ready) {
            TeslaChargingLogger.writeToLog("INIT: Proxy Tesla non initialisé, démarrage initialisation");
            TeslaProxy.init();
            TeslaChargingLogger.writeToLog("INIT: Proxy Tesla initialisé");
        } else {
            TeslaChargingLogger.writeToLog("INIT: Proxy Tesla déjà initialisé, statut prêt");
        }

        // 2. Récupération de la configuration et mise à jour du mode de charge
        TeslaChargingLogger.writeToLog("INIT: Chargement de la configuration");
        const config = configuration_service_1.ConfigurationService.getConfiguration();
        TeslaChargingLogger.writeToLog(`INIT CONFIG: Tesla activé=${config.tesla.useTesla}, Mode de charge=${config.tesla.chargingMode}`);
        TeslaChargingLogger.writeToLog(`INIT CONFIG: TeslaMate activé=${config.teslamate?.enabled}, MQTT configuré=${!!config.teslamate?.mqtt}`);
        TeslaService.chargingMode = config.tesla.chargingMode;
        TeslaChargingLogger.writeToLog(`INIT: Mode de charge défini à "${TeslaService.chargingMode}"`);

        // 3. Initialiser last_success_get_data pour éviter une fausse alerte
        TeslaService.last_success_get_data = new Date().getTime();
        TeslaChargingLogger.writeToLog(`INIT: last_success_get_data initialisé à ${TeslaService.last_success_get_data}`);

        // 4. Initialiser les écouteurs MQTT si configuré
        if (config.teslamate?.enabled && config.teslamate?.mqtt) {
            TeslaChargingLogger.writeToLog("INIT: MQTT activé, initialisation du client MQTT");
            TeslaChargingLogger.writeToLog(`INIT MQTT: Paramètres de connexion - Host=${config.teslamate.mqtt.host}, Port=${config.teslamate.mqtt.port}`);
            
            // Chargement et initialisation du client MQTT standard
            const TeslaMqttClient = require('./tesla.mqtt-client');
            TeslaChargingLogger.writeToLog("INIT MQTT: Initialisation du client MQTT");
            TeslaMqttClient.init(config.teslamate.mqtt);
            TeslaChargingLogger.writeToLog("INIT MQTT: Client MQTT initialisé");
            
            // Configuration du listener standard pour les mises à jour MQTT
            TeslaMqttClient.addUpdateListener((carId, attribute, value) => {
                // Log de base pour chaque mise à jour
                TeslaChargingLogger.writeToLog(`📡 MQTT: [${attribute}] ${value}`);
                
                // Mise à jour pour niveau batterie (important pour last_success_get_data)
                if (attribute === 'battery_level') {
                    TeslaService.last_success_get_data = Date.now();
                    TeslaChargingLogger.writeToLog(`📡 MQTT: Mise à jour du niveau batterie à ${value}%`);
                }
                
                // Ne mettre à jour que si nous avons déjà des données (initialisées par une requête API)
                if (!TeslaService.last_receive_data) {
                    TeslaChargingLogger.writeToLog("MQTT UPDATE: Initialisation de l'objet de données car inexistant");
                    TeslaService.last_receive_data = {};
                }
                
                // Mettre à jour la structure de données avec les valeurs MQTT
                if (typeof TeslaService.updateLastReceiveDataFromMqtt === 'function') {
                    TeslaService.updateLastReceiveDataFromMqtt(attribute, value);
                }
                
                // Pour les attributs critiques, déclencher le traitement post-mise à jour
                if (['battery_level', 'charging_state', 'state'].includes(attribute)) {
                    TeslaChargingLogger.writeToLog(`MQTT UPDATE: Attribut critique ${attribute} détecté, déclenchement traitement`);
                    if (typeof TeslaService.syncDataSources === 'function') {
                        TeslaService.syncDataSources();
                    }
                    if (typeof TeslaService.handleVehicleDataReceived === 'function') {
                        TeslaService.handleVehicleDataReceived();
                    }
                }
            });
            TeslaChargingLogger.writeToLog("✅ INIT MQTT: Écouteur d'événements ajouté avec succès");
        } else {
            TeslaChargingLogger.writeToLog("INIT: MQTT non configuré ou désactivé");
        }

        // 4.A Initialisation du client BLE si activé dans la configuration
        if (config.tesla?.useBle) {
            TeslaChargingLogger.writeToLog("INIT BLE: Configuration BLE détectée, initialisation du service");
            
            try {
                const { TeslaControlBLE } = require('./tesla.ble.service');
                if (TeslaControlBLE) {
                    const bleInitialized = TeslaControlBLE.init(config);
                    if (bleInitialized) {
                        TeslaChargingLogger.writeToLog(`✅ INIT BLE: Service BLE initialisé avec succès - VIN: ${TeslaControlBLE.config.vin}, Clé: ${TeslaControlBLE.config.keyFile}`);
                    } else {
                        TeslaChargingLogger.writeToLog("⚠️ INIT BLE: Échec d'initialisation du service BLE");
                    }
                } else {
                    TeslaChargingLogger.writeToLog("⚠️ INIT BLE: Module TeslaControlBLE non disponible");
                }
            } catch (bleError) {
                TeslaChargingLogger.writeToLog(`❌ INIT BLE ERREUR: ${bleError.message}`);
                console.warn("Tesla BLE: Erreur d'initialisation -", bleError.message);
            }
        } else {
            TeslaChargingLogger.writeToLog("INIT: BLE non configuré ou désactivé");
        }
        
        // 5. Démarrer le service de polling qui émet les données via Socket.IO
        if (typeof TeslaService.startPollingWithSocketEmit === 'function') {
            TeslaChargingLogger.writeToLog("INIT SOCKET.IO: Démarrage du service de polling");
            TeslaService.startPollingWithSocketEmit();
            TeslaChargingLogger.writeToLog("✅ INIT SOCKET.IO: Service de polling démarré");
        }
        
        // 6. Si Tesla est activé, lancer la boucle principale avec récupération VIN
        if (config.tesla.useTesla) {
            TeslaChargingLogger.writeToLog("INIT: Tesla activé, vérification des identifiants du véhicule");
            
            // NOUVELLE SECTION: Récupération SYNCHRONE des identifiants du véhicule au démarrage
            TeslaChargingLogger.writeToLog("INIT CYCLE: Initialisation COMPLÈTE des identifiants du véhicule");
            
            try {
                // Vérifier si le VIN est déjà configuré
                if (config.tesla.vin) {
                    TeslaChargingLogger.writeToLog(`INIT VEHICLE: VIN déjà configuré: ${config.tesla.vin.substring(0, 5)}...${config.tesla.vin.substring(config.tesla.vin.length - 5)}`);
                    // Initialiser le cache avec le VIN connu
                    if (!TeslaService.last_receive_data) TeslaService.last_receive_data = {};
                    TeslaService.last_receive_data.vin = config.tesla.vin;
                    TeslaChargingLogger.writeToLog(`INIT VEHICLE: VIN initialisé dans le cache de données`);
                } else {
                    // Si pas de VIN, tenter de récupérer la liste des véhicules immédiatement
                    TeslaChargingLogger.writeToLog("INIT VEHICLE: Aucun VIN configuré, récupération immédiate de la liste");
                    
                    // S'assurer que le token est à jour
                    TeslaChargingLogger.writeToLog("INIT VEHICLE: Connexion à l'API Tesla...");
                    await TeslaService.connect();
                    TeslaChargingLogger.writeToLog("INIT VEHICLE: Connexion établie, récupération de la liste des véhicules...");
                    
                    // Récupérer la liste des véhicules
                    await TeslaService.getVehicleList();
                    
                    // Vérifier que l'identifiant a bien été récupéré
                    if (!TeslaService.vehicle_id && !TeslaService.last_receive_data?.vin) {
                        TeslaChargingLogger.writeToLog("⚠️ INIT VEHICLE: Impossible de récupérer l'identifiant du véhicule au démarrage");
                    } else {
                        TeslaChargingLogger.writeToLog("✅ INIT VEHICLE: Identifiant du véhicule récupéré avec succès");
                    }
                }
            } catch (vehicleError) {
                TeslaChargingLogger.writeToLog(`❌ INIT VEHICLE ERROR: ${vehicleError.message}`);
                console.error("Erreur lors de l'initialisation des identifiants du véhicule:", vehicleError);
            }
            
            // Code original - appel à ensureVehicleIdentifiers qui reste pour compatibilité
            TeslaService.ensureVehicleIdentifiers();
            TeslaChargingLogger.writeToLog("INIT: Démarrage du polling des données avec intervalle de 1 seconde");
            setInterval(TeslaService.checkGetNewData, 1000);
            
            // Ajout du log périodique pour le diagnostic des données
            TeslaChargingLogger.writeToLog("INIT: Démarrage du log périodique des données véhicule (intervalle: 10s)");
            setInterval(TeslaService.logPeriodicStatus, 10000);
            
            // Ajout de l'intervalle de synchronisation des sources
            if (typeof TeslaService.syncDataSources === 'function') {
                TeslaChargingLogger.writeToLog("INIT SYNC: Configuration de la synchronisation périodique API/MQTT");
                setInterval(TeslaService.syncDataSources, 30000);
                TeslaChargingLogger.writeToLog("✅ INIT SYNC: Synchronisation périodique activée (30s)");
            }
        } else {
            TeslaChargingLogger.writeToLog("INIT: Tesla désactivé, aucun polling de données démarré");
        }
        
        // 7. Définir l'étape initiale du cycle
        TeslaService.step = TeslaService.STEP_CONNECT;
        TeslaService.next_action_time = Date.now();
        TeslaChargingLogger.writeToLog(`INIT ÉTAT: Cycle défini à CONNECT, prochain traitement immédiat`);
        
    } catch (error) {
        TeslaChargingLogger.writeToLog(`❌ INIT ÉCHEC: Erreur d'initialisation - ${error.message}`);
        console.error("❌ Erreur d'initialisation du service Tesla:", error);
    }
    
    // Fin de l'initialisation
    TeslaChargingLogger.writeToLog("INIT: Fin de l'initialisation de TeslaService");
};


/**
 * Définit le temps entre chaque réveil forcé
 */
TeslaService.setForcedWakeUp = function (delayMs) {
    // Début de l'exécution de setForcedWakeUp
    TeslaChargingLogger.writeToLog(`SET FORCED WAKEUP: Début de setForcedWakeUp avec delayMs = ${delayMs}`);

    // Définir l'intervalle maximal entre chaque récupération de données
    TeslaService.max_time_between_get_data = delayMs;
    TeslaChargingLogger.writeToLog(`SET FORCED WAKEUP: max_time_between_get_data défini à ${TeslaService.max_time_between_get_data}`);

    // Fin de l'exécution de setForcedWakeUp
    TeslaChargingLogger.writeToLog("SET FORCED WAKEUP: Fin de setForcedWakeUp");
};



// Fonction utilitaire pour aplatir les clés d'un objet
function flattenKeys(obj, prefix = '') {
    TeslaChargingLogger.writeToLog(`FLATTEN KEYS: Début de flattenKeys avec prefix="${prefix}"`);
    
    const keys = Object.keys(obj || {});
    TeslaChargingLogger.writeToLog(`FLATTEN KEYS: Clés trouvées: ${JSON.stringify(keys)}`);
    
    const result = keys.reduce((acc, key) => {
        const newKey = prefix ? `${prefix}.${key}` : key;
        TeslaChargingLogger.writeToLog(`FLATTEN KEYS: Traitement de la clé "${key}" => nouvelle clé: "${newKey}"`);
        
        if (typeof obj[key] === 'object' && obj[key] !== null) {
            TeslaChargingLogger.writeToLog(`FLATTEN KEYS: La valeur associée à "${newKey}" est un objet, appel récursif`);
            acc.push(...flattenKeys(obj[key], newKey));
        } else {
            TeslaChargingLogger.writeToLog(`FLATTEN KEYS: La valeur associée à "${newKey}" n'est pas un objet, ajout direct`);
            acc.push(newKey);
        }
        return acc;
    }, []);
    
    TeslaChargingLogger.writeToLog(`FLATTEN KEYS: Fin de flattenKeys, résultat: ${JSON.stringify(result)}`);
    return result;
}
